@@ -1,4 +1,10 @@
-import six
+from os import path
+
+from pkg_resources import parse_version, require, Requirement, ResolutionError
+
+from pyinfra import __version__, state
+
+from .exceptions import PyinfraError
 
 
 config_defaults = {
@@ -20,11 +26,6 @@ config_defaults = {
     # Note: this can also include pyinfra, potentially replacing REQUIRE_PYINFRA_VERSION
     'REQUIRE_PACKAGES': None,
 
-    # COMPAT w/<1.1
-    # TODO: remove this in favour of above at v2
-    # Specify a minimum required pyinfra version for a deploy
-    'MIN_PYINFRA_VERSION': None,
-
     # All these can be overridden inside individual operation calls:
 
     # Switch to this user (from ssh_user) using su before executing operations
@@ -56,6 +57,52 @@ config_defaults = {
 }
 
 
+def check_pyinfra_version(version):
+    if not version:
+        return
+
+    running_version = parse_version(__version__)
+    required_versions = Requirement.parse(
+        'pyinfra{0}'.format(version),
+    )
+
+    if running_version not in required_versions:
+        raise PyinfraError((
+            'pyinfra version requirement not met '
+            '(requires {0}, running {1})'
+        ).format(
+            version,
+            __version__,
+        ))
+
+
+def check_require_packages(requirements_config):
+    if not requirements_config:
+        return
+
+    if isinstance(requirements_config, (list, tuple)):
+        requirements = requirements_config
+    else:
+        with open(path.join(state.cwd, requirements_config)) as f:
+            requirements = [
+                line.split('#egg=')[-1]
+                for line in f.read().splitlines()
+            ]
+
+    try:
+        require(requirements)
+    except ResolutionError as e:
+        raise PyinfraError('Deploy requirements ({0}) not met: {1}'.format(
+            requirements_config, e,
+        ))
+
+
+config_checkers = {
+    'REQUIRE_PYINFRA_VERSION': check_pyinfra_version,
+    'REQUIRE_PACKAGES': check_require_packages,
+}
+
+
 class Config(object):
     '''
     The default/base configuration options for a pyinfra deploy.
@@ -71,9 +118,16 @@ class Config(object):
         config = config_defaults.copy()
         config.update(kwargs)
 
-        for key, value in six.iteritems(config):
+        for key, value in config.items():
             setattr(self, key, value)
 
+    def __setattr__(self, key, value):
+        super().__setattr__(key, value)
+
+        checker = config_checkers.get(key)
+        if checker:
+            checker(value)
+
     def get_current_state(self):
         return [
             (key, getattr(self, key))
@@ -89,3 +143,6 @@ class Config(object):
 
     def reset_locked_state(self):
         self.set_current_state(self._locked_config)
+
+    def copy(self):
+        return Config(**dict(self.get_current_state()))
