@@ -6,24 +6,22 @@ creation (eg pyinfra-openstack).
 
 from functools import wraps
 
-import six
-
 import pyinfra
 
-from pyinfra import logger, pseudo_host, pseudo_state
+from pyinfra import context, logger
+from pyinfra.context import ctx_host, ctx_state
 
+from .arguments import pop_global_arguments
 from .exceptions import PyinfraError
 from .host import Host
-from .operation_kwargs import pop_global_op_kwargs
-from .state import State
-from .util import get_call_location, get_caller_frameinfo, memoize
+from .util import get_args_kwargs_spec, get_call_location, get_caller_frameinfo, memoize
 
 
 @memoize
 def show_state_host_arguments_warning(call_location):
     logger.warning((
-        '{0}\n\tPassing `state` and `host` as the first two arguments to deploys is '
-        'deprecated, please use `state` and `host` arguments.'
+        '{0}:\n\tLegacy deploy function detected! Deploys should no longer define '
+        '`state` and `host` arguments.'
     ).format(call_location))
 
 
@@ -43,8 +41,6 @@ def add_deploy(state, deploy_func, *args, **kwargs):
             '`add_deploy` should not be called when pyinfra is executing in CLI mode! ({0})'
         ).format(get_call_location()))
 
-    kwargs['state'] = state
-
     hosts = kwargs.pop('host', state.inventory.iter_active_hosts())
     if isinstance(hosts, Host):
         hosts = [hosts]
@@ -52,65 +48,57 @@ def add_deploy(state, deploy_func, *args, **kwargs):
     # Append operations called in this deploy to the current order
     kwargs['_op_order_number'] = len(state.op_meta)
 
-    for host in hosts:
-        kwargs['host'] = host
-        deploy_func(*args, **kwargs)
+    with ctx_state.use(state):
+        for deploy_host in hosts:
+            with ctx_host.use(deploy_host):
+                deploy_func(*args, **kwargs)
 
 
-def deploy(func_or_name, data_defaults=None):
+def deploy(func_or_name, data_defaults=None, _call_location=None):
     '''
     Decorator that takes a deploy function (normally from a pyinfra_* package)
     and wraps any operations called inside with any deploy-wide kwargs/data.
     '''
 
     # If not decorating, return function with config attached
-    if isinstance(func_or_name, six.string_types):
+    if isinstance(func_or_name, str):
         name = func_or_name
 
         def decorator(f):
-            setattr(f, 'deploy_name', name)
-
+            f.deploy_name = name
             if data_defaults:
-                setattr(f, 'deploy_data', data_defaults)
-
-            return deploy(f)
-
+                f.deploy_data = data_defaults
+            return deploy(f, _call_location=get_call_location())
         return decorator
 
     # Actually decorate!
     func = func_or_name
 
+    # Check whether an operation is "legacy" - ie contains state=None, host=None kwargs
+    # TODO: remove this in v3
+    is_legacy = False
+    args, kwargs = get_args_kwargs_spec(func)
+    if all(key in kwargs and kwargs[key] is None for key in ('state', 'host')):
+        show_state_host_arguments_warning(_call_location or get_call_location())
+        is_legacy = True
+    func.is_legacy = is_legacy
+
     @wraps(func)
     def decorated_func(*args, **kwargs):
-        # State & host passed in as kwargs (API, nested op, @deploy op)
-        if 'state' in kwargs and 'host' in kwargs:
-            state = kwargs['state']
-            host = kwargs['host']
+        deploy_kwargs, _ = pop_global_arguments(kwargs)
 
-        # State & host passed in as first two arguments (LEGACY)
-        elif len(args) >= 2 and isinstance(args[0], State) and isinstance(args[1], Host):
-            show_state_host_arguments_warning(get_call_location())
-            state = kwargs['state'] = args[0]
-            host = kwargs['host'] = args[1]
-            args_copy = list(args)
-            args = args_copy[2:]
-
-        # Finally, still no state+host? Use pseudo if we're CLI mode, or fail
-        elif pyinfra.is_cli:
-            state = kwargs['state'] = pseudo_state._module
-            host = kwargs['host'] = pseudo_host._module
-
-            if not state or not host or state.in_deploy:
-                raise PyinfraError((
-                    'Nested deploy called without state/host: {0} ({1})'
-                ).format(func, get_call_location()))
+        # If this is a legacy operation function (ie - state & host arg kwargs), ensure that state
+        # and host are included as kwargs.
+        if func.is_legacy:
+            if 'state' not in kwargs:
+                kwargs['state'] = context.state
+            if 'host' not in kwargs:
+                kwargs['host'] = context.host
 
+        # If not legacy, pop off any state/host kwargs that may come from legacy @deploy functions
         else:
-            raise PyinfraError((
-                'Deploy called without state/host: {0} ({1})'
-            ).format(func, get_call_location()))
-
-        deploy_kwargs, _ = pop_global_op_kwargs(state, host, kwargs)
+            kwargs.pop('state', None)
+            kwargs.pop('host', None)
 
         # Name the deploy
         deploy_name = getattr(func, 'deploy_name', func.__name__)
@@ -124,16 +112,16 @@ def deploy(func_or_name, data_defaults=None):
             if op_order_number is not None:
                 deploy_op_order = [op_order_number]
             # API mode: nested deploy wrapped function call
-            elif state.deploy_op_order:
+            elif context.host.current_deploy_op_order:
                 frameinfo = get_caller_frameinfo()
-                deploy_op_order = state.deploy_op_order + [frameinfo.lineno]
+                deploy_op_order = context.host.current_deploy_op_order + [frameinfo.lineno]
             else:
                 raise PyinfraError((
                     'Operation order number not provided in API mode - '
                     'you must use `add_deploy` to add operations.'
                 ))
 
-        with state.deploy(
+        with context.host.deploy(
             name=deploy_name,
             kwargs=deploy_kwargs,
             data=deploy_data,
