@@ -1,69 +1,74 @@
-from __future__ import unicode_literals
+from contextlib import contextmanager
 
 import click
+from gevent.lock import BoundedSemaphore
 
 from pyinfra import logger
+from pyinfra.connectors.util import remove_any_sudo_askpass_file
 
-from .connectors import EXECUTION_CONNECTORS
-from .connectors.util import remove_any_sudo_askpass_file
-from .exceptions import ConnectError, PyinfraError
-from .facts import (
-    create_host_fact,
-    delete_host_fact,
-    get_fact_class,
-    get_fact_names,
-    get_host_fact,
-    is_fact,
-)
+from .connectors import get_execution_connector
+from .exceptions import ConnectError
+from .facts import create_host_fact, delete_host_fact, get_host_fact
 
 
-# TODO: remove this! COMPAT w/<2
-# This is the old-style `host.fact.<snake_case_name>` "magic" fact attributes
-# which have been replaced by the much cleaner `host.get_fact(fact_cls, *args, **kwargs)`.
-class HostFacts(object):
-    def __init__(self, host=None):
-        self.host = host
+def extract_callable_datas(datas):
+    for data in datas:
+        # Support for dynamic data, ie @deploy wrapped data defaults where
+        # the data is stored on the state temporarily.
+        if callable(data):
+            data = data()
 
-    def __dir__(self):
-        return get_fact_names()
+        yield data
 
-    def _check_host(self):
-        if not self.host:
-            raise TypeError('Cannot call this function with no host!')
+
+class HostData(object):
+    '''
+    Combines multiple AttrData's to search for attributes.
+    '''
+
+    override_datas = None
+
+    def __init__(self, host, *datas):
+        self.__dict__['host'] = host
+
+        datas = list(datas)
+
+        # Inject an empty override data so we can assign during deploy
+        self.__dict__['override_datas'] = {}
+        datas.insert(0, self.override_datas)
+
+        self.__dict__['datas'] = tuple(datas)
 
     def __getattr__(self, key):
-        self._check_host()
+        for data in extract_callable_datas(self.datas):
+            try:
+                return data[key]
+            except KeyError:
+                pass
 
-        if not is_fact(key):
-            raise AttributeError('No such fact: {0}'.format(key))
+        raise AttributeError(f'Host `{self.host}` has no data `{key}`')
 
-        # Ensure this host is connected
-        connection = self.host.connect(reason='for {0} fact'.format(key))
+    def __setattr__(self, key, value):
+        self.override_datas[key] = value
 
-        # If we can't connect - fail immediately as we specifically need this
-        # fact for this host and without it we cannot satisfy the deploy.
-        if not connection:
-            raise PyinfraError('Could not connect to {0} for fact {1}!'.format(
-                self.host, key,
-            ))
+    def __str__(self):
+        return str(self.datas)
 
-        # Expecting a function to return
-        if callable(getattr(get_fact_class(key), 'command', None)):
-            def wrapper(*args, **kwargs):
-                return get_host_fact(self.host.state, self.host, key, args=args, kwargs=kwargs)
-            return wrapper
+    def get(self, key, default=None):
+        return getattr(self, key, default)
 
-        # Expecting the fact as a return value
-        else:
-            return get_host_fact(self.host.state, self.host, key)
+    def dict(self):
+        out = {}
 
-    def _create(self, key, data=None, args=None):
-        self._check_host()
-        return create_host_fact(self.host.state, self.host, key, data, args)
+        # Copy and reverse data objects (such that the first items override
+        # the last, matching __getattr__ output).
+        datas = list(self.datas)
+        datas.reverse()
 
-    def _delete(self, key, args=None):
-        self._check_host()
-        return delete_host_fact(self.host.state, self.host, key, args)
+        for data in extract_callable_datas(datas):
+            out.update(data)
+
+        return out
 
 
 class Host(object):
@@ -74,24 +79,46 @@ class Host(object):
 
     connection = None
     state = None
-    fact = HostFacts()  # this isn't usable, but provides support for dir()
+
+    # Current context inside an @operation function
+    in_op = False
+    current_op_hash = None
+    current_op_global_kwargs = None
+
+    # Current context inside a @deploy function
+    in_deploy = False
+    current_deploy_name = None
+    current_deploy_kwargs = None
+    current_deploy_data = None
+    current_deploy_op_order = None
 
     def __init__(
-        self, name, inventory, groups, data,
-        executor=EXECUTION_CONNECTORS['ssh'],
+        self, name, inventory, groups,
+        executor=get_execution_connector('ssh'),
     ):
         self.inventory = inventory
         self.groups = groups
-        self.data = data
         self.executor = executor
         self.name = name
 
-        # Attach the fact proxy
-        self.fact = HostFacts(self)
-
         # Arbitrary dict for connector use
         self.connector_data = {}
 
+        # Fact data store
+        self.facts = {}
+        self.facts_lock = BoundedSemaphore()
+
+        # Create the (waterfall data: override, host, group, global)
+        self.data = HostData(
+            self,
+            lambda: inventory.get_override_data(),
+            lambda: inventory.get_host_data(name),
+            lambda: inventory.get_groups_data(groups),
+            lambda: inventory.get_data(),
+            # @deploy function data are default values, so come last
+            self.get_deploy_data,
+        )
+
     def __str__(self):
         return '{0}'.format(self.name)
 
@@ -123,6 +150,12 @@ class Host(object):
             click.style(self.name, *args, **kwargs),
         )
 
+    def get_deploy_data(self):
+        if self.current_deploy_data:
+            return self.current_deploy_data
+
+        return {}
+
     def noop(self, description):
         '''
         Log a description for a noop operation.
@@ -131,6 +164,47 @@ class Host(object):
         handler = logger.info if self.state.print_noop_info else logger.debug
         handler('{0}noop: {1}'.format(self.print_prefix, description))
 
+    @contextmanager
+    def deploy(self, name, kwargs, data, in_deploy=True, deploy_op_order=None):
+        '''
+        Wraps a group of operations as a deploy, this should not be used
+        directly, instead use ``pyinfra.api.deploy.deploy``.
+        '''
+
+        # Handle nested deploy names
+        if self.current_deploy_name:
+            name = '{0} | {1}'.format(self.current_deploy_name, name)
+
+        # Store the previous values
+        old_in_deploy = self.in_deploy
+        old_deploy_name = self.current_deploy_name
+        old_deploy_kwargs = self.current_deploy_kwargs
+        old_deploy_data = self.current_deploy_data
+        old_deploy_op_order = self.current_deploy_op_order
+        self.in_deploy = in_deploy
+
+        # Set the new values
+        self.current_deploy_name = name
+        self.current_deploy_kwargs = kwargs
+        self.current_deploy_data = data
+        self.current_deploy_op_order = deploy_op_order
+        logger.debug('Starting deploy {0} (args={1}, data={2})'.format(
+            name, kwargs, data,
+        ))
+
+        yield
+
+        # Restore the previous values
+        self.in_deploy = old_in_deploy
+        self.current_deploy_name = old_deploy_name
+        self.current_deploy_kwargs = old_deploy_kwargs
+        self.current_deploy_data = old_deploy_data
+        self.current_deploy_op_order = old_deploy_op_order
+
+        logger.debug('Reset deploy to {0} (args={1}, data={2})'.format(
+            old_deploy_name, old_deploy_kwargs, old_deploy_data,
+        ))
+
     # Host facts
     #
 
@@ -150,7 +224,7 @@ class Host(object):
         if not self.state:
             raise TypeError('Cannot call this function with no state!')
 
-    def connect(self, reason=None, show_errors=True):
+    def connect(self, reason=None, show_errors=True, raise_exceptions=False):
         self._check_state()
         if not self.connection:
             self.state.trigger_callbacks('host_before_connect', self)
@@ -166,6 +240,9 @@ class Host(object):
                     logger.error(log_message)
 
                 self.state.trigger_callbacks('host_connect_error', self, e)
+
+                if raise_exceptions:
+                    raise
             else:
                 log_message = '{0}{1}'.format(
                     self.print_prefix,
