@@ -1,13 +1,8 @@
 from collections import defaultdict
 
-import six
-
-from pyinfra import logger
-
-from .connectors import ALL_CONNECTORS, EXECUTION_CONNECTORS
+from .connectors import get_all_connectors, get_execution_connectors
 from .exceptions import NoConnectorError, NoGroupError, NoHostError
 from .host import Host
-from .util import FallbackDict
 
 
 def extract_name_data(names):
@@ -39,41 +34,13 @@ class Inventory(object):
     def __init__(
         self, names_data,
         override_data=None,
-        # The below are deprecated, override_data.X is now preferred
-        # TODO: remove these in v2
-        ssh_user=None, ssh_port=None, ssh_key=None,
-        ssh_key_password=None, ssh_password=None,
-        winrm_username=None, winrm_password=None,
-        winrm_port=None, winrm_transport=None,
         **groups
     ):
         # Setup basics
         self.groups = defaultdict(list)  # lists of Host objects
         self.host_data = defaultdict(dict)  # dict of name -> data
         self.group_data = defaultdict(dict)  # dict of name -> data
-
-        override_data = override_data or {}
-
-        # TODO: remove these in v2
-        for key, value in (
-            ('ssh_user', ssh_user),
-            ('ssh_key', ssh_key),
-            ('ssh_key_password', ssh_key_password),
-            ('ssh_port', ssh_port),
-            ('ssh_password', ssh_password),
-            ('winrm_username', winrm_username),
-            ('winrm_password', winrm_password),
-            ('winrm_port', winrm_port),
-            ('winrm_transport', winrm_transport),
-        ):
-            if value is not None:
-                logger.warning((
-                    'Use of `{0}` is deprecated when creating `Inventory` classes, '
-                    'please use `override_data.{0}`'
-                ).format(key))
-                override_data[key] = value
-
-        self.override_data = override_data
+        self.override_data = override_data or {}
 
         names, data = names_data
 
@@ -84,12 +51,15 @@ class Inventory(object):
         self.hosts = self.make_hosts_and_groups(names, groups)
 
     def make_hosts_and_groups(self, names, groups):
+        all_connectors = get_all_connectors()
+        execution_connectors = get_execution_connectors()
+
         # Map name -> data
         name_to_data = defaultdict(dict)
         # Map name -> group names
         name_to_group_names = defaultdict(list)
 
-        for group_name, (group_names, group_data) in six.iteritems(groups):
+        for group_name, (group_names, group_data) in groups.items():
             # Assign group data
             self.group_data[group_name] = group_data
 
@@ -110,7 +80,7 @@ class Inventory(object):
             host_data = name_to_data[name]
 
             # Default to executing commands with the ssh connector
-            executor = EXECUTION_CONNECTORS['ssh']
+            executor = execution_connectors['ssh']
 
             if name[0] == '@':
                 connector_name = name[1:]
@@ -119,36 +89,35 @@ class Inventory(object):
                 if '/' in connector_name:
                     connector_name, arg_string = connector_name.split('/', 1)
 
-                if connector_name not in ALL_CONNECTORS:
+                if connector_name not in get_all_connectors():
                     raise NoConnectorError(
                         'Invalid connector: {0}'.format(connector_name),
                     )
 
                 # Execution connector? Simple, just set it for their host
-                if connector_name in EXECUTION_CONNECTORS:
-                    executor = EXECUTION_CONNECTORS[connector_name]
+                if connector_name in execution_connectors:
+                    executor = execution_connectors[connector_name]
 
-                names_data = ALL_CONNECTORS[connector_name].make_names_data(arg_string)
+                names_data = all_connectors[connector_name].make_names_data(arg_string)
                 connector_inventory_name = name
             else:
                 names_data = [(name, {}, [])]
                 connector_inventory_name = None
 
-            for name, data, group_names in names_data:
-                # Make a copy of the host data, update with any from
-                # the connector.
-                sub_host_data = host_data.copy()
-                sub_host_data.update(data)
+            for sub_name, sub_data, sub_groups in names_data:
+                # Update any connector data with a copy of the host data (so that
+                # host data can override connector data).
+                sub_data.update(host_data.copy())
 
-                # Assign the name/data/group_names from the connector
-                self.host_data[name] = sub_host_data
-                names_executors.append((name, executor))
-                name_to_group_names[name].extend(group_names)
+                # Assign the name/data/groups from the connector
+                self.host_data[sub_name] = sub_data
+                names_executors.append((sub_name, executor))
+                name_to_group_names[sub_name].extend(sub_groups)
 
                 # If we have a connector inventory name, copy any groups attached
                 # to the newly generated host name.
                 if connector_inventory_name:
-                    name_to_group_names[name].extend(
+                    name_to_group_names[sub_name].extend(
                         name_to_group_names[connector_inventory_name],
                     )
 
@@ -158,25 +127,7 @@ class Inventory(object):
         for name, executor in names_executors:
             host_groups = name_to_group_names[name]
 
-            # Create the (waterfall data: override, host, group, global)
-            host_data = FallbackDict(
-                self.get_override_data(),
-                self.get_host_data(name),
-                self.get_groups_data(host_groups),
-                self.get_data(),
-                # Pass the method, rather than data, as this comes from the
-                # state and can change during deploy(s).
-                self.get_deploy_data,
-            )
-
-            # Create the Host object
-            host = Host(
-                name,
-                inventory=self,
-                groups=name_to_group_names.get(name),
-                data=host_data,
-                executor=executor,
-            )
+            host = Host(name, inventory=self, groups=host_groups, executor=executor)
             hosts[name] = host
 
             # And push into any groups
@@ -198,15 +149,7 @@ class Inventory(object):
         Iterates over all inventory hosts.
         '''
 
-        return six.itervalues(self.hosts)
-
-    def len_all_hosts(self):
-        logger.warning('`Inventory.len_all_hosts` is deprecated, please use `len(Inventory)`.')
-        return len(self)
-
-    def iter_all_hosts(self):
-        logger.warning('`Inventory.iter_all_hosts` is deprecated, please use `iter(Inventory)`.')
-        return iter(self)
+        return iter(self.hosts.values())
 
     def iter_active_hosts(self):
         '''
