@@ -5,43 +5,29 @@ to the deploy state. This is then run later by pyinfra's ``__main__`` or the
 :doc:`./pyinfra.api.operations` module.
 '''
 
-from __future__ import unicode_literals
-
 from functools import wraps
 from types import FunctionType
 
-import six
-
 import pyinfra
 
-from pyinfra import logger, pseudo_host, pseudo_state
+from pyinfra import context, logger
+from pyinfra.context import ctx_host, ctx_state
 
+from .arguments import get_execution_kwarg_keys, pop_global_arguments
 from .command import StringCommand
 from .exceptions import OperationValueError, PyinfraError
 from .host import Host
-from .operation_kwargs import get_execution_kwarg_keys, pop_global_op_kwargs
-from .state import State
 from .util import (
-    get_arg_value,
+    get_args_kwargs_spec,
     get_call_location,
     get_caller_frameinfo,
     get_operation_order_from_stack,
     make_hash,
     memoize,
-    unroll_generators,
 )
 
 
-# List of available operation names
-OPERATIONS = []
-
-
-def get_operation_names():
-    '''
-    Returns a list of available operations.
-    '''
-
-    return OPERATIONS
+op_meta_default = object()
 
 
 class OperationMeta(object):
@@ -83,38 +69,35 @@ def add_op(state, op_func, *args, **kwargs):
     # appended to the operation order.
     kwargs['_op_order_number'] = len(state.op_meta)
 
-    kwargs['state'] = state
-
     hosts = kwargs.pop('host', state.inventory.iter_active_hosts())
     if isinstance(hosts, Host):
         hosts = [hosts]
 
-    results = {}
-    for host in hosts:
-        kwargs['host'] = host
-        results[host] = op_func(*args, **kwargs)
-        after_host_callback(host)
+    with ctx_state.use(state):
+        results = {}
+        for op_host in hosts:
+            with ctx_host.use(op_host):
+                results[op_host] = op_func(*args, **kwargs)
+            after_host_callback(op_host)
 
     return results
 
 
-@memoize
-def show_set_name_warning(call_location):
-    logger.warning((
-        '{0}:\n\tUse of a `set` to name operations is deprecated, '
-        'please us the `name` keyword argument.'
-    ).format(call_location))
-
-
 @memoize
 def show_state_host_arguments_warning(call_location):
     logger.warning((
-        '{0}:\n\tPassing `state` and `host` as the first two arguments to operations is '
-        'deprecated, please use `state` and `host` keyword arguments.'
+        '{0}:\n\tLegacy operation function detected! Operations should no longer define '
+        '`state` and `host` arguments.'
     ).format(call_location))
 
 
-def operation(func=None, pipeline_facts=None, is_idempotent=True):
+def operation(
+    func=None,
+    pipeline_facts=None,
+    is_idempotent=True,
+    idempotent_notice=None,
+    frame_offset=1,
+):
     '''
     Decorator that takes a simple module function and turn it into the internal
     operation representation that consists of a list of commands + options
@@ -124,73 +107,60 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
     # If not decorating, return function with config attached
     if func is None:
         def decorator(f):
-            setattr(f, 'pipeline_facts', pipeline_facts)
-            setattr(f, 'is_idempotent', is_idempotent)
-            return operation(f)
+            f.pipeline_facts = pipeline_facts
+            f.is_idempotent = is_idempotent
+            f.idempotent_notice = idempotent_notice
+            return operation(f, frame_offset=2)
         return decorator
 
-    # Index the operation!
-    # TODO: remove this in v2
-    if func.__module__:
-        module_bits = func.__module__.split('.')
-        module_name = module_bits[-1]
-        op_name = '.'.join((module_name, func.__name__))
-        OPERATIONS.append(op_name)
+    # Check whether an operation is "legacy" - ie contains state=None, host=None kwargs
+    # TODO: remove this in v3
+    is_legacy = False
+    args, kwargs = get_args_kwargs_spec(func)
+    if all(key in kwargs and kwargs[key] is None for key in ('state', 'host')):
+        show_state_host_arguments_warning(get_call_location(frame_offset=frame_offset))
+        is_legacy = True
+    func.is_legacy = is_legacy
 
     # Actually decorate!
     @wraps(func)
     def decorated_func(*args, **kwargs):
-        # Prepare state/host
-        #
+        state = context.state
+        host = context.host
 
-        # State & host passed in as kwargs (API, nested op, @deploy op)
-        if 'state' in kwargs and 'host' in kwargs:
-            state = kwargs['state']
-            host = kwargs['host']
-
-        # State & host passed in as first two arguments (LEGACY)
-        elif len(args) >= 2 and isinstance(args[0], State) and isinstance(args[1], Host):
-            show_state_host_arguments_warning(get_call_location())
-            state = kwargs['state'] = args[0]
-            host = kwargs['host'] = args[1]
-            args_copy = list(args)
-            args = args_copy[2:]
-
-        # Finally, still no state+host? Use pseudo if we're CLI mode, or fail
-        elif pyinfra.is_cli:
-            state = kwargs['state'] = pseudo_state._module
-            host = kwargs['host'] = pseudo_host._module
-
-            if state.in_op:
-                raise PyinfraError((
-                    'Nested operation called without state/host: {0} ({1})'
-                ).format(op_name, get_call_location()))
-
-            if state.in_deploy:
-                raise PyinfraError((
-                    'Nested deploy operation called without state/host: {0} ({1})'
-                ).format(op_name, get_call_location()))
-
-        else:
+        if state.is_executing:
             raise PyinfraError((
-                'API operation called without state/host: {0} ({1})'
-            ).format(op_name, get_call_location()))
+                'Operations cannot be used during execution, see '
+                '`host.run_shell_command` as an alternative'
+            ))
 
         # Configure operation
         #
 
         # Get the meta kwargs (globals that apply to all hosts)
-        global_kwargs, global_kwarg_keys = pop_global_op_kwargs(state, host, kwargs)
+        global_kwargs, global_kwarg_keys = pop_global_arguments(kwargs)
 
         # If this op is being called inside another, just return here
         # (any unwanted/op-related kwargs removed above).
-        if state.in_op:
+        if host.in_op:
             if global_kwarg_keys:
                 raise PyinfraError((
                     'Nested operation called with global arguments: {0} ({1})'
                 ).format(global_kwarg_keys, get_call_location()))
             return func(*args, **kwargs) or []
 
+        # If this is a legacy operation function (ie - state & host arg kwargs), ensure that state
+        # and host are included as kwargs.
+        if func.is_legacy:
+            if 'state' not in kwargs:
+                kwargs['state'] = state
+            if 'host' not in kwargs:
+                kwargs['host'] = host
+        # If not legacy, pop off any state/host kwargs that may come from legacy @deploy functions
+        else:
+            kwargs.pop('state', None)
+            kwargs.pop('host', None)
+
         # Name the operation
         name = global_kwargs.get('name')
         add_args = False
@@ -198,14 +168,6 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
         if name:
             names = {name}
 
-        # Look for a set as the first argument
-        # TODO: remove this! COMPAT w/<1
-        elif len(args) > 0 and isinstance(args[0], set):
-            show_set_name_warning(get_call_location())
-            names = args[0]
-            args_copy = list(args)
-            args = args[1:]
-
         # Generate an operation name if needed (Module/Operation format)
         else:
             add_args = True
@@ -219,9 +181,9 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
 
             names = {name}
 
-        if state.deploy_name:
+        if host.current_deploy_name:
             names = {
-                '{0} | {1}'.format(state.deploy_name, name)
+                '{0} | {1}'.format(host.current_deploy_name, name)
                 for name in names
             }
 
@@ -244,9 +206,9 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
             # deploy function is called first, and then the operation position *within*
             # that function. Because functions have to exist in one file, we can simply
             # use the line number to get correct ordering.
-            if state.in_deploy:
+            if host.in_deploy:
                 frameinfo = get_caller_frameinfo()
-                op_order = state.deploy_op_order + [frameinfo.lineno]
+                op_order = host.current_deploy_op_order + [frameinfo.lineno]
             else:
                 raise PyinfraError((
                     'Operation order number not provided in API mode - '
@@ -287,20 +249,13 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
 
         for key in get_execution_kwarg_keys():
             global_value = global_kwargs.pop(key)
-            op_meta_value = op_meta.get(key)
+            op_meta_value = op_meta.get(key, op_meta_default)
 
-            if op_meta_value and global_value != op_meta_value:
+            if op_meta_value is not op_meta_default and global_value != op_meta_value:
                 raise OperationValueError('Cannot have different values for `{0}`.'.format(key))
 
             op_meta[key] = global_value
 
-        # Add any meta kwargs (sudo, etc) to the meta - first parse any strings
-        # as jinja templates.
-        actual_global_kwargs = {
-            key: get_arg_value(state, host, a)
-            for key, a in six.iteritems(global_kwargs)
-        }
-
         # Add any new names to the set
         op_meta['names'].update(names)
 
@@ -314,13 +269,10 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
                     op_meta['args'].append(arg)
 
             # Attach keyword args
-            for key, value in six.iteritems(kwargs):
+            for key, value in kwargs.items():
                 if isinstance(value, FunctionType):
                     value = value.__name__
 
-                if key in ('state', 'host'):
-                    continue
-
                 arg = '='.join((str(key), str(value)))
                 if arg not in op_meta['args']:
                     op_meta['args'].append(arg)
@@ -331,7 +283,7 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
         # Run once and we've already added meta for this op? Stop here.
         if op_meta['run_once']:
             has_run = False
-            for ops in six.itervalues(state.ops):
+            for ops in state.ops.values():
                 if op_hash in ops:
                     has_run = True
                     break
@@ -343,35 +295,21 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
         #
 
         # Otherwise, flag as in-op and run it to get the commands
-        state.in_op = True
-        state.current_op_hash = op_hash
-        state.current_op_global_kwargs = actual_global_kwargs
-
-        # Generate actual arguments by parsing strings as jinja2 templates. This
-        # means you can string format arguments w/o generating multiple
-        # operations. Only affects top level operations, as must be run "in_op"
-        # so facts are gathered correctly.
-        actual_args = [
-            get_arg_value(state, host, a)
-            for a in args
-        ]
-
-        actual_kwargs = {
-            key: get_arg_value(state, host, a)
-            for key, a in six.iteritems(kwargs)
-        }
+        host.in_op = True
+        host.current_op_hash = op_hash
+        host.current_op_global_kwargs = global_kwargs
 
         # Convert to list as the result may be a generator
-        commands = unroll_generators(func(*actual_args, **actual_kwargs))
+        commands = func(*args, **kwargs)
         commands = [  # convert any strings -> StringCommand's
             StringCommand(command.strip())
-            if isinstance(command, six.string_types) else command
+            if isinstance(command, str) else command
             for command in commands
         ]
 
-        state.in_op = False
-        state.current_op_hash = None
-        state.current_op_global_kwargs = None
+        host.in_op = False
+        host.current_op_hash = None
+        host.current_op_global_kwargs = None
 
         # Add host-specific operation data to state
         #
@@ -383,7 +321,7 @@ def operation(func=None, pipeline_facts=None, is_idempotent=True):
         # Add the server-relevant commands
         state.ops[host][op_hash] = {
             'commands': commands,
-            'global_kwargs': actual_global_kwargs,
+            'global_kwargs': global_kwargs,
         }
 
         # Return result meta for use in deploy scripts
