@@ -1,16 +1,10 @@
-from __future__ import division, unicode_literals
-
 from contextlib import contextmanager
 from multiprocessing import cpu_count
-from os import path
 from uuid import uuid4
 
-import six
-
 from gevent.pool import Pool
-from pkg_resources import parse_version, require, Requirement, ResolutionError
 
-from pyinfra import __version__, logger
+from pyinfra import logger
 
 from .config import Config
 from .exceptions import PyinfraError
@@ -90,27 +84,13 @@ class State(object):
     # Main gevent pool
     pool = None
 
-    # Whether we are in an @operation (so inner ops aren't wrapped)
-    in_op = False
-    # Whether we are deploying (ie hosts are all ready)
-    deploying = False
-
-    # Current op hash
-    current_op_hash = None
-    # Current op global args for use w/facts
-    current_op_global_kwargs = None
+    # Whether we are executing operations (ie hosts are all ready)
+    is_executing = False
 
     loop_counter = None
     loop_line = None
     loop_filename = None
 
-    # Name of the current deploy
-    in_deploy = False
-    deploy_name = None
-    deploy_kwargs = None
-    deploy_data = None
-    deploy_op_order = None
-
     print_noop_info = False  # print "[host] noop: reason for noop"
     print_fact_info = False  # print "loaded fact X"
     print_input = False
@@ -119,9 +99,10 @@ class State(object):
     print_fact_output = False
 
     # Used in CLI
-    deploy_dir = None  # base directory for locating files/templates/etc
+    cwd = None  # base directory for locating files/templates/etc
     current_deploy_filename = None
     current_exec_filename = None
+    current_op_file_number = 0
 
     def __init__(self, inventory=None, config=None, **kwargs):
         if inventory:
@@ -135,52 +116,6 @@ class State(object):
         if config is None:
             config = Config()
 
-        # Error if our min version is not met
-        if config.MIN_PYINFRA_VERSION is not None:
-            # TODO: remove this
-            if config.REQUIRE_PYINFRA_VERSION is None:
-                config.REQUIRE_PYINFRA_VERSION = '>={0}'.format(config.MIN_PYINFRA_VERSION)
-                logger.warning(
-                    '`MIN_PYINFRA_VERSION` is deprecated, please use `REQUIRE_PYINFRA_VERSION`.',
-                )
-            else:
-                logger.warning(
-                    'Ignoring legacy `MIN_PYINFRA_VERSION` because '
-                    '`REQUIRE_PYINFRA_VERSION` also exists.',
-                )
-
-        if config.REQUIRE_PYINFRA_VERSION is not None:
-            running_version = parse_version(__version__)
-            required_versions = Requirement.parse(
-                'pyinfra{0}'.format(config.REQUIRE_PYINFRA_VERSION),
-            )
-
-            if running_version not in required_versions:
-                raise PyinfraError((
-                    'pyinfra version requirement not met '
-                    '(requires {0}, running {1})'
-                ).format(
-                    config.REQUIRE_PYINFRA_VERSION,
-                    __version__,
-                ))
-
-        if config.REQUIRE_PACKAGES is not None:
-            if isinstance(config.REQUIRE_PACKAGES, (list, tuple)):
-                requirements = config.REQUIRE_PACKAGES
-            else:
-                with open(path.join(self.deploy_dir, config.REQUIRE_PACKAGES)) as f:
-                    requirements = [
-                        line.split('#egg=')[-1]
-                        for line in f.read().splitlines()
-                    ]
-
-            try:
-                require(requirements)
-            except ResolutionError as e:
-                raise PyinfraError('Deploy requirements ({0}) not met: {1}'.format(
-                    config.REQUIRE_PACKAGES, e,
-                ))
-
         if not config.PARALLEL:
             # TODO: benchmark this
             # In my own tests the optimum number of parallel SSH processes is
@@ -297,47 +232,6 @@ class State(object):
             func = getattr(handler, method_name)
             func(self, *args, **kwargs)
 
-    @contextmanager
-    def deploy(self, name, kwargs, data, in_deploy=True, deploy_op_order=None):
-        '''
-        Wraps a group of operations as a deploy, this should not be used
-        directly, instead use ``pyinfra.api.deploy.deploy``.
-        '''
-
-        # Handle nested deploy names
-        if self.deploy_name:
-            name = '{0} | {1}'.format(self.deploy_name, name)
-
-        # Store the previous values
-        old_in_deploy = self.in_deploy
-        old_deploy_name = self.deploy_name
-        old_deploy_kwargs = self.deploy_kwargs
-        old_deploy_data = self.deploy_data
-        old_deploy_op_order = self.deploy_op_order
-        self.in_deploy = in_deploy
-
-        # Set the new values
-        self.deploy_name = name
-        self.deploy_kwargs = kwargs
-        self.deploy_data = data
-        self.deploy_op_order = deploy_op_order
-        logger.debug('Starting deploy {0} (args={1}, data={2})'.format(
-            name, kwargs, data,
-        ))
-
-        yield
-
-        # Restore the previous values
-        self.in_deploy = old_in_deploy
-        self.deploy_name = old_deploy_name
-        self.deploy_kwargs = old_deploy_kwargs
-        self.deploy_data = old_deploy_data
-        self.deploy_op_order = old_deploy_op_order
-
-        logger.debug('Reset deploy to {0} (args={1}, data={2})'.format(
-            old_deploy_name, old_deploy_kwargs, old_deploy_data,
-        ))
-
     @contextmanager
     def preserve_loop_order(self, items):
         frameinfo = get_caller_frameinfo(frame_offset=1)  # escape contextlib
@@ -435,7 +329,7 @@ class State(object):
         '''
 
         if not hash_key:
-            hash_key = six.text_type(uuid4())
+            hash_key = str(uuid4())
 
         if hash_filename:
             hash_key = sha1_hash(hash_key)
