@@ -1,33 +1,24 @@
-from __future__ import division, print_function, unicode_literals
-
-import re
-
 from functools import wraps
 from hashlib import sha1
-from inspect import getframeinfo, stack
+from inspect import getframeinfo, getfullargspec, stack
 from os import getcwd, path, stat
 from socket import (
     error as socket_error,
     timeout as timeout_error,
 )
-from types import GeneratorType
 
 import click
-import six
 
 from jinja2 import (
     Environment,
     FileSystemLoader,
     StrictUndefined,
-    TemplateSyntaxError,
-    UndefinedError,
 )
 from paramiko import SSHException
 
+import pyinfra
 from pyinfra import logger
 
-from .exceptions import PyinfraError
-
 # 64kb chunks
 BLOCKSIZE = 65536
 
@@ -38,6 +29,38 @@ FILE_SHAS = {}
 PYINFRA_API_DIR = path.dirname(__file__)
 
 
+def get_file_path(state, filename):
+    if path.isabs(filename):
+        return filename
+
+    relative_to = state.cwd
+
+    if state.current_exec_filename and (filename.startswith('./') or filename.startswith('.\\')):
+        relative_to = path.dirname(state.current_exec_filename)
+
+    return path.join(relative_to, filename)
+
+
+def get_args_kwargs_spec(func):
+    args = []
+    kwargs = {}
+
+    argspec = getfullargspec(func)
+    if not argspec.args:
+        return args, kwargs
+
+    if argspec.defaults:
+        kwargs = dict(zip(
+            argspec.args[-len(argspec.defaults):],
+            argspec.defaults,
+        ))
+        args = argspec.args[:-len(argspec.defaults)]
+    else:
+        args = argspec.args
+
+    return args, kwargs
+
+
 def get_kwargs_str(kwargs):
     if not kwargs:
         return ''
@@ -72,12 +95,18 @@ def memoize(func):
     return wrapper
 
 
-def get_call_location():
-    frame = get_caller_frameinfo(frame_offset=1)  # escape *this* function
-    return 'line {0} in {1}'.format(
-        frame.lineno,
-        path.relpath(frame.filename),
-    )
+def get_call_location(frame_offset=1):
+    frame = get_caller_frameinfo(frame_offset=frame_offset)  # escape *this* function
+    relpath = frame.filename
+
+    try:
+        # On Windows if pyinfra is on a different drive to the filename here, this will
+        # error as there's no way to do relative paths between drives.
+        relpath = path.relpath(frame.filename)
+    except ValueError:
+        pass
+
+    return 'line {0} in {1}'.format(frame.lineno, relpath)
 
 
 def get_caller_frameinfo(frame_offset=0):
@@ -110,6 +139,10 @@ def get_operation_order_from_stack(state):
 
     # Now generate a list of line numbers *following that file*
     line_numbers = []
+
+    if pyinfra.is_cli:
+        line_numbers.append(state.current_op_file_number)
+
     for stack_item in stack_items[i:]:
         frame = getframeinfo(stack_item[0])
 
@@ -126,128 +159,31 @@ def get_operation_order_from_stack(state):
     return line_numbers
 
 
-def extract_callable_datas(datas):
-    for data in datas:
-        # Support for dynamic data, ie @deploy wrapped data defaults where
-        # the data is stored on the state temporarily.
-        if callable(data):
-            data = data()
-
-        yield data
-
-
-class FallbackDict(object):
-    '''
-    Combines multiple AttrData's to search for attributes.
-    '''
-
-    override_datas = None
-
-    def __init__(self, *datas):
-        datas = list(datas)
-
-        # Inject an empty override data so we can assign during deploy
-        self.__dict__['override_datas'] = {}
-        datas.insert(0, self.override_datas)
-
-        self.__dict__['datas'] = tuple(datas)
-
-    def __getattr__(self, key):
-        value = self.get(key)
-        if value is None:
-            logger.warning((
-                'Non-existent host data will raise an `AttributeError` '
-                'in pyinfra v2, please use `host.data.get("{0}")` ({1})'.format(
-                    key, get_call_location(),
-                )
-            ))
-        return value
-
-    def __setattr__(self, key, value):
-        self.override_datas[key] = value
-
-    def __str__(self):
-        return six.text_type(self.datas)
-
-    def get(self, key, default=None):
-        for data in extract_callable_datas(self.datas):
-            if key in data:
-                return data[key]
-        return default
-
-    def dict(self):
-        out = {}
-
-        # Copy and reverse data objects (such that the first items override
-        # the last, matching __getattr__ output).
-        datas = list(self.datas)
-        datas.reverse()
-
-        for data in extract_callable_datas(datas):
-            out.update(data)
-
-        return out
-
-
-def unroll_generators(generator):
-    '''
-    Take a generator and unroll any sub-generators recursively. This is
-    essentially a Python 2 way of doing `yield from` in Python 3 (given
-    iterating the entire thing).
-    '''
-
-    # Ensure we have a generator (prevents ccommands returning lists)
-    if not isinstance(generator, GeneratorType):
-        raise TypeError('{0} is not a generator'.format(generator))
-
-    items = []
-
-    for item in generator:
-        if isinstance(item, GeneratorType):
-            items.extend(unroll_generators(item))
-        else:
-            items.append(item)
-
-    return items
-
-
-def get_template(filename_or_string, is_string=False):
+def get_template(filename_or_io):
     '''
     Gets a jinja2 ``Template`` object for the input filename or string, with caching
     based on the filename of the template, or the SHA1 of the input string.
     '''
 
-    # Cache against string sha or just the filename
-    cache_key = sha1_hash(filename_or_string) if is_string else filename_or_string
+    file_data = get_file_io(filename_or_io, mode='r')
+    cache_key = file_data.cache_key
 
-    if cache_key in TEMPLATES:
+    if cache_key and cache_key in TEMPLATES:
         return TEMPLATES[cache_key]
 
-    if is_string:
-        # Set the input string as our template
-        template_string = filename_or_string
+    with file_data as file_io:
+        template_string = file_io.read()
 
-    else:
-        # Load template data into memory
-        with open(filename_or_string, 'r') as file_io:
-            template_string = file_io.read()
-
-    TEMPLATES[cache_key] = Environment(
+    template = Environment(
         undefined=StrictUndefined,
         keep_trailing_newline=True,
         loader=FileSystemLoader(getcwd()),
     ).from_string(template_string)
 
-    return TEMPLATES[cache_key]
+    if cache_key:
+        TEMPLATES[cache_key] = template
 
-
-def underscore(name):
-    '''
-    Transform CamelCase -> snake_case.
-    '''
-
-    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
-    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
+    return template
 
 
 def sha1_hash(string):
@@ -325,50 +261,6 @@ def log_host_command_error(host, e, timeout=0):
         raise e
 
 
-@memoize
-def show_get_arg_value_warning():
-    logger.warning((
-        'Use of jinja2 templating in arguments is deprecated, '
-        'please use builtin Python string formatting.'
-    ))
-
-
-# TODO: remove this! COMPAT w/<1
-def get_arg_value(state, host, arg):
-    '''
-    This functions is **deprecated**.
-    '''
-
-    if isinstance(arg, six.string_types):
-        data = {
-            'host': host,
-            'inventory': state.inventory,
-        }
-
-        try:
-            rendered_arg = get_template(arg, is_string=True).render(data)
-        except (TemplateSyntaxError, UndefinedError) as e:
-            raise PyinfraError('Error in template string: {0}'.format(e))
-        else:
-            if rendered_arg != arg:
-                show_get_arg_value_warning()
-            return rendered_arg
-
-    elif isinstance(arg, list):
-        return [get_arg_value(state, host, value) for value in arg]
-
-    elif isinstance(arg, tuple):
-        return tuple(get_arg_value(state, host, value) for value in arg)
-
-    elif isinstance(arg, dict):
-        return {
-            key: get_arg_value(state, host, value)
-            for key, value in six.iteritems(arg)
-        }
-
-    return arg
-
-
 def make_hash(obj):
     '''
     Make a hash from an arbitrary nested dictionary, list, tuple or set, used to generate
@@ -381,7 +273,7 @@ def make_hash(obj):
     elif isinstance(obj, dict):
         hash_string = ''.join(
             ''.join((key, make_hash(value)))
-            for key, value in six.iteritems(obj)
+            for key, value in obj.items()
         )
 
     else:
@@ -392,7 +284,7 @@ def make_hash(obj):
             # group them under the same operation hash.
             else '_PYINFRA_CONSTANT' if obj in (True, False, None)
             # Plain strings
-            else obj if isinstance(obj, six.string_types)
+            else obj if isinstance(obj, str)
             # Objects with __name__s
             else obj.__name__ if hasattr(obj, '__name__')
             # Objects with names
@@ -417,7 +309,7 @@ class get_file_io(object):
             # Check we can be read
             hasattr(filename_or_io, 'read')
             # Or we're a filename
-            or isinstance(filename_or_io, six.string_types)
+            or isinstance(filename_or_io, str)
         ):
             raise TypeError('Invalid filename or IO object: {0}'.format(
                 filename_or_io,
@@ -451,7 +343,7 @@ class get_file_io(object):
     def cache_key(self):
         # If we're a filename, cache against that - we don't cache in-memory
         # file objects.
-        if isinstance(self.filename_or_io, six.string_types):
+        if isinstance(self.filename_or_io, str):
             return self.filename_or_io
 
 
@@ -471,7 +363,7 @@ def get_file_sha1(filename_or_io):
         buff = file_io.read(BLOCKSIZE)
 
         while len(buff) > 0:
-            if isinstance(buff, six.text_type):
+            if isinstance(buff, str):
                 buff = buff.encode('utf-8')
 
             hasher.update(buff)
