@@ -1,7 +1,5 @@
 # encoding: utf8
 
-from __future__ import unicode_literals
-
 from pyinfra.api import FactBase
 
 from .util.packaging import parse_packages
@@ -16,7 +14,7 @@ class NpmPackages(FactBase):
     .. code:: python
 
         {
-            'package_name': ['version'],
+            "package_name": ["version"],
         }
     '''
 
