@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from pyinfra.api import FactBase
 
 from .util.packaging import parse_packages
@@ -14,7 +12,7 @@ class PipPackages(FactBase):
     .. code:: python
 
         {
-            'package_name': ['version'],
+            "package_name": ["version"],
         }
     '''
 
