@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import re
 
 from pyinfra.api import FactBase
@@ -17,7 +15,7 @@ class RpmPackages(FactBase):
     .. code:: python
 
         {
-            'package_name': ['version'],
+            "package_name": ["version"],
         }
     '''
 
@@ -27,11 +25,7 @@ class RpmPackages(FactBase):
     default = dict
 
     def process(self, output):
-        return parse_packages(
-            rpm_regex, output,
-            # yum packages are case-sensitive
-            lower=False,
-        )
+        return parse_packages(rpm_regex, output)
 
 
 class RpmPackage(FactBase):
@@ -41,8 +35,8 @@ class RpmPackage(FactBase):
     .. code:: python
 
         {
-            'name': 'my_package',
-            'version': '1.0.0',
+            "name": "my_package",
+            "version": "1.0.0",
         }
     '''
 
