@@ -1,18 +1,14 @@
 import re
 
 
-def parse_packages(regex, output, lower=True):
+def parse_packages(regex, output):
     packages = {}
 
     for line in output:
         matches = re.match(regex, line)
 
         if matches:
-            # Sort out name
             name = matches.group(1)
-            if lower:
-                name = name.lower()
-
             packages.setdefault(name, set())
             packages[name].add(matches.group(2))
 
