@@ -12,11 +12,11 @@ class ZypperRepositories(FactBase):
 
         [
             {
-                'name': 'Main Repository',
-                'enabled': '1',
-                'autorefresh': '0',
-                'baseurl': 'http://download.opensuse.org/distribution/leap/$releasever/repo/oss/',
-                'type': 'rpm-md',
+                "name": "Main Repository",
+                "enabled": "1",
+                "autorefresh": "0",
+                "baseurl": "http://download.opensuse.org/distribution/leap/$releasever/repo/oss/",
+                "type": "rpm-md",
             },
         ]
     '''
