@@ -1,34 +1,29 @@
-from __future__ import print_function, unicode_literals
-
 from os import path
 
 import click
-import six
 
 import pyinfra
-
-from . import logger, pseudo_config, pseudo_state
-from .api.connectors.util import run_local_process, split_combined_output
-from .api.exceptions import PyinfraError
+from pyinfra import config, host, logger, state
+from pyinfra.api.exceptions import PyinfraError
+from pyinfra.api.util import get_file_path
+from pyinfra.connectors.util import run_local_process, split_combined_output
+from pyinfra.context import ctx_state
 
 
 def include(filename):
     '''
-    Executes a local python file within the ``pyinfra.pseudo_state.deploy_dir``
+    Executes a local python file within the ``pyinfra.state.cwd``
     directory.
     '''
 
     if not pyinfra.is_cli:
         raise PyinfraError('local.include is only available in CLI mode.')
 
-    if filename.startswith('.'):
-        filename = path.join(path.dirname(pseudo_state.current_exec_filename), filename)
-    elif pseudo_state.deploy_dir:
-        filename = path.join(pseudo_state.deploy_dir, filename)
+    filename = get_file_path(state, filename)
 
     logger.debug('Including local file: {0}'.format(filename))
 
-    config_state = pseudo_config.get_current_state()
+    config_state = config.get_current_state()
 
     try:
         # Fixes a circular import because `pyinfra.local` is really a CLI
@@ -36,21 +31,9 @@ def include(filename):
         # to maintain backwards compatibility and the nicer public import
         # (ideally users never need to import from `pyinfra_cli`).
 
-        from pyinfra_cli.config import extract_file_config
         from pyinfra_cli.util import exec_file
 
-        # Load any config defined in the file and setup like a @deploy
-        # TODO: remove this in v2
-        config_data = extract_file_config(filename)
-        kwargs = {
-            key.lower(): value
-            for key, value in six.iteritems(config_data)
-            if key in [
-                'SUDO', 'SUDO_USER', 'SU_USER',
-                'PRESERVE_SUDO_ENV', 'IGNORE_ERRORS',
-            ]
-        }
-        with pseudo_state.deploy(path.normpath(filename), kwargs, None, in_deploy=False):
+        with host.deploy(path.relpath(filename, state.cwd), None, None, in_deploy=False):
             exec_file(filename)
 
         # One potential solution to the above is to add local as an actual
@@ -62,7 +45,7 @@ def include(filename):
         )
 
     finally:
-        pseudo_config.set_current_state(config_state)
+        config.set_current_state(config_state)
 
 
 def shell(commands, splitlines=False, ignore_errors=False, print_output=False, print_input=False):
@@ -75,16 +58,16 @@ def shell(commands, splitlines=False, ignore_errors=False, print_output=False, p
         ignore_errors (bool): ignore errors when executing these commands
     '''
 
-    if isinstance(commands, six.string_types):
+    if isinstance(commands, str):
         commands = [commands]
 
     all_stdout = []
 
-    # Checking for pseudo_state means this function works outside a deploy
+    # Checking for state context being set means this function works outside a deploy
     # e.g.: the vagrant connector.
-    if pseudo_state.isset():
-        print_output = pseudo_state.print_output
-        print_input = pseudo_state.print_input
+    if ctx_state.isset():
+        print_output = state.print_output
+        print_input = state.print_input
 
     for command in commands:
         print_prefix = 'localhost: '
