@@ -2,14 +2,10 @@
 Manage apt packages and repositories.
 '''
 
-from __future__ import unicode_literals
-
 from datetime import datetime, timedelta
+from urllib.parse import urlparse
 
-import six
-
-from six.moves.urllib.parse import urlparse
-
+from pyinfra import host, state
 from pyinfra.api import operation, OperationError
 from pyinfra.facts.apt import AptKeys, AptSources, parse_apt_repo
 from pyinfra.facts.deb import DebPackage, DebPackages
@@ -39,7 +35,7 @@ def noninteractive_apt(command, force=False):
 
 
 @operation
-def key(src=None, keyserver=None, keyid=None, state=None, host=None):
+def key(src=None, keyserver=None, keyid=None):
     '''
     Add apt gpg keys with ``apt-key``.
 
@@ -50,19 +46,19 @@ def key(src=None, keyserver=None, keyid=None, state=None, host=None):
     keyserver/id:
         These must be provided together.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # Note: If using URL, wget is assumed to be installed.
         apt.key(
-            name='Add the Docker apt gpg key',
-            src='https://download.docker.com/linux/ubuntu/gpg',
+            name="Add the Docker apt gpg key",
+            src="https://download.docker.com/linux/ubuntu/gpg",
         )
 
         apt.key(
-            name='Install VirtualBox key',
-            src='https://www.virtualbox.org/download/oracle_vbox_2016.asc',
+            name="Install VirtualBox key",
+            src="https://www.virtualbox.org/download/oracle_vbox_2016.asc",
         )
     '''
 
@@ -91,7 +87,7 @@ def key(src=None, keyserver=None, keyid=None, state=None, host=None):
         if not keyid:
             raise OperationError('`keyid` must be provided with `keyserver`')
 
-        if isinstance(keyid, six.string_types):
+        if isinstance(keyid, str):
             keyid = [keyid]
 
         needed_keys = sorted(set(keyid) - set(existing_keys.keys()))
@@ -108,7 +104,7 @@ def key(src=None, keyserver=None, keyid=None, state=None, host=None):
 
 
 @operation
-def repo(src, present=True, filename=None, state=None, host=None):
+def repo(src, present=True, filename=None):
     '''
     Add/remove apt repositories.
 
@@ -117,13 +113,13 @@ def repo(src, present=True, filename=None, state=None, host=None):
     + filename: optional filename to use ``/etc/apt/sources.list.d/<filename>.list``. By
       default uses ``/etc/apt/sources.list``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         apt.repo(
-            name='Install VirtualBox repo',
-            src='deb https://download.virtualbox.org/virtualbox/debian bionic contrib',
+            name="Install VirtualBox repo",
+            src="deb https://download.virtualbox.org/virtualbox/debian bionic contrib",
         )
     '''
 
@@ -143,25 +139,21 @@ def repo(src, present=True, filename=None, state=None, host=None):
 
     # Doesn't exist and we want it
     if not is_present and present:
-        yield files.line(
+        yield from files.line(
             filename,
             src,
             escape_regex_characters=True,
-            state=state,
-            host=host,
         )
         apt_sources.append(repo)
 
     # Exists and we don't want it
     elif is_present and not present:
-        yield files.line(
+        yield from files.line(
             filename,
             src,
             present=False,
             assume_present=True,
             escape_regex_characters=True,
-            state=state,
-            host=host,
         )
         apt_sources.remove(repo)
 
@@ -173,7 +165,7 @@ def repo(src, present=True, filename=None, state=None, host=None):
 
 
 @operation(is_idempotent=False)
-def ppa(src, present=True, state=None, host=None):
+def ppa(src, present=True):
     '''
     Add/remove Ubuntu ppa repositories.
 
@@ -183,14 +175,14 @@ def ppa(src, present=True, state=None, host=None):
     Note:
         requires ``apt-add-repository`` on the remote host
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Note: Assumes software-properties-common is installed.
         apt.ppa(
-            name='Add the Bitcoin ppa',
-            src='ppa:bitcoin/bitcoin',
+            name="Add the Bitcoin ppa",
+            src="ppa:bitcoin/bitcoin",
         )
 
     '''
@@ -203,7 +195,7 @@ def ppa(src, present=True, state=None, host=None):
 
 
 @operation
-def deb(src, present=True, force=False, state=None, host=None):
+def deb(src, present=True, force=False):
     '''
     Add/remove ``.deb`` file packages.
 
@@ -219,14 +211,14 @@ def deb(src, present=True, force=False, state=None, host=None):
         If the ``.deb`` file isn't downloaded, pyinfra can't remove any existing
         package as the file won't exist until mid-deploy.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Note: Assumes wget is installed.
         apt.deb(
-            name='Install Chrome via deb',
-            src='https://dl.google.com/linux/direct/google-chrome-stable_current_amd64.deb',
+            name="Install Chrome via deb",
+            src="https://dl.google.com/linux/direct/google-chrome-stable_current_amd64.deb",
         )
     '''
 
@@ -238,7 +230,7 @@ def deb(src, present=True, force=False, state=None, host=None):
         temp_filename = state.get_temp_filename(src)
 
         # Ensure it's downloaded
-        yield files.download(src, temp_filename, state=state, host=host)
+        yield from files.download(src, temp_filename)
 
         # Override the source with the downloaded file
         src = temp_filename
@@ -285,20 +277,25 @@ def deb(src, present=True, force=False, state=None, host=None):
             host.noop('deb {0} is not installed'.format(original_src))
 
 
-@operation
-def update(cache_time=None, touch_periodic=False, state=None, host=None):
+@operation(
+    is_idempotent=False,
+    idempotent_notice=(
+        'This operation will always execute commands '
+        'unless the ``cache_time`` argument is provided.'
+    ),
+)
+def update(cache_time=None):
     '''
     Updates apt repositories.
 
     + cache_time: cache updates for this many seconds
-    + touch_periodic: touch ``/var/lib/apt/periodic/update-success-stamp`` after update
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         apt.update(
-            name='Update apt repositories',
+            name="Update apt repositories",
             cache_time=3600,
         )
     '''
@@ -335,16 +332,16 @@ _update = update  # noqa: E305
 
 
 @operation(is_idempotent=False)
-def upgrade(state, host):
+def upgrade():
     '''
     Upgrades all apt packages.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         apt.upgrade(
-            name='Upgrade apt packages',
+            name="Upgrade apt packages",
         )
     '''
 
@@ -353,17 +350,17 @@ def upgrade(state, host):
 _upgrade = upgrade  # noqa: E305 (for use below where update is a kwarg)
 
 
-@operation
+@operation(is_idempotent=False)
 def dist_upgrade(state, host):
     '''
     Updates all apt packages, employing dist-upgrade.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         apt.dist_upgrade(
-            name='Upgrade apt packages using dist-upgrade',
+            name="Upgrade apt packages using dist-upgrade",
         )
     '''
 
@@ -376,7 +373,6 @@ def packages(
     update=False, cache_time=None, upgrade=False,
     force=False, no_recommends=False, allow_downgrades=False,
     extra_install_args=None, extra_uninstall_args=None,
-    state=None, host=None,
 ):
     '''
     Install/remove/update packages & update apt.
@@ -401,37 +397,37 @@ def packages(
         is touched upon successful update. Some distros already do this (Ubuntu), but others
         simply leave the periodic directory empty (Debian).
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # Update package list and install packages
         apt.packages(
-            name='Install Asterisk and Vim',
-            packages=['asterisk', 'vim'],
+            name="Install Asterisk and Vim",
+            packages=["asterisk", "vim"],
             update=True,
         )
 
         # Install the latest versions of packages (always check)
         apt.packages(
-            name='Install latest Vim',
-            packages=['vim'],
+            name="Install latest Vim",
+            packages=["vim"],
             latest=True,
         )
 
         # Note: host.get_fact(OsVersion) is the same as `uname -r` (ex: '4.15.0-72-generic')
         apt.packages(
-            name='Install kernel headers',
-            packages=['linux-headers-{}'.format(host.get_fact(OsVersion))],
+            name="Install kernel headers",
+            packages=[f"linux-headers-{host.get_fact(OsVersion)}"],
             update=True,
         )
     '''
 
     if update:
-        yield _update(cache_time=cache_time, state=state, host=host)
+        yield from _update(cache_time=cache_time)
 
     if upgrade:
-        yield _upgrade(state=state, host=host)
+        yield from _upgrade()
 
     install_command = ['install']
     if no_recommends is True:
@@ -453,7 +449,7 @@ def packages(
     uninstall_command = ' '.join(uninstall_command)
 
     # Compare/ensure packages are present/not
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, host.get_fact(DebPackages), present,
         install_command=noninteractive_apt(install_command, force=force),
         uninstall_command=noninteractive_apt(uninstall_command, force=force),
