@@ -2,6 +2,7 @@
 Manage brew packages on mac/OSX. See https://brew.sh/
 '''
 
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.brew import (
     BrewCasks,
@@ -15,7 +16,7 @@ from .util.packaging import ensure_packages
 
 
 @operation(is_idempotent=False)
-def update(state=None, host=None):
+def update():
     '''
     Updates brew repositories.
     '''
@@ -26,7 +27,7 @@ _update = update  # noqa: E305
 
 
 @operation(is_idempotent=False)
-def upgrade(state=None, host=None):
+def upgrade():
     '''
     Upgrades all brew packages.
     '''
@@ -40,7 +41,6 @@ _upgrade = upgrade  # noqa: E305
 def packages(
     packages=None, present=True, latest=False,
     update=False, upgrade=False,
-    state=None, host=None,
 ):
     '''
     Add/remove/update brew packages.
@@ -54,32 +54,32 @@ def packages(
     Versions:
         Package versions can be pinned like brew: ``<pkg>@<version>``.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # Update package list and install packages
         brew.packages(
             name='Install Vim and vimpager',
-            packages=['vimpager', 'vim'],
+            packages=["vimpager", "vim"],
             update=True,
         )
 
         # Install the latest versions of packages (always check)
         brew.packages(
-            name='Install latest Vim',
-            packages=['vim'],
+            name="Install latest Vim",
+            packages=["vim"],
             latest=True,
         )
     '''
 
     if update:
-        yield _update(state=state, host=host)
+        yield from _update()
 
     if upgrade:
-        yield _upgrade(state=state, host=host)
+        yield from _upgrade()
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, host.get_fact(BrewPackages), present,
         install_command='brew install',
         uninstall_command='brew uninstall',
@@ -98,7 +98,7 @@ def cask_args(host):
     pipeline_facts={
         'brew_version': '',
     })
-def cask_upgrade(state=None, host=None):
+def cask_upgrade():
     '''
     Upgrades all brew casks.
     '''
@@ -111,7 +111,6 @@ def cask_upgrade(state=None, host=None):
 })
 def casks(
     casks=None, present=True, latest=False, upgrade=False,
-    state=None, host=None,
 ):
     '''
     Add/remove/update brew casks.
@@ -124,13 +123,13 @@ def casks(
     Versions:
         Cask versions can be pinned like brew: ``<pkg>@<version>``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         brew.casks(
             name='Upgrade and install the latest cask',
-            casks=['godot'],
+            casks=["godot"],
             upgrade=True,
             latest=True,
         )
@@ -138,11 +137,11 @@ def casks(
     '''
 
     if upgrade:
-        yield cask_upgrade(state=state, host=host)
+        yield from cask_upgrade()
 
     args = cask_args(host)
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, casks, host.get_fact(BrewCasks), present,
         install_command='brew %sinstall%s' % args,
         uninstall_command='brew %suninstall%s' % args,
@@ -153,27 +152,26 @@ def casks(
 
 
 @operation
-def tap(src, present=True, state=None, host=None):
+def tap(src, present=True):
     '''
     Add/remove brew taps.
 
     + src: the name of the tap
     + present: whether this tap should be present or not
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         brew.tap(
-            name='Add a brew tap',
-            src='includeos/includeos',
+            name="Add a brew tap",
+            src="includeos/includeos",
         )
 
-        # multiple taps
-        taps = ['includeos/includeos', 'ktr0731/evans']
-        for tap in taps:
+        # Multiple taps
+        for tap in ["includeos/includeos", "ktr0731/evans"]:
             brew.tap(
-                name={f'Add brew tap {tap}'},
+                name={f"Add brew tap {tap}"},
                 src=tap,
             )
 
