@@ -2,8 +2,7 @@
 Manage BSD init services (``/etc/rc.d``, ``/usr/local/etc/rc.d``).
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.bsdinit import RcdStatus
 from pyinfra.facts.server import Os
@@ -17,7 +16,6 @@ def service(
     service,
     running=True, restarted=False, reloaded=False,
     command=None, enabled=None,
-    state=None, host=None,
 ):
     '''
     Manage the state of BSD init services.
@@ -34,7 +32,7 @@ def service(
     if host.get_fact(Os) == 'OpenBSD':
         status_argument = 'check'
 
-    yield handle_service_control(
+    yield from handle_service_control(
         host,
         service, host.get_fact(RcdStatus),
         'test -e /etc/rc.d/{0} && /etc/rc.d/{0} {1} || /usr/local/etc/rc.d/{0} {1}',
@@ -44,10 +42,9 @@ def service(
 
     # BSD init is simple, just add/remove <service>_enabled="YES"
     if isinstance(enabled, bool):
-        yield files.line(
+        yield from files.line(
             '/etc/rc.conf.local',
             '^{0}_enable='.format(service),
             replace='{0}_enable="YES"'.format(service),
             present=enabled,
-            state=state, host=host,
         )
