@@ -2,6 +2,7 @@
 Manage ``choco`` (Chocolatey) packages (https://chocolatey.org).
 '''
 
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.choco import ChocoPackages
 
@@ -9,7 +10,7 @@ from .util.packaging import ensure_packages
 
 
 @operation
-def packages(packages=None, present=True, latest=False, state=None, host=None):
+def packages(packages=None, present=True, latest=False):
     '''
     Add/remove/update ``choco`` packages.
 
@@ -20,19 +21,19 @@ def packages(packages=None, present=True, latest=False, state=None, host=None):
     Versions:
         Package versions can be pinned like gem: ``<pkg>:<version>``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Note: Assumes that 'choco' is installed and
         #       user has Administrator permission.
         choco.packages(
-            name='Install Notepad++',
-            packages=['notepadplusplus'],
+            name="Install Notepad++",
+            packages=["notepadplusplus"],
         )
     '''
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, host.get_fact(ChocoPackages), present,
         install_command='choco install -y',
         uninstall_command='choco uninstall -y -x',
@@ -42,8 +43,8 @@ def packages(packages=None, present=True, latest=False, state=None, host=None):
     )
 
 
-@operation
-def install(state=None, host=None):
+@operation(is_idempotent=False)
+def install():
     '''
     Install ``choco`` (Chocolatey).
     '''
