@@ -2,8 +2,7 @@
 Manage dnf packages and repositories. Note that dnf package names are case-sensitive.
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host, state
 from pyinfra.api import operation
 from pyinfra.facts.rpm import RpmPackageProvides, RpmPackages
 
@@ -12,7 +11,7 @@ from .util.packaging import ensure_packages, ensure_rpm, ensure_yum_repo
 
 
 @operation(is_idempotent=False)
-def key(src, state=None, host=None):
+def key(src):
     '''
     Add dnf gpg keys with ``rpm``.
 
@@ -21,14 +20,14 @@ def key(src, state=None, host=None):
     Note:
         always returns one command, not idempotent
 
-    Example:
+    **Example:**
 
     .. code:: python
 
-        linux_id = host.get_fact(LinuxDistribution)['release_meta'].get('ID')
+        linux_id = host.get_fact(LinuxDistribution)["release_meta"].get("ID")
         dnf.key(
-            name='Add the Docker CentOS gpg key',
-            src='https://download.docker.com/linux/{}/gpg'.format(linux_id),
+            name="Add the Docker CentOS gpg key",
+            src=f"https://download.docker.com/linux/{linux_id}/gpg",
         )
 
     '''
@@ -42,7 +41,6 @@ def repo(
     present=True,
     baseurl=None, description=None,
     enabled=True, gpgcheck=True, gpgkey=None,
-    state=None, host=None,
 ):
     # NOTE: if updating this docstring also update `yum.repo`
     '''
@@ -61,32 +59,32 @@ def repo(
         for manual construction of repository files. Use a URL to download and
         install remote repository files.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # Download a repository file
         dnf.rpm(
-            name='Install Docker-CE repo via URL',
-            src='https://download.docker.com/linux/centos/docker-ce.repo',
+            name="Install Docker-CE repo via URL",
+            src="https://download.docker.com/linux/centos/docker-ce.repo",
         )
 
         # Create the repository file from baseurl/etc
         dnf.repo(
-            name='Add the Docker CentOS repo',
-            src='DockerCE',
-            baseurl='https://download.docker.com/linux/centos/7/$basearch/stable',
+            name="Add the Docker CentOS repo",
+            src="DockerCE",
+            baseurl="https://download.docker.com/linux/centos/7/$basearch/stable",
         )
     '''
 
-    yield ensure_yum_repo(
+    yield from ensure_yum_repo(
         state, host, files,
         src, baseurl, present, description, enabled, gpgcheck, gpgkey,
     )
 
 
 @operation
-def rpm(src, present=True, state=None, host=None):
+def rpm(src, present=True):
     # NOTE: if updating this docstring also update `yum.rpm`
     '''
     Add/remove ``.rpm`` file packages.
@@ -98,22 +96,22 @@ def rpm(src, present=True, state=None, host=None):
         If the ``.rpm`` file isn't downloaded, pyinfra can't remove any existing
         package as the file won't exist until mid-deploy.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
-        major_centos_version = host.get_fact(LinuxDistribution)['major']
+        major_centos_version = host.get_fact(LinuxDistribution)["major"]
         dnf.rpm(
-           name='Install EPEL rpm to enable EPEL repo',
-           src='https://dl.fedoraproject.org/pub/epel/epel-release-latest-{}.noarch.rpm'.format(major_centos_version),
+           name="Install EPEL rpm to enable EPEL repo",
+           src=f"https://dl.fedoraproject.org/pub/epel/epel-release-latest-{major_centos_version}.noarch.rpm",
         )
     '''
 
-    yield ensure_rpm(state, host, files, src, present, 'dnf')
+    yield from ensure_rpm(state, host, files, src, present, 'dnf')
 
 
-@operation
-def update(state=None, host=None):
+@operation(is_idempotent=False)
+def update():
     '''
     Updates all dnf packages.
     '''
@@ -128,7 +126,6 @@ def packages(
     packages=None,
     present=True, latest=False, update=False, clean=False, nobest=False,
     extra_install_args=None, extra_uninstall_args=None,
-    state=None, host=None,
 ):
     '''
     Install/remove/update dnf packages & updates.
@@ -145,21 +142,21 @@ def packages(
     Versions:
         Package versions can be pinned as follows: ``<pkg>=<version>``
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # Update package list and install packages
         dnf.packages(
             name='Install Vim and Vim enhanced',
-            packages=['vim-enhanced', 'vim'],
+            packages=["vim-enhanced", "vim"],
             update=True,
         )
 
         # Install the latest versions of packages (always check)
         dnf.packages(
-            name='Install latest Vim',
-            packages=['vim'],
+            name="Install latest Vim",
+            packages=["vim"],
             latest=True,
         )
     '''
@@ -168,7 +165,7 @@ def packages(
         yield 'dnf clean all'
 
     if update:
-        yield _update(state=state, host=host)
+        yield from _update()
 
     install_command = ['dnf', 'install', '-y']
 
@@ -183,13 +180,12 @@ def packages(
     if extra_uninstall_args:
         uninstall_command.append(extra_uninstall_args)
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, host.get_fact(RpmPackages), present,
         install_command=' '.join(install_command),
         uninstall_command=' '.join(uninstall_command),
         upgrade_command='dnf update -y',
         version_join='=',
-        lower=False,  # dnf packages are case sensitive
         latest=latest,
         expand_package_fact=lambda package: host.get_fact(
             RpmPackageProvides,
