@@ -1,9 +1,7 @@
 '''
-The files module handles filesystem state, file uploads and template generation.
+The files operations handles filesystem state, file uploads and template generation.
 '''
 
-from __future__ import unicode_literals
-
 import os
 import posixpath
 import re
@@ -11,12 +9,12 @@ import sys
 import traceback
 from datetime import timedelta
 from fnmatch import fnmatch
+from io import StringIO
 
-import six
 from jinja2 import TemplateRuntimeError, TemplateSyntaxError, UndefinedError
 
 import pyinfra
-from pyinfra import logger
+from pyinfra import host, logger, state
 from pyinfra.api import (
     FileDownloadCommand,
     FileUploadCommand,
@@ -29,6 +27,7 @@ from pyinfra.api import (
 )
 from pyinfra.api.command import make_formatted_string_command
 from pyinfra.api.util import (
+    get_call_location,
     get_file_sha1,
     get_path_permissions_mode,
     get_template,
@@ -46,7 +45,6 @@ from pyinfra.facts.files import (
 )
 from pyinfra.facts.server import Date, Which
 
-from .util.compat import fspath
 from .util.files import (
     chmod,
     chown,
@@ -65,7 +63,6 @@ def download(
     src, dest,
     user=None, group=None, mode=None, cache_time=None, force=False,
     sha256sum=None, sha1sum=None, md5sum=None,
-    state=None, host=None,
 ):
     '''
     Download files from remote locations using ``curl`` or ``wget``.
@@ -81,14 +78,14 @@ def download(
     + sha1sum: sha1 hash to checksum the downloaded file against
     + md5sum: md5 hash to checksum the downloaded file against
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         files.download(
-            name='Download the Docker repo file',
-            src='https://download.docker.com/linux/centos/docker-ce.repo',
-            dest='/etc/yum.repos.d/docker-ce.repo',
+            name="Download the Docker repo file",
+            src="https://download.docker.com/linux/centos/docker-ce.repo",
+            dest="/etc/yum.repos.d/docker-ce.repo",
         )
     '''
 
@@ -206,7 +203,6 @@ def line(
     present=True, replace=None, flags=None, backup=False,
     interpolate_variables=False,
     escape_regex_characters=False,
-    state=None, host=None,
     assume_present=False,
 ):
     '''
@@ -240,49 +236,49 @@ def line(
         If ``line`` is not in the file but we want it (``present`` set to ``True``), then
         it will be append to the end of the file.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # prepare to do some maintenance
-        maintenance_line = 'SYSTEM IS DOWN FOR MAINTENANCE'
+        maintenance_line = "SYSTEM IS DOWN FOR MAINTENANCE"
         files.line(
-            name='Add the down-for-maintence line in /etc/motd',
-            path='/etc/motd',
+            name="Add the down-for-maintence line in /etc/motd",
+            path="/etc/motd",
             line=maintenance_line,
         )
 
         # Then, after the mantenance is done, remove the maintenance line
         files.line(
-            name='Remove the down-for-maintenance line in /etc/motd',
-            path='/etc/motd',
+            name="Remove the down-for-maintenance line in /etc/motd",
+            path="/etc/motd",
             line=maintenance_line,
-            replace='',
+            replace="",
             present=False,
         )
 
         # example where there is '*' in the line
         files.line(
-            name='Ensure /netboot/nfs is in /etc/exports',
-            path='/etc/exports',
-            line=r'/netboot/nfs .*',
-            replace='/netboot/nfs *(ro,sync,no_wdelay,insecure_locks,no_root_squash,'
-            'insecure,no_subtree_check)',
+            name="Ensure /netboot/nfs is in /etc/exports",
+            path="/etc/exports",
+            line=r"/netboot/nfs .*",
+            replace="/netboot/nfs *(ro,sync,no_wdelay,insecure_locks,no_root_squash,"
+            "insecure,no_subtree_check)",
         )
 
         files.line(
-            name='Ensure myweb can run /usr/bin/python3 without password',
-            path='/etc/sudoers',
-            line=r'myweb .*',
-            replace='myweb ALL=(ALL) NOPASSWD: /usr/bin/python3',
+            name="Ensure myweb can run /usr/bin/python3 without password",
+            path="/etc/sudoers",
+            line=r"myweb .*",
+            replace="myweb ALL=(ALL) NOPASSWD: /usr/bin/python3",
         )
 
         # example when there are double quotes (")
         line = 'QUOTAUSER=""'
         files.line(
-            name='Example with double quotes (")',
-            path='/etc/adduser.conf',
-            line='^{}$'.format(line),
+            name="Example with double quotes (")",
+            path="/etc/adduser.conf",
+            line="^{}$".format(line),
             replace=line,
         )
     '''
@@ -417,7 +413,6 @@ def replace(
     flags=None, backup=False,
     interpolate_variables=False,
     match=None,  # deprecated
-    state=None, host=None,
 ):
     '''
     Replace contents of a file using ``sed``.
@@ -434,22 +429,25 @@ def replace(
         date (taken from the machine running ``pyinfra``) appended as the extension. If
         you pass a string value this will be used as the extension of the backed up file.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         files.replace(
-            name='Change part of a line in a file',
-            path='/etc/motd',
-            text='verboten',
-            replace='forbidden',
+            name="Change part of a line in a file",
+            path="/etc/motd",
+            text="verboten",
+            replace="forbidden",
         )
     '''
 
     if text is None and match:
         text = match
         logger.warning(
-            'The `match` argument has been replaced by `text` in the `files.replace` operation',
+            (
+                'The `match` argument has been replaced by '
+                '`text` in the `files.replace` operation ({0})'
+            ).format(get_call_location()),
         )
 
     if text is None:
@@ -494,7 +492,6 @@ def sync(
     src, dest,
     user=None, group=None, mode=None, delete=False,
     exclude=None, exclude_dir=None, add_deploy_dir=True,
-    state=None, host=None,
 ):
     '''
     Syncs a local directory with a remote one, with delete support. Note that delete will
@@ -510,15 +507,15 @@ def sync(
     + exclude_dir: string or list/tuple of strings to match & exclude directories (eg node_modules)
     + add_deploy_dir: interpret src as relative to deploy directory instead of current directory
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Sync local files/tempdir to remote /tmp/tempdir
         files.sync(
-            name='Sync a local directory with remote',
-            src='files/tempdir',
-            dest='/tmp/tempdir',
+            name="Sync a local directory with remote",
+            src="files/tempdir",
+            dest="/tmp/tempdir",
         )
 
     Note: ``exclude`` and ``exclude_dir`` use ``fnmatch`` behind the scenes to do the filtering.
@@ -534,8 +531,8 @@ def sync(
     src = os.path.normpath(src)
 
     # Add deploy directory?
-    if add_deploy_dir and state.deploy_dir:
-        src = os.path.join(state.deploy_dir, src)
+    if add_deploy_dir and state.cwd:
+        src = os.path.join(state.cwd, src)
 
     # Ensure the source directory exists
     if not os.path.isdir(src):
@@ -586,30 +583,27 @@ def sync(
     if dest_link_info:
         dest_to_ensure = dest_link_info['link_target']
 
-    yield directory(
+    yield from directory(
         dest_to_ensure,
         user=user, group=group,
         mode=get_path_permissions_mode(src),
-        state=state, host=host,
     )
 
     # Ensure any remote dirnames
     for dirpath, dir_mode in ensure_dirnames:
-        yield directory(
+        yield from directory(
             unix_path_join(dest, dirpath),
             user=user, group=group, mode=dir_mode,
-            state=state, host=host,
         )
 
     # Put each file combination
     for local_filename, remote_filename in put_files:
-        yield put(
+        yield from put(
             local_filename, remote_filename,
             user=user, group=group,
             mode=mode or get_path_permissions_mode(local_filename),
             add_deploy_dir=False,
             create_remote_dir=False,  # handled above
-            state=state, host=host,
         )
 
     # Delete any extra files
@@ -622,7 +616,7 @@ def sync(
             if exclude and any(fnmatch(filename, match) for match in exclude):
                 continue
 
-            yield file(filename, present=False, state=state, host=host)
+            yield from file(filename, present=False)
 
 
 @memoize
@@ -630,11 +624,8 @@ def show_rsync_warning():
     logger.warning('The `files.rsync` operation is in alpha!')
 
 
-@operation
-def rsync(
-    src, dest, flags=['-ax', '--delete'],
-    state=None, host=None,
-):
+@operation(is_idempotent=False)
+def rsync(src, dest, flags=['-ax', '--delete']):
     '''
     Use ``rsync`` to sync a local directory to the remote system. This operation will actually call
     the ``rsync`` binary on your system.
@@ -662,12 +653,11 @@ def _create_remote_dir(state, host, remote_filename, user, group):
     # Always use POSIX style path as local might be Windows, remote always *nix
     remote_dirname = posixpath.dirname(remote_filename)
     if remote_dirname:
-        yield directory(
+        yield from directory(
             path=remote_dirname,
             user=user, group=group,
             _no_check_owner_mode=True,  # don't check existing user/mode
             _no_fail_on_link=True,  # don't fail if the path is a link
-            state=state, host=host,
         )
 
 
@@ -683,7 +673,6 @@ def _create_remote_dir(state, host, remote_filename, user, group):
 def get(
     src, dest,
     add_deploy_dir=True, create_local_dir=False, force=False,
-    state=None, host=None,
 ):
     '''
     Download a file from the remote system.
@@ -698,19 +687,19 @@ def get(
         This operation is not suitable for large files as it may involve copying
         the remote file before downloading it.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         files.get(
-            name='Download a file from a remote',
-            src='/etc/centos-release',
-            dest='/tmp/whocares',
+            name="Download a file from a remote",
+            src="/etc/centos-release",
+            dest="/tmp/whocares",
         )
     '''
 
-    if add_deploy_dir and state.deploy_dir:
-        dest = os.path.join(state.deploy_dir, dest)
+    if add_deploy_dir and state.cwd:
+        dest = os.path.join(state.cwd, dest)
 
     if create_local_dir:
         local_pathname = os.path.dirname(dest)
@@ -745,12 +734,11 @@ def put(
     src, dest,
     user=None, group=None, mode=None, add_deploy_dir=True,
     create_remote_dir=True, force=False, assume_exists=False,
-    state=None, host=None,
 ):
     '''
-    Upload a local file to the remote system.
+    Upload a local file, or file-like object, to the remote system.
 
-    + src: local filename to upload
+    + src: filename or IO-like object to upload
     + dest: remote filename to upload to
     + user: user to own the files
     + group: group to own the files
@@ -773,16 +761,21 @@ def put(
         This operation is not suitable for large files as it may involve copying
         the file before uploading it.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
-        # Note: This requires a 'files/motd' file on the local filesystem
         files.put(
-            name='Update the message of the day file',
-            src='files/motd',
-            dest='/etc/motd',
-            mode='644',
+            name="Update the message of the day file",
+            src="files/motd",
+            dest="/etc/motd",
+            mode="644",
+        )
+
+        files.put(
+            name="Upload a StringIO object",
+            src=StringIO("file contents"),
+            dest="/etc/motd",
         )
     '''
 
@@ -793,8 +786,8 @@ def put(
     # Assume string filename
     else:
         # Add deploy directory?
-        if add_deploy_dir and state.deploy_dir:
-            src = os.path.join(state.deploy_dir, src)
+        if add_deploy_dir and state.cwd:
+            src = os.path.join(state.cwd, src)
 
         local_file = src
 
@@ -808,7 +801,7 @@ def put(
         dest = unix_path_join(dest, os.path.basename(src))
 
     if create_remote_dir:
-        yield _create_remote_dir(state, host, dest, user, group)
+        yield from _create_remote_dir(state, host, dest, user, group)
 
     local_sum = get_file_sha1(src)
 
@@ -876,13 +869,12 @@ def put(
 def template(
     src, dest,
     user=None, group=None, mode=None, create_remote_dir=True,
-    state=None, host=None,
     **data
 ):
     '''
     Generate a template using jinja2 and write it to the remote system.
 
-    + src: local template filename
+    + src: template filename or IO-like object
     + dest: remote filename
     + user: user to own the files
     + group: group to own the files
@@ -901,23 +893,23 @@ def template(
        For information on the template syntax, see
        `the jinja2 docs <https://jinja.palletsprojects.com>`_.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         files.template(
-            name='Create a templated file',
-            src='templates/somefile.conf.j2',
-            dest='/etc/somefile.conf',
+            name="Create a templated file",
+            src="templates/somefile.conf.j2",
+            dest="/etc/somefile.conf",
         )
 
         files.template(
-            name='Create service file',
-            src='templates/myweb.service.j2',
-            dest='/etc/systemd/system/myweb.service',
-            mode='755',
-            user='root',
-            group='root',
+            name="Create service file",
+            src="templates/myweb.service.j2",
+            dest="/etc/systemd/system/myweb.service",
+            mode="755",
+            user="root",
+            group="root",
         )
 
         # Example showing how to pass python variable to template file. You can also
@@ -931,18 +923,8 @@ def template(
             "entry 1",
             "entry 2"
         ]
-        files.template(
-            name='Create a templated file',
-            src='templates/foo.yml.j2',
-            dest='/tmp/foo.yml',
-            foo_variable=foo_variable,
-            foo_dict=foo_dict,
-            foo_list=foo_list
-        )
 
-    .. code:: yml
-
-        # templates/foo.j2
+        template = StringIO("""
         name: "{{ foo_variable }}"
         dict_contents:
             str1: "{{ foo_dict.str1 }}"
@@ -951,10 +933,20 @@ def template(
         {% for entry in foo_list %}
             - "{{ entry }}"
         {% endfor %}
+        """)
+
+        files.template(
+            name="Create a templated file",
+            src=template,
+            dest="/tmp/foo.yml",
+            foo_variable=foo_variable,
+            foo_dict=foo_dict,
+            foo_list=foo_list
+        )
     '''
 
-    if state.deploy_dir:
-        src = os.path.join(state.deploy_dir, src)
+    if not hasattr(src, 'read') and state.cwd:
+        src = os.path.join(state.cwd, src)
 
     # Ensure host/state/inventory are available inside templates (if not set)
     data.setdefault('host', host)
@@ -985,23 +977,22 @@ def template(
             src, line_number, e, '\n'.join(relevant_lines),
         ))
 
-    output_file = six.StringIO(output)
+    output_file = StringIO(output)
     # Set the template attribute for nicer debugging
     output_file.template = src
 
     # Pass to the put function
-    yield put(
+    yield from put(
         output_file, dest,
         user=user, group=group, mode=mode,
         add_deploy_dir=False,
         create_remote_dir=create_remote_dir,
-        state=state, host=host,
     )
 
 
 def _validate_path(path):
     try:
-        return fspath(path)
+        return os.fspath(path)
     except TypeError:
         raise OperationTypeError('`path` must be a string or `os.PathLike` object')
 
@@ -1011,6 +1002,7 @@ def _raise_or_remove_invalid_path(fs_type, path, force, force_backup, force_back
         if force_backup:
             backup_path = '{0}.{1}'.format(path, get_timestamp())
             if force_backup_dir:
+                backup_path = os.path.basename(backup_path)
                 backup_path = '{0}/{1}'.format(force_backup_dir, backup_path)
             yield StringCommand('mv', QuoteString(path), QuoteString(backup_path))
         else:
@@ -1028,7 +1020,6 @@ def link(
     user=None, group=None, symbolic=True,
     create_remote_dir=True,
     force=False, force_backup=True, force_backup_dir=None,
-    state=None, host=None,
 ):
     '''
     Add/remove/update links.
@@ -1054,29 +1045,28 @@ def link(
         If the link exists and points to a different target, pyinfra will remove it and
         recreate a new one pointing to then new target.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
-        # simple example showing how to link to a file
         files.link(
-            name='Create link /etc/issue2 that points to /etc/issue',
-            path='/etc/issue2',
-            target='/etc/issue',
+            name="Create link /etc/issue2 that points to /etc/issue",
+            path="/etc/issue2",
+            target="/etc/issue",
         )
 
 
-        # complex example demonstrating the assume_present option
+        # Complex example demonstrating the assume_present option
         from pyinfra.operations import apt, files
 
         install_nginx = apt.packages(
-            name='Install nginx',
-            packages=['nginx'],
+            name="Install nginx",
+            packages=["nginx"],
         )
 
         files.link(
-            name='Remove default nginx site',
-            path='/etc/nginx/sites-enabled/default',
+            name="Remove default nginx site",
+            path="/etc/nginx/sites-enabled/default",
             present=False,
             assume_present=install_nginx.changed,
         )
@@ -1091,7 +1081,7 @@ def link(
 
     # Not a link?
     if info is False:
-        yield _raise_or_remove_invalid_path(
+        yield from _raise_or_remove_invalid_path(
             'link', path, force, force_backup, force_backup_dir,
         )
         info = None
@@ -1106,7 +1096,7 @@ def link(
     # No link and we want it
     if not assume_present and info is None and present:
         if create_remote_dir:
-            yield _create_remote_dir(state, host, path, user, group)
+            yield from _create_remote_dir(state, host, path, user, group)
 
         yield add_cmd
 
@@ -1178,7 +1168,6 @@ def file(
     user=None, group=None, mode=None, touch=False,
     create_remote_dir=True,
     force=False, force_backup=True, force_backup_dir=None,
-    state=None, host=None,
 ):
     '''
     Add/remove/update files.
@@ -1200,17 +1189,17 @@ def file(
         user & group as passed to ``files.put``. The mode will *not* be copied over,
         if this is required call ``files.directory`` separately.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Note: The directory /tmp/secret will get created with the default umask.
         files.file(
-            name='Create /tmp/secret/file',
-            path='/tmp/secret/file',
-            mode='600',
-            user='root',
-            group='root',
+            name="Create /tmp/secret/file",
+            path="/tmp/secret/file",
+            mode="600",
+            user="root",
+            group="root",
             touch=True,
             create_remote_dir=True,
         )
@@ -1223,7 +1212,7 @@ def file(
 
     # Not a file?!
     if info is False:
-        yield _raise_or_remove_invalid_path(
+        yield from _raise_or_remove_invalid_path(
             'file', path, force, force_backup, force_backup_dir,
         )
         info = None
@@ -1231,7 +1220,7 @@ def file(
     # Doesn't exist & we want it
     if not assume_present and info is None and present:
         if create_remote_dir:
-            yield _create_remote_dir(state, host, path, user, group)
+            yield from _create_remote_dir(state, host, path, user, group)
 
         yield StringCommand('touch', QuoteString(path))
 
@@ -1296,7 +1285,6 @@ def directory(
     force=False, force_backup=True, force_backup_dir=None,
     _no_check_owner_mode=False,
     _no_fail_on_link=False,
-    state=None, host=None,
 ):
     '''
     Add/remove/update directories.
@@ -1312,28 +1300,27 @@ def directory(
     + force_backup: set to ``False`` to remove any existing non-file when ``force=True``
     + force_backup_dir: directory to move any backup to when ``force=True``
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         files.directory(
-            name='Ensure the /tmp/dir_that_we_want_removed is removed',
-            path='/tmp/dir_that_we_want_removed',
+            name="Ensure the /tmp/dir_that_we_want_removed is removed",
+            path="/tmp/dir_that_we_want_removed",
             present=False,
         )
 
         files.directory(
-            name='Ensure /web exists',
-            path='/web',
-            user='myweb',
-            group='myweb',
+            name="Ensure /web exists",
+            path="/web",
+            user="myweb",
+            group="myweb",
         )
 
-        # multiple directories
-        dirs = ['/netboot/tftp', '/netboot/nfs']
-        for dir in dirs:
+        # Multiple directories
+        for dir in ["/netboot/tftp", "/netboot/nfs"]:
             files.directory(
-                name='Ensure the directory `{}` exists'.format(dir),
+                name="Ensure the directory `{}` exists".format(dir),
                 path=dir,
             )
     '''
@@ -1348,7 +1335,7 @@ def directory(
         if _no_fail_on_link and host.get_fact(Link, path=path):
             host.noop('directory {0} already exists (as a link)'.format(path))
             return
-        yield _raise_or_remove_invalid_path(
+        yield from _raise_or_remove_invalid_path(
             'directory', path, force, force_backup, force_backup_dir,
         )
         info = None
