@@ -2,6 +2,7 @@
 Manage Ruby gem packages. (see https://rubygems.org/ )
 '''
 
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.gem import GemPackages
 
@@ -9,7 +10,7 @@ from .util.packaging import ensure_packages
 
 
 @operation
-def packages(packages=None, present=True, latest=False, state=None, host=None):
+def packages(packages=None, present=True, latest=False):
     '''
     Add/remove/update gem packages.
 
@@ -20,18 +21,18 @@ def packages(packages=None, present=True, latest=False, state=None, host=None):
     Versions:
         Package versions can be pinned like gem: ``<pkg>:<version>``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Note: Assumes that 'gem' is installed.
         gem.packages(
-            name='Install rspec',
-            packages=['rspec'],
+            name="Install rspec",
+            packages=["rspec"],
         )
     '''
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, host.get_fact(GemPackages), present,
         install_command='gem install',
         uninstall_command='gem uninstall',
