@@ -2,10 +2,7 @@
 The iptables modules handles iptables rules
 '''
 
-from __future__ import unicode_literals
-
-import six
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.api.exceptions import OperationError
 from pyinfra.facts.iptables import (
@@ -20,7 +17,6 @@ from pyinfra.facts.iptables import (
 def chain(
     chain, present=True,
     table='filter', policy=None, version=4,
-    state=None, host=None,
 ):
     '''
     Add/remove/update iptables chains.
@@ -47,6 +43,7 @@ def chain(
     if not present:
         if chain in chains:
             yield '{0} -X {1}'.format(command, chain)
+            chains.pop(chain)
         else:
             host.noop('iptables chain {0} does not exist'.format(chain))
         return
@@ -54,12 +51,14 @@ def chain(
     if present:
         if chain not in chains:
             yield '{0} -N {1}'.format(command, chain)
+            chains[chain] = None  # policy will be set below
         else:
             host.noop('iptables chain {0} exists'.format(chain))
 
         if policy:
             if chain not in chains or chains[chain] != policy:
                 yield '{0} -P {1} {2}'.format(command, chain, policy)
+                chains[chain] = policy
 
 
 @operation
@@ -76,7 +75,6 @@ def rule(
     to_destination=None, to_source=None, to_ports=None, log_prefix=None,
     # Extras and extra shortcuts
     destination_port=None, source_port=None, extras='',
-    state=None, host=None,
 ):
     '''
     Add/remove iptables rules.
@@ -105,24 +103,25 @@ def rule(
     + destination_port: destination port (requires protocol)
     + source_port: source port (requires protocol)
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         iptables.rule(
-            name='Block SSH traffic',
-            chain='INPUT',
-            jump='DROP',
-            destination_port=22
+            name="Block SSH traffic",
+            chain="INPUT",
+            jump="DROP",
+            destination_port=22,
         )
 
         iptables.rule(
-            name='NAT traffic on from 8.8.8.8:53 to 8.8.4.4:8080',
-            chain='PREROUTING',
-            jump='DNAT',
-            table='nat',
-            source='8.8.8.8', destination_port=53,
-            to_destination='8.8.4.4:8080'
+            name="NAT traffic on from 8.8.8.8:53 to 8.8.4.4:8080",
+            chain="PREROUTING",
+            jump="DNAT",
+            table="nat",
+            source="8.8.8.8",
+            destination_port=53,
+            to_destination="8.8.4.4:8080",
         )
     '''
 
@@ -212,7 +211,7 @@ def rule(
             )
             else value
         )
-        for key, value in six.iteritems(definition)
+        for key, value in definition.items()
         if value
     }
 
