@@ -2,8 +2,7 @@
 Manage launchd services.
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.launchd import LaunchdStatus
 
@@ -14,7 +13,6 @@ from .util.service import handle_service_control
 def service(
     service,
     running=True, restarted=False, command=None,
-    state=None, host=None,
 ):
     '''
     Manage the state of systemd managed services.
@@ -28,7 +26,7 @@ def service(
 
     was_running = host.get_fact(LaunchdStatus).get(service, None)
 
-    yield handle_service_control(
+    yield from handle_service_control(
         host,
         service, host.get_fact(LaunchdStatus),
         'launchctl {1} {0}',
