@@ -2,8 +2,7 @@
 The LXD modules manage LXD containers
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.lxd import LxdContainers
 
@@ -20,7 +19,6 @@ def get_container_named(name, containers):
 def container(
     id,
     present=True, image='ubuntu:16.04',
-    state=None, host=None,
 ):
     '''
     Add/remove LXD containers.
@@ -32,18 +30,19 @@ def container(
     + image: image to base the container on
     + present: whether the container should be present or absent
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         lxd.container(
-            name='Add an ubuntu container',
-            id='ubuntu19',
-            image='ubuntu:19.10',
+            name="Add an ubuntu container",
+            id="ubuntu19",
+            image="ubuntu:19.10",
         )
     '''
 
-    container = get_container_named(id, host.get_fact(LxdContainers))
+    current_containers = host.get_fact(LxdContainers)
+    container = get_container_named(id, current_containers)
 
     # Container exists and we don't want it
     if not present:
@@ -53,6 +52,8 @@ def container(
 
             # Command to remove the container:
             yield 'lxc delete {0}'.format(id)
+
+            current_containers.remove(container)
         else:
             host.noop('container {0} does not exist'.format(id))
 
@@ -61,5 +62,9 @@ def container(
         if not container:
             # Command to create the container:
             yield 'lxc launch {image} {id}'.format(id=id, image=image)
+            current_containers.append({
+                'name': id,
+                'image': image,
+            })
         else:
             host.noop('container {0} exists'.format(id))
