@@ -3,7 +3,7 @@ Manage MySQL databases, users and privileges.
 
 Requires the ``mysql`` CLI executable on the target host(s).
 
-All operations in this module take four optional global arguments:
+All operations in this module take four optional arguments:
     + ``mysql_user``: the username to connect to mysql to
     + ``mysql_password``: the password for the connecting user
     + ``mysql_host``: the hostname of the server to connect to
@@ -13,9 +13,7 @@ See the example/mysql.py
 
 '''
 
-import six
-
-from pyinfra import logger
+from pyinfra import host
 from pyinfra.api import MaskString, operation, OperationError, StringCommand
 from pyinfra.facts.mysql import (
     make_execute_mysql_command,
@@ -33,7 +31,6 @@ def sql(
     # Details for speaking to MySQL via `mysql` CLI
     mysql_user=None, mysql_password=None,
     mysql_host=None, mysql_port=None,
-    state=None, host=None,
 ):
     '''
     Execute arbitrary SQL against MySQL.
@@ -75,7 +72,6 @@ def user(
     mysql_password=None,
     mysql_host=None,
     mysql_port=None,
-    state=None, host=None,
 ):
     '''
     Add/remove/update MySQL users.
@@ -96,40 +92,40 @@ def user(
         detect if the current password doesn't match the one provided, so won't
         attempt to change it.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         mysql.user(
-            name='Create the pyinfra@localhost MySQL user',
-            user='pyinfra',
-            password='somepassword',
+            name="Create the pyinfra@localhost MySQL user",
+            user="pyinfra",
+            password="somepassword",
         )
 
         # Create a user with resource limits
         mysql.user(
-            name='Create the pyinfra@localhost MySQL user',
-            user='pyinfra',
+            name="Create the pyinfra@localhost MySQL user",
+            user="pyinfra",
             max_connections=50,
             max_updates_per_hour=10,
         )
 
         # Create a user that requires SSL for connections
         mysql.user(
-            name='Create the pyinfra@localhost MySQL user',
-            user='pyinfra',
-            password='somepassword',
-            require='SSL',
+            name="Create the pyinfra@localhost MySQL user",
+            user="pyinfra",
+            password="somepassword",
+            require="SSL",
         )
 
         # Create a user that requires a specific certificate
         mysql.user(
-            name='Create the pyinfra@localhost MySQL user',
-            user='pyinfra',
-            password='somepassword',
-            require='X509',
-            require_issuer='/C=SE/ST=Stockholm...',
-            require_cipher='EDH-RSA-DES-CBC3-SHA',
+            name="Create the pyinfra@localhost MySQL user",
+            user="pyinfra",
+            password="somepassword",
+            require="X509",
+            require_issuer="/C=SE/ST=Stockholm...",
+            require_cipher="EDH-RSA-DES-CBC3-SHA",
         )
     '''
 
@@ -286,12 +282,11 @@ def user(
     # If we're here either the user exists or we just created them; either way
     # now we can check any privileges are set.
     if privileges:
-        yield _privileges(
+        yield from _privileges(
             user, privileges,
             user_hostname=user_hostname,
             mysql_user=mysql_user, mysql_password=mysql_password,
             mysql_host=mysql_host, mysql_port=mysql_port,
-            state=state, host=host,
         )
 
 
@@ -305,7 +300,6 @@ def database(
     # Details for speaking to MySQL via `mysql` CLI
     mysql_user=None, mysql_password=None,
     mysql_host=None, mysql_port=None,
-    state=None, host=None,
 ):
     '''
     Add/remove MySQL databases.
@@ -323,16 +317,16 @@ def database(
         these will only be applied if the database does not exist - ie pyinfra
         will not attempt to alter the existing databases collate/character sets.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         mysql.database(
-            name='Create the pyinfra_stuff database',
-            database='pyinfra_stuff',
-            user='pyinfra',
-            user_privileges=['SELECT', 'INSERT'],
-            charset='utf8',
+            name="Create the pyinfra_stuff database",
+            database="pyinfra_stuff",
+            user="pyinfra",
+            user_privileges=["SELECT", "INSERT"],
+            charset="utf8",
         )
     '''
 
@@ -355,6 +349,7 @@ def database(
                 host=mysql_host,
                 port=mysql_port,
             )
+            current_databases.pop(database)
         else:
             host.noop('mysql database {0} does not exist'.format(database))
         return
@@ -376,18 +371,20 @@ def database(
             host=mysql_host,
             port=mysql_port,
         )
+        current_databases[database] = {
+            'collate': collate,
+            'charset': charset,
+        }
     else:
         host.noop('mysql database {0} exists'.format(database))
 
     # Ensure any user privileges for this database
     if user and user_privileges:
-        yield privileges(
+        yield from privileges(
             user,
             user_hostname=user_hostname,
             privileges=user_privileges,
             database=database,
-            state=state,
-            host=host,
             mysql_user=mysql_user,
             mysql_password=mysql_password,
             mysql_host=mysql_host,
@@ -395,56 +392,39 @@ def database(
         )
 
 
-# TODO: make this behave like a proper state op in v2, by setting present=None as the default
-# and having that mode add/remove privileges to match the provided list. Retain True/False support
-# to ensure certain matches exist or not.
 @operation
 def privileges(
     user, privileges,
     user_hostname='localhost',
     database='*', table='*',
-    present=True,
     flush=True,
-    with_grant_option=None,
+    with_grant_option=False,
     # Details for speaking to MySQL via `mysql` CLI
     mysql_user=None, mysql_password=None,
     mysql_host=None, mysql_port=None,
-    state=None, host=None,
 ):
     '''
     Add/remove MySQL privileges for a user, either global, database or table specific.
 
     + user: name of the user to manage privileges for
-    + privileges: list of privileges the user should have
+    + privileges: list of privileges the user should have (see also: ``with_grant_option`` argument)
     + user_hostname: the hostname of the user
     + database: name of the database to grant privileges to (defaults to all)
     + table: name of the table to grant privileges to (defaults to all)
-    + present: whether these privileges should exist (False to ``REVOKE``)
     + flush: whether to flush (and update) the privileges table after any changes
-    + with_grant_option: whether to add the with grant option privilege
+    + with_grant_option: whether the grant option privilege should be set
     + mysql_*: global module arguments, see above
-
-    Note:
-        This operation will either ensure permissions exist or are removed for a given database
-        & table combination. This means when ``present=True`` it won't add/remove any permissions
-        that already exist but aren't passed in as ``privileges``.
     '''
 
-    logger.warning((
-        'Please note that the `present` argument in the `mysql.privileges` operation '
-        'will be removed in pyinfra v2, wich will grant/revoke privileges as required '
-        'to match the list in the `privileges` argument'
-    ))
-
     # Ensure we have a list
-    if isinstance(privileges, six.string_types):
-        privileges = [privileges]
+    if isinstance(privileges, str):
+        privileges = {privileges}
 
-    if (
-        (present and with_grant_option)
-        or (present is False and with_grant_option is False)
-    ):
-        privileges.append('GRANT OPTION')
+    if isinstance(privileges, list):
+        privileges = set(privileges)
+
+    if with_grant_option:
+        privileges.add('GRANT OPTION')
 
     if database != '*':
         database = '`{0}`'.format(database)
@@ -469,44 +449,22 @@ def privileges(
         mysql_port=mysql_port,
     )
 
-    existing_privileges = []
+    existing_privileges = set()
     if database_table in user_grants:
-        existing_privileges = [
+        existing_privileges = {
             'ALL' if privilege == 'ALL PRIVILEGES' else privilege
-            for privilege in user_grants[database_table]['privileges']
-        ]
+            for privilege in user_grants[database_table]
+        }
+    else:
+        user_grants[database_table] = set()
 
-    target = action = None
-
-    # No privilege and we want it
-    if present:
-        missing_privileges = [p for p in privileges if p not in existing_privileges]
-        if missing_privileges:
-            privileges_to_apply = missing_privileges
-            action = 'GRANT'
-            target = 'TO'
-        else:
-            host.noop('mysql privileges exist')
-            return
-
-    # Permission we don't want
-    if not present:
-        unwanted_privileges = [p for p in privileges if p in existing_privileges]
-        if unwanted_privileges:
-            privileges_to_apply = unwanted_privileges
-            action = 'REVOKE'
-            target = 'FROM'
-        else:
-            host.noop('mysql privileges do not exist')
-            return
-
-    if target and action:
+    def handle_privileges(action, target, privileges_to_apply):
         command = (
             '{action} {privileges} '
             'ON {database}.{table} '
             '{target} "{user}"@"{user_hostname}"'
         ).format(
-            privileges=', '.join(privileges_to_apply),
+            privileges=', '.join(sorted(privileges_to_apply)),
             action=action,
             target=target,
             database=database,
@@ -523,6 +481,21 @@ def privileges(
             port=mysql_port,
         )
 
+    # Find / revoke any privileges that exist that do not match the desired state
+    privileges_to_revoke = existing_privileges.difference(privileges)
+    # Find / grant any privileges that we want but do not exist
+    privileges_to_grant = privileges - existing_privileges
+
+    # No privilege and we want it
+    if privileges_to_grant:
+        yield from handle_privileges('GRANT', 'TO', privileges_to_grant)
+        user_grants[database_table].update(privileges_to_grant)
+
+    if privileges_to_revoke:
+        yield from handle_privileges('REVOKE', 'FROM', privileges_to_revoke)
+        user_grants[database_table] -= privileges_to_revoke
+
+    if privileges_to_grant or privileges_to_revoke:
         if flush:
             yield make_execute_mysql_command(
                 'FLUSH PRIVILEGES',
@@ -531,6 +504,8 @@ def privileges(
                 host=mysql_host,
                 port=mysql_port,
             )
+    else:
+        host.noop('mysql privileges are already correct')
 
 _privileges = privileges  # noqa: E305 (for use where kwarg is the same)
 
@@ -541,7 +516,6 @@ def dump(
     # Details for speaking to MySQL via `mysql` CLI
     mysql_user=None, mysql_password=None,
     mysql_host=None, mysql_port=None,
-    state=None, host=None,
 ):
     '''
     Dump a MySQL database into a ``.sql`` file. Requires ``mysqldump``.
@@ -550,14 +524,14 @@ def dump(
     + database: name of the database to dump
     + mysql_*: global module arguments, see above
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         mysql.dump(
-            name='Dump the pyinfra_stuff database',
-            dest='/tmp/pyinfra_stuff.dump',
-            database='pyinfra_stuff',
+            name="Dump the pyinfra_stuff database",
+            dest="/tmp/pyinfra_stuff.dump",
+            database="pyinfra_stuff",
         )
     '''
 
@@ -577,7 +551,6 @@ def load(
     # Details for speaking to MySQL via `mysql` CLI
     mysql_user=None, mysql_password=None,
     mysql_host=None, mysql_port=None,
-    state=None, host=None,
 ):
     '''
     Load ``.sql`` file into a database.
@@ -586,14 +559,14 @@ def load(
     + database: name of the database to import into
     + mysql_*: global module arguments, see above
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         mysql.load(
-            name='Import the pyinfra_stuff dump into pyinfra_stuff_copy',
-            src='/tmp/pyinfra_stuff.dump',
-            database='pyinfra_stuff_copy',
+            name="Import the pyinfra_stuff dump into pyinfra_stuff_copy",
+            src="/tmp/pyinfra_stuff.dump",
+            database="pyinfra_stuff_copy",
         )
     '''
 
