@@ -2,8 +2,7 @@
 Manage npm (aka node aka Node.js) packages.
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.npm import NpmPackages
 
@@ -11,7 +10,7 @@ from .util.packaging import ensure_packages
 
 
 @operation
-def packages(packages=None, present=True, latest=False, directory=None, state=None, host=None):
+def packages(packages=None, present=True, latest=False, directory=None):
     '''
     Install/remove/update npm packages.
 
@@ -44,7 +43,7 @@ def packages(packages=None, present=True, latest=False, directory=None, state=No
         else 'cd {0} && npm update'.format(directory)
     )
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, current_packages, present,
         install_command=install_command,
         uninstall_command=uninstall_command,
