@@ -2,8 +2,7 @@
 Manage OpenRC init services.
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.openrc import OpenrcEnabled, OpenrcStatus
 
@@ -16,7 +15,6 @@ def service(
     running=True, restarted=False, reloaded=False,
     command=None, enabled=None,
     runlevel='default',
-    state=None, host=None,
 ):
     '''
     Manage the state of OpenRC services.
@@ -30,7 +28,7 @@ def service(
     + runlevel: runlevel to manage services for
     '''
 
-    yield handle_service_control(
+    yield from handle_service_control(
         host,
         service,
         host.get_fact(OpenrcStatus, runlevel=runlevel),
