@@ -2,14 +2,15 @@
 Manage pacman packages. (Arch Linux package manager)
 '''
 
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.pacman import PacmanPackages, PacmanUnpackGroup
 
 from .util.packaging import ensure_packages
 
 
-@operation
-def upgrade(state=None, host=None):
+@operation(is_idempotent=False)
+def upgrade():
     '''
     Upgrades all pacman packages.
     '''
@@ -19,8 +20,8 @@ def upgrade(state=None, host=None):
 _upgrade = upgrade  # noqa: E305
 
 
-@operation
-def update(state=None, host=None):
+@operation(is_idempotent=False)
+def update():
     '''
     Updates pacman repositories.
     '''
@@ -34,7 +35,6 @@ _update = update  # noqa: E305
 def packages(
     packages=None, present=True,
     update=False, upgrade=False,
-    state=None, host=None,
 ):
     '''
     Add/remove pacman packages.
@@ -47,28 +47,27 @@ def packages(
     Versions:
         Package versions can be pinned like pacman: ``<pkg>=<version>``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         pacman.packages(
-            name='Install Vim and a plugin',
-            packages=['vim-fugitive', 'vim'],
+            name="Install Vim and a plugin",
+            packages=["vim-fugitive", "vim"],
             update=True,
         )
     '''
 
     if update:
-        yield _update(state=state, host=host)
+        yield from _update()
 
     if upgrade:
-        yield _upgrade(state=state, host=host)
+        yield from _upgrade()
 
-    yield ensure_packages(
+    yield from ensure_packages(
         host, packages, host.get_fact(PacmanPackages), present,
         install_command='pacman --noconfirm -S',
         uninstall_command='pacman --noconfirm -R',
-        lower=False,
         expand_package_fact=lambda package: host.get_fact(
             PacmanUnpackGroup,
             name=package,
