@@ -3,8 +3,7 @@ Manage pip (python) packages. Compatible globally or inside
 a virtualenv (virtual environment).
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.files import File
 from pyinfra.facts.pip import PipPackages
@@ -17,7 +16,6 @@ from .util.packaging import ensure_packages
 def virtualenv(
     path,
     python=None, venv=False, site_packages=False, always_copy=False, present=True,
-    state=None, host=None,
 ):
     '''
     Add/remove Python virtualenvs.
@@ -28,13 +26,13 @@ def virtualenv(
     + always_copy: always copy files rather than symlinking
     + present: whether the virtualenv should exist
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         pip.virtualenv(
-            name='Create a virtualenv',
-            path='/usr/local/bin/venv',
+            name="Create a virtualenv",
+            path="/usr/local/bin/venv",
         )
     '''
 
@@ -44,7 +42,7 @@ def virtualenv(
 
     if present is False:
         if host.get_fact(File, path=activate_script_path):
-            yield files.directory(path, present=False, state=state, host=host)
+            yield from files.directory(path, present=False)
         else:
             host.noop('virtualenv {0} does not exist'.format(path))
 
@@ -86,7 +84,6 @@ _virtualenv = virtualenv  # noqa
 def venv(
     path,
     python=None, site_packages=False, always_copy=False, present=True,
-    state=None, host=None,
 ):
     '''
     Add/remove Python virtualenvs.
@@ -96,25 +93,23 @@ def venv(
     + always_copy: always copy files rather than symlinking
     + present: whether the virtualenv should exist
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         pip.venv(
-            name='Create a virtualenv',
-            path='/usr/local/bin/venv',
+            name="Create a virtualenv",
+            path="/usr/local/bin/venv",
         )
     '''
 
-    yield virtualenv(
+    yield from virtualenv(
         venv=True,
         path=path,
         python=python,
         site_packages=site_packages,
         always_copy=always_copy,
         present=present,
-        state=state,
-        host=host,
     )
 
 
@@ -123,7 +118,6 @@ def packages(
     packages=None, present=True, latest=False,
     requirements=None, pip='pip', virtualenv=None, virtualenv_kwargs=None,
     extra_install_args=None,
-    state=None, host=None,
 ):
     '''
     Install/remove/update pip packages.
@@ -145,14 +139,14 @@ def packages(
     Versions:
         Package versions can be pinned like pip: ``<pkg>==<version>``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         pip.packages(
-            name='Install pyinfra into a virtualenv',
-            packages=['pyinfra'],
-            virtualenv='/usr/local/bin/venv',
+            name="Install pyinfra into a virtualenv",
+            packages=["pyinfra"],
+            virtualenv="/usr/local/bin/venv",
         )
     '''
 
@@ -160,7 +154,7 @@ def packages(
 
     # Ensure any virtualenv
     if virtualenv:
-        yield _virtualenv(virtualenv, state=state, host=host, **virtualenv_kwargs)
+        yield from _virtualenv(virtualenv, **virtualenv_kwargs)
 
         # And update pip path
         virtualenv = virtualenv.rstrip('/')
@@ -184,7 +178,7 @@ def packages(
     if packages:
         current_packages = host.get_fact(PipPackages, pip=pip)
 
-        yield ensure_packages(
+        yield from ensure_packages(
             host, packages, current_packages, present,
             install_command=install_command,
             uninstall_command=uninstall_command,
