@@ -3,16 +3,17 @@ The PostgreSQL modules manage PostgreSQL databases, users and privileges.
 
 Requires the ``psql`` CLI executable on the target host(s).
 
-All operations in this module take four optional global arguments:
-    + ``postgresql_user``: the username to connect to postgresql to
-    + ``postgresql_password``: the password for the connecting user
-    + ``postgresql_host``: the hostname of the server to connect to
-    + ``postgresql_port``: the port of the server to connect to
+All operations in this module take four optional arguments:
+    + ``psql_user``: the username to connect to postgresql to
+    + ``psql_password``: the password for the connecting user
+    + ``psql_host``: the hostname of the server to connect to
+    + ``psql_port``: the port of the server to connect to
 
 See example/postgresql.py for detailed example
 
 '''
 
+from pyinfra import host, logger
 from pyinfra.api import MaskString, operation, StringCommand
 from pyinfra.facts.postgresql import (
     make_execute_psql_command,
@@ -22,43 +23,65 @@ from pyinfra.facts.postgresql import (
 )
 
 
+LEGACY_ARG_MAP = {
+    'postgresql_user': 'psql_user',
+    'postgresql_password': 'psql_password',
+    'postgresql_host': 'psql_host',
+    'postgresql_port': 'psql_port',
+}
+
+
+def _translate_legacy_args(func):
+    def decorated_func(*args, **kwargs):
+        for legacy_key, key in LEGACY_ARG_MAP.items():
+            if legacy_key in kwargs:
+                kwargs[key] = kwargs.pop(legacy_key)
+                logger.warning((
+                    f'The `{legacy_key}` has been replaced '
+                    f'by `{key}` in `postgresql.*` operations.'
+                ))
+        return func(*args, **kwargs)
+    decorated_func._pyinfra_op = func
+    return decorated_func
+
+
 @operation(is_idempotent=False)
+@_translate_legacy_args
 def sql(
     sql,
     database=None,
     # Details for speaking to PostgreSQL via `psql` CLI
-    postgresql_user=None, postgresql_password=None,
-    postgresql_host=None, postgresql_port=None,
-    state=None, host=None,
+    psql_user=None, psql_password=None,
+    psql_host=None, psql_port=None,
 ):
     '''
     Execute arbitrary SQL against PostgreSQL.
 
     + sql: SQL command(s) to execute
     + database: optional database to execute against
-    + postgresql_*: global module arguments, see above
+    + psql_*: global module arguments, see above
     '''
 
     yield make_execute_psql_command(
         sql,
         database=database,
-        user=postgresql_user,
-        password=postgresql_password,
-        host=postgresql_host,
-        port=postgresql_port,
+        user=psql_user,
+        password=psql_password,
+        host=psql_host,
+        port=psql_port,
     )
 
 
 @operation
+@_translate_legacy_args
 def role(
     role,
     present=True,
     password=None, login=True, superuser=False, inherit=False,
     createdb=False, createrole=False, replication=False, connection_limit=None,
     # Details for speaking to PostgreSQL via `psql` CLI
-    postgresql_user=None, postgresql_password=None,
-    postgresql_host=None, postgresql_port=None,
-    state=None, host=None,
+    psql_user=None, psql_password=None,
+    psql_host=None, psql_port=None,
 ):
     '''
     Add/remove PostgreSQL roles.
@@ -73,34 +96,34 @@ def role(
     + createrole: whether the role is allowed to create new roles
     + replication: whether this role is allowed to replicate
     + connection_limit: the connection limit for the role
-    + postgresql_*: global module arguments, see above
+    + psql_*: global module arguments, see above
 
     Updates:
         pyinfra will not attempt to change existing roles - it will either
         create or drop roles, but not alter them (if the role exists this
         operation will make no changes).
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         postgresql.role(
-            name='Create the pyinfra PostgreSQL role',
-            role='pyinfra',
-            password='somepassword',
+            name="Create the pyinfra PostgreSQL role",
+            role="pyinfra",
+            password="somepassword",
             superuser=True,
             login=True,
-            sudo_user='postgres',
+            sudo_user="postgres",
         )
 
     '''
 
     roles = host.get_fact(
         PostgresqlRoles,
-        postgresql_user=postgresql_user,
-        postgresql_password=postgresql_password,
-        postgresql_host=postgresql_host,
-        postgresql_port=postgresql_port,
+        psql_user=psql_user,
+        psql_password=psql_password,
+        psql_host=psql_host,
+        psql_port=psql_port,
     )
 
     is_present = role in roles
@@ -110,11 +133,12 @@ def role(
         if is_present:
             yield make_execute_psql_command(
                 'DROP ROLE "{0}"'.format(role),
-                user=postgresql_user,
-                password=postgresql_password,
-                host=postgresql_host,
-                port=postgresql_port,
+                user=psql_user,
+                password=psql_password,
+                host=psql_host,
+                port=psql_port,
             )
+            roles.pop(role)
         else:
             host.noop('postgresql role {0} does not exist'.format(role))
         return
@@ -142,16 +166,22 @@ def role(
 
         yield make_execute_psql_command(
             StringCommand(*sql_bits),
-            user=postgresql_user,
-            password=postgresql_password,
-            host=postgresql_host,
-            port=postgresql_port,
+            user=psql_user,
+            password=psql_password,
+            host=psql_host,
+            port=psql_port,
         )
+        roles[role] = {
+            'super': superuser,
+            'cretedb': createdb,
+            'createrole': createrole,
+        }
     else:
         host.noop('postgresql role {0} exists'.format(role))
 
 
 @operation
+@_translate_legacy_args
 def database(
     database,
     present=True, owner=None,
@@ -159,9 +189,8 @@ def database(
     lc_collate=None, lc_ctype=None, tablespace=None,
     connection_limit=None,
     # Details for speaking to PostgreSQL via `psql` CLI
-    postgresql_user=None, postgresql_password=None,
-    postgresql_host=None, postgresql_port=None,
-    state=None, host=None,
+    psql_user=None, psql_password=None,
+    psql_host=None, psql_port=None,
 ):
     '''
     Add/remove PostgreSQL databases.
@@ -175,33 +204,33 @@ def database(
     + lc_ctype: lc_ctype of the database
     + tablespace: the tablespace to use for the template
     + connection_limit: the connection limit to apply to the database
-    + postgresql_*: global module arguments, see above
+    + psql_*: global module arguments, see above
 
     Updates:
         pyinfra will not attempt to change existing databases - it will either
         create or drop databases, but not alter them (if the db exists this
         operation will make no changes).
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         postgresql.database(
-            name='Create the pyinfra_stuff database',
-            database='pyinfra_stuff',
-            owner='pyinfra',
-            encoding='UTF8',
-            sudo_user='postgres',
+            name="Create the pyinfra_stuff database",
+            database="pyinfra_stuff",
+            owner="pyinfra",
+            encoding="UTF8",
+            sudo_user="postgres",
         )
 
     '''
 
     current_databases = host.get_fact(
         PostgresqlDatabases,
-        postgresql_user=postgresql_user,
-        postgresql_password=postgresql_password,
-        postgresql_host=postgresql_host,
-        postgresql_port=postgresql_port,
+        psql_user=psql_user,
+        psql_password=psql_password,
+        psql_host=psql_host,
+        psql_port=psql_port,
     )
 
     is_present = database in current_databases
@@ -210,11 +239,12 @@ def database(
         if is_present:
             yield make_execute_psql_command(
                 'DROP DATABASE "{0}"'.format(database),
-                user=postgresql_user,
-                password=postgresql_password,
-                host=postgresql_host,
-                port=postgresql_port,
+                user=psql_user,
+                password=psql_password,
+                host=psql_host,
+                port=psql_port,
             )
+            current_databases.pop(database)
         else:
             host.noop('postgresql database {0} does not exist'.format(database))
         return
@@ -237,39 +267,47 @@ def database(
 
         yield make_execute_psql_command(
             StringCommand(*sql_bits),
-            user=postgresql_user,
-            password=postgresql_password,
-            host=postgresql_host,
-            port=postgresql_port,
+            user=psql_user,
+            password=psql_password,
+            host=psql_host,
+            port=psql_port,
         )
+        current_databases[database] = {
+            'template': template,
+            'encoding': encoding,
+            'lc_collate': lc_collate,
+            'lc_ctype': lc_ctype,
+            'tablespace': tablespace,
+            'connection_limit': connection_limit,
+        }
     else:
         host.noop('postgresql database {0} exists'.format(database))
 
 
 @operation(is_idempotent=False)
+@_translate_legacy_args
 def dump(
     dest, database=None,
     # Details for speaking to PostgreSQL via `psql` CLI
-    postgresql_user=None, postgresql_password=None,
-    postgresql_host=None, postgresql_port=None,
-    state=None, host=None,
+    psql_user=None, psql_password=None,
+    psql_host=None, psql_port=None,
 ):
     '''
     Dump a PostgreSQL database into a ``.sql`` file. Requires ``pg_dump``.
 
     + dest: name of the file to dump the SQL to
     + database: name of the database to dump
-    + postgresql_*: global module arguments, see above
+    + psql_*: global module arguments, see above
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         postgresql.dump(
-            name='Dump the pyinfra_stuff database',
-            dest='/tmp/pyinfra_stuff.dump',
-            database='pyinfra_stuff',
-            sudo_user='postgres',
+            name="Dump the pyinfra_stuff database",
+            dest="/tmp/pyinfra_stuff.dump",
+            database="pyinfra_stuff",
+            sudo_user="postgres",
         )
 
     '''
@@ -277,45 +315,45 @@ def dump(
     yield StringCommand(make_psql_command(
         executable='pg_dump',
         database=database,
-        user=postgresql_user,
-        password=postgresql_password,
-        host=postgresql_host,
-        port=postgresql_port,
+        user=psql_user,
+        password=psql_password,
+        host=psql_host,
+        port=psql_port,
     ), '>', dest)
 
 
 @operation(is_idempotent=False)
+@_translate_legacy_args
 def load(
     src, database=None,
     # Details for speaking to PostgreSQL via `psql` CLI
-    postgresql_user=None, postgresql_password=None,
-    postgresql_host=None, postgresql_port=None,
-    state=None, host=None,
+    psql_user=None, psql_password=None,
+    psql_host=None, psql_port=None,
 ):
     '''
     Load ``.sql`` file into a database.
 
     + src: the filename to read from
     + database: name of the database to import into
-    + postgresql_*: global module arguments, see above
+    + psql_*: global module arguments, see above
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         postgresql.load(
-            name='Import the pyinfra_stuff dump into pyinfra_stuff_copy',
-            src='/tmp/pyinfra_stuff.dump',
-            database='pyinfra_stuff_copy',
-            sudo_user='postgres',
+            name="Import the pyinfra_stuff dump into pyinfra_stuff_copy",
+            src="/tmp/pyinfra_stuff.dump",
+            database="pyinfra_stuff_copy",
+            sudo_user="postgres",
         )
 
     '''
 
     yield StringCommand(make_psql_command(
         database=database,
-        user=postgresql_user,
-        password=postgresql_password,
-        host=postgresql_host,
-        port=postgresql_port,
+        user=psql_user,
+        password=psql_password,
+        host=psql_host,
+        port=psql_port,
     ), '<', src)
