@@ -3,19 +3,16 @@ The server module takes care of os-level state. Targets POSIX compatibility, tes
 Linux/BSD.
 '''
 
-from __future__ import division, unicode_literals
-
-from itertools import tee
+import shlex
+from io import StringIO
+from itertools import filterfalse, tee
 from os import path
 from time import sleep
 
-import six
-
-from six.moves import filterfalse, shlex_quote
-
+from pyinfra import host, state
 from pyinfra.api import FunctionCommand, operation, OperationError, StringCommand
-from pyinfra.api.connectors.util import remove_any_sudo_askpass_file
 from pyinfra.api.util import try_int
+from pyinfra.connectors.util import remove_any_sudo_askpass_file
 from pyinfra.facts.files import Directory
 from pyinfra.facts.server import (
     Crontab,
@@ -50,7 +47,7 @@ from .util.files import chmod, sed_replace
 
 
 @operation(is_idempotent=False)
-def reboot(delay=10, interval=1, reboot_timeout=300, state=None, host=None):
+def reboot(delay=10, interval=1, reboot_timeout=300):
     '''
     Reboot the server and wait for reconnection.
 
@@ -58,12 +55,12 @@ def reboot(delay=10, interval=1, reboot_timeout=300, state=None, host=None):
     + interval: interval (s) between reconnect attempts
     + reboot_timeout: total time before giving up reconnecting
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.reboot(
-            name='Reboot the server and wait to reconnect',
+            name="Reboot the server and wait to reconnect",
             delay=60,
             reboot_timeout=600,
         )
@@ -103,19 +100,19 @@ def reboot(delay=10, interval=1, reboot_timeout=300, state=None, host=None):
 
 
 @operation(is_idempotent=False)
-def wait(port=None, state=None, host=None):
+def wait(port=None):
     '''
     Waits for a port to come active on the target machine. Requires netstat, checks every
     second.
 
     + port: port number to wait for
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.wait(
-            name='Wait for webserver to start',
+            name="Wait for webserver to start",
             port=80,
         )
     '''
@@ -129,7 +126,7 @@ def wait(port=None, state=None, host=None):
 
 
 @operation(is_idempotent=False)
-def shell(commands, state=None, host=None):
+def shell(commands):
     '''
     Run raw shell code on server during a deploy. If the command would
     modify data that would be in a fact, the fact would not be updated
@@ -137,18 +134,18 @@ def shell(commands, state=None, host=None):
 
     + commands: command or list of commands to execute on the remote server
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.shell(
-            name='Run lxd auto init',
-            commands=['lxd init --auto'],
+            name="Run lxd auto init",
+            commands=["lxd init --auto"],
         )
     '''
 
     # Ensure we have a list
-    if isinstance(commands, six.string_types):
+    if isinstance(commands, str):
         commands = [commands]
 
     for command in commands:
@@ -156,38 +153,38 @@ def shell(commands, state=None, host=None):
 
 
 @operation(is_idempotent=False)
-def script(src, state=None, host=None):
+def script(src):
     '''
     Upload and execute a local script on the remote host.
 
     + src: local script filename to upload & execute
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # Note: This assumes there is a file in files/hello.bash locally.
         server.script(
-            name='Hello',
-            src='files/hello.bash',
+            name="Hello",
+            src="files/hello.bash",
         )
     '''
 
     temp_file = state.get_temp_filename(src)
-    yield files.put(src, temp_file, state=state, host=host)
+    yield from files.put(src, temp_file)
 
     yield chmod(temp_file, '+x')
     yield temp_file
 
 
 @operation(is_idempotent=False)
-def script_template(src, state=None, host=None, **data):
+def script_template(src, **data):
     '''
     Generate, upload and execute a local script template on the remote host.
 
     + src: local script template filename
 
-    Example:
+    **Example:**
 
     .. code:: python
 
@@ -197,21 +194,21 @@ def script_template(src, state=None, host=None, **data):
         # Note: This assumes there is a file in templates/hello2.bash.j2 locally.
         some_var = 'blah blah blah '
         server.script_template(
-            name='Hello from script',
-            src='templates/hello2.bash.j2',
+            name="Hello from script",
+            src="templates/hello2.bash.j2",
             some_var=some_var,
         )
     '''
 
     temp_file = state.get_temp_filename('{0}{1}'.format(src, data))
-    yield files.template(src, temp_file, state=state, host=host, **data)
+    yield from files.template(src, temp_file, **data)
 
     yield chmod(temp_file, '+x')
     yield temp_file
 
 
 @operation
-def modprobe(module, present=True, force=False, state=None, host=None):
+def modprobe(module, present=True, force=False):
     '''
     Load/unload kernel modules.
 
@@ -219,18 +216,18 @@ def modprobe(module, present=True, force=False, state=None, host=None):
     + present: whether the module should be loaded or not
     + force: whether to force any add/remove modules
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.modprobe(
-            name='Silly example for modprobe',
-            module='floppy',
+            name="Silly example for modprobe",
+            module="floppy",
         )
     '''
     list_value = (
         [module]
-        if isinstance(module, six.string_types)
+        if isinstance(module, str)
         else module
     )
 
@@ -273,7 +270,6 @@ def mount(
     mounted=True, options=None,
     # TODO: do we want to manage fstab here?
     # update_fstab=False, device=None, fs_type=None,
-    state=None, host=None,
 ):
     '''
     Manage mounted filesystems.
@@ -326,7 +322,7 @@ def mount(
 
 
 @operation
-def hostname(hostname, hostname_file=None, state=None, host=None):
+def hostname(hostname, hostname_file=None):
     '''
     Set the system hostname using ``hostnamectl`` or ``hostname`` on older systems.
 
@@ -342,13 +338,13 @@ def hostname(hostname, hostname_file=None, state=None, host=None):
 
         To completely disable writing the hostname file, set ``hostname_file=False``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.hostname(
-            name='Set the hostname',
-            hostname='server1.example.com',
+            name="Set the hostname",
+            hostname="server1.example.com",
         )
     '''
 
@@ -357,6 +353,7 @@ def hostname(hostname, hostname_file=None, state=None, host=None):
     if host.get_fact(Which, command='hostnamectl'):
         if current_hostname != hostname:
             yield 'hostnamectl set-hostname {0}'.format(hostname)
+            host.create_fact(Hostname, data=hostname)
         else:
             host.noop('hostname is set')
         return
@@ -371,26 +368,22 @@ def hostname(hostname, hostname_file=None, state=None, host=None):
 
     if current_hostname != hostname:
         yield 'hostname {0}'.format(hostname)
+        host.create_fact(Hostname, data=hostname)
     else:
         host.noop('hostname is set')
 
     if hostname_file:
         # Create a whole new hostname file
-        file = six.StringIO('{0}\n'.format(hostname))
+        file = StringIO('{0}\n'.format(hostname))
 
         # And ensure it exists
-        yield files.put(
-            file, hostname_file,
-            state=state,
-            host=host,
-        )
+        yield from files.put(file, hostname_file)
 
 
 @operation
 def sysctl(
     key, value,
     persist=False, persist_file='/etc/sysctl.conf',
-    state=None, host=None,
 ):
     '''
     Edit sysctl configuration.
@@ -400,13 +393,13 @@ def sysctl(
     + persist: whether to write this sysctl to the config
     + persist_file: file to write the sysctl to persist on reboot
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.sysctl(
-            name='Change the fs.file-max value',
-            key='fs.file-max',
+            name="Change the fs.file-max value",
+            key="fs.file-max",
             value=100000,
             persist=True,
         )
@@ -434,12 +427,10 @@ def sysctl(
         host.noop('sysctl {0} is set to {1}'.format(key, string_value))
 
     if persist:
-        yield files.line(
+        yield from files.line(
             path=persist_file,
             line='{0}[[:space:]]*=[[:space:]]*{1}'.format(key, string_value),
             replace='{0} = {1}'.format(key, string_value),
-            state=state,
-            host=host,
         )
 
 
@@ -448,7 +439,6 @@ def service(
     service,
     running=True, restarted=False, reloaded=False,
     command=None, enabled=None,
-    state=None, host=None,
 ):
     '''
     Manage the state of services. This command checks for the presence of all the
@@ -461,13 +451,13 @@ def service(
     + command: custom command execute
     + enabled: whether this service should be enabled/disabled on boot
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.service(
-            name='Enable open-vm-tools service',
-            service='open-vm-tools',
+            name="Enable open-vm-tools service",
+            service="open-vm-tools",
             enabled=True,
         )
     '''
@@ -493,18 +483,16 @@ def service(
             '(no systemctl, initctl, /etc/init.d or /etc/rc.d found)'
         ))
 
-    yield service_operation(
+    yield from service_operation(
         service,
         running=running, restarted=restarted, reloaded=reloaded,
         command=command, enabled=enabled,
-        state=state, host=host,
     )
 
 
 @operation
 def packages(
     packages, present=True,
-    state=None, host=None,
 ):
     '''
     Add or remove system packages. This command checks for the presence of all the
@@ -513,13 +501,13 @@ def packages(
     + packages: list of packages to ensure
     + present: whether the packages should be installed
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         server.packages(
-            name='Install Vim and vimpager',
-            packages=['vimpager', 'vim'],
+            name="Install Vim and vimpager",
+            packages=["vimpager", "vim"],
         )
     '''
 
@@ -556,7 +544,7 @@ def packages(
             '(no apk, apt, brew, dnf, pacman, pkg, xbps, yum or zypper found)'
         ))
 
-    yield package_operation(packages=packages, present=present, state=state, host=host)
+    yield from package_operation(packages=packages, present=present)
 
 
 @operation
@@ -572,7 +560,6 @@ def crontab(
     day_of_month='*',
     special_time=None,
     interpolate_variables=False,
-    state=None, host=None,
 ):
     '''
     Add/remove/update crontab entries.
@@ -598,15 +585,15 @@ def crontab(
         When provided, ``special_time`` will be used instead of any values passed in
         for ``minute``/``hour``/``month``/``day_of_week``/``day_of_month``.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         # simple example for a crontab
         server.crontab(
-            name='Backup /etc weekly',
-            command='/bin/tar cf /tmp/etc_bup.tar /etc',
-            name='backup_etc',
+            name="Backup /etc weekly",
+            command="/bin/tar cf /tmp/etc_bup.tar /etc",
+            name="backup_etc",
             day_of_week=0,
             hour=1,
             minute=0,
@@ -670,11 +657,11 @@ def crontab(
             if crontab:  # append a blank line if cron entries already exist
                 edit_commands.append("echo '' >> {0}".format(temp_filename))
             edit_commands.append('echo {0} >> {1}'.format(
-                shlex_quote(name_comment), temp_filename,
+                shlex.quote(name_comment), temp_filename,
             ))
 
         edit_commands.append('echo {0} >> {1}'.format(
-            shlex_quote(new_crontab_line), temp_filename,
+            shlex.quote(new_crontab_line), temp_filename,
         ))
 
     # We have the cron and it exists, do it's details? If not, replace the line
@@ -730,7 +717,7 @@ def crontab(
 
 
 @operation
-def group(group, present=True, system=False, gid=None, state=None, host=None):
+def group(group, present=True, system=False, gid=None):
     '''
     Add/remove system groups.
 
@@ -741,19 +728,19 @@ def group(group, present=True, system=False, gid=None, state=None, host=None):
     System users:
         System users don't exist on BSD, so the argument is ignored for BSD targets.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         server.group(
-            name='Create docker group',
-            group='docker',
+            name="Create docker group",
+            group="docker",
         )
 
         # multiple groups
-        for group in ['wheel', 'lusers']:
+        for group in ["wheel", "lusers"]:
             server.group(
-                name=f'Create the group {group}',
+                name=f"Create the group {group}",
                 group=group,
             )
     '''
@@ -795,7 +782,6 @@ def user(
     public_keys=None, delete_keys=False, ensure_home=True,
     system=False, uid=None, comment=None, add_deploy_dir=True,
     unique=True,
-    state=None, host=None,
 ):
     '''
     Add/remove/update system users & their ssh `authorized_keys`.
@@ -822,26 +808,26 @@ def user(
         These can be provided as strings containing the public key or as a path to
         a public key file which ``pyinfra`` will read.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         server.user(
-            name='Ensure user is removed',
-            user='kevin',
+            name="Ensure user is removed",
+            user="kevin",
             present=False,
         )
 
         server.user(
-            name='Ensure myweb user exists',
-            user='myweb',
-            shell='/bin/bash',
+            name="Ensure myweb user exists",
+            user="myweb",
+            shell="/bin/bash",
         )
 
         # multiple users
-        for user in ['kevin', 'bob']:
+        for user in ["kevin", "bob"]:
             server.user(
-                name=f'Ensure user {user} is removed',
+                name=f"Ensure user {user} is removed",
                 user=user,
                 present=False,
             )
@@ -945,21 +931,20 @@ def user(
 
     # Ensure home directory ownership
     if ensure_home:
-        yield files.directory(
+        yield from files.directory(
             home,
             user=user, group=group or user,
-            state=state, host=host,
         )
 
     # Add SSH keys
     if public_keys is not None:
-        if isinstance(public_keys, six.string_types):
+        if isinstance(public_keys, str):
             public_keys = [public_keys]
 
         def read_any_pub_key_file(key):
             try_path = key
-            if add_deploy_dir and state.deploy_dir:
-                try_path = path.join(state.deploy_dir, key)
+            if add_deploy_dir and state.cwd:
+                try_path = path.join(state.cwd, key)
 
             if path.exists(try_path):
                 with open(try_path, 'r') as f:
@@ -972,45 +957,39 @@ def user(
         # Ensure .ssh directory
         # note that this always outputs commands unless the SSH user has access to the
         # authorized_keys file, ie the SSH user is the user defined in this function
-        yield files.directory(
+        yield from files.directory(
             '{0}/.ssh'.format(home),
             user=user,
             group=group or user,
             mode=700,
-            state=state,
-            host=host,
         )
 
         filename = '{0}/.ssh/authorized_keys'.format(home)
 
         if delete_keys:
             # Create a whole new authorized_keys file
-            keys_file = six.StringIO('{0}\n'.format(
+            keys_file = StringIO('{0}\n'.format(
                 '\n'.join(public_keys),
             ))
 
             # And ensure it exists
-            yield files.put(
+            yield from files.put(
                 src=keys_file,
                 dest=filename,
                 user=user,
                 group=group or user,
                 mode=600,
-                state=state,
-                host=host,
             )
 
         else:
             # Ensure authorized_keys exists
-            yield files.file(
+            yield from files.file(
                 path=filename,
                 user=user,
                 group=group or user,
                 mode=600,
-                state=state,
-                host=host,
             )
 
             # And every public key is present
             for key in public_keys:
-                yield files.line(path=filename, line=key, state=state, host=host)
+                yield from files.line(path=filename, line=key)
