@@ -4,9 +4,9 @@ Execute commands and up/download files *from* the remote host.
 Eg: ``pyinfra -> inventory-host.net <-> another-host.net``
 '''
 
-from six.moves import shlex_quote
+import shlex
 
-from pyinfra import logger
+from pyinfra import host, state
 from pyinfra.api import operation, OperationError
 from pyinfra.facts.files import File, FindInFile
 from pyinfra.facts.server import Home
@@ -15,30 +15,28 @@ from . import files
 
 
 @operation
-def keyscan(hostname, force=False, port=22, state=None, host=None):
+def keyscan(hostname, force=False, port=22):
     '''
     Check/add hosts to the ``~/.ssh/known_hosts`` file.
 
     + hostname: hostname that should have a key in ``known_hosts``
     + force: if the key already exists, remove and rescan
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         ssh.keyscan(
-            name='Set add server two to known_hosts on one',
-            hostname='two.example.com',
+            name="Set add server two to known_hosts on one",
+            hostname="two.example.com",
         )
     '''
 
     homedir = host.get_fact(Home)
 
-    yield files.directory(
+    yield from files.directory(
         '{0}/.ssh'.format(homedir),
         mode=700,
-        state=state,
-        host=host,
     )
 
     hostname_present = host.get_fact(
@@ -72,17 +70,8 @@ def keyscan(hostname, force=False, port=22, state=None, host=None):
         )
 
 
-def _user_or_ssh_user(user, ssh_user):
-    if ssh_user:
-        logger.warning((
-            'The `ssh_user` argument is deprecated in `ssh.*` operations, '
-            'please use the `user` argument.'
-        ))
-    return user or ssh_user
-
-
 @operation(is_idempotent=False)
-def command(hostname, command, user=None, port=22, ssh_user=None, state=None, host=None):
+def command(hostname, command, user=None, port=22):
     '''
     Execute commands on other servers over SSH.
 
@@ -91,22 +80,19 @@ def command(hostname, command, user=None, port=22, ssh_user=None, state=None, ho
     + user: connect with this user
     + port: connect to this port
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         ssh.command(
-            name='Create file by running echo from host one to host two',
-            hostname='two.example.com',
-            command='echo "one was here" > /tmp/one.txt',
-            user='vagrant',
+            name="Create file by running echo from host one to host two",
+            hostname="two.example.com",
+            command="echo "one was here" > /tmp/one.txt",
+            user="vagrant",
         )
     '''
 
-    # TODO: remove this (ssh_user is a legacy arg)
-    user = _user_or_ssh_user(user, ssh_user)
-
-    command = shlex_quote(command)
+    command = shlex.quote(command)
 
     connection_target = hostname
     if user:
@@ -123,8 +109,6 @@ def upload(
     user=None,
     use_remote_sudo=False,
     ssh_keyscan=False,
-    ssh_user=None,
-    state=None, host=None,
 ):
     '''
     Upload files to other servers using ``scp``.
@@ -140,16 +124,13 @@ def upload(
 
     remote_filename = remote_filename or filename
 
-    # TODO: remove this (ssh_user is a legacy arg)
-    user = _user_or_ssh_user(user, ssh_user)
-
     # Figure out where we're connecting (host or user@host)
     connection_target = hostname
     if user:
         connection_target = '@'.join((user, hostname))
 
     if ssh_keyscan:
-        yield keyscan(hostname, state=state, host=host)
+        yield from keyscan(hostname)
 
     # If we're not using sudo on the remote side, just scp the file over
     if not use_remote_sudo:
@@ -169,13 +150,11 @@ def upload(
         yield upload_cmd
 
         # And sudo sudo to move it
-        yield command(
+        yield from command(
             hostname=hostname,
             command='sudo mv {0} {1}'.format(temp_remote_filename, remote_filename),
             port=port,
             user=user,
-            state=state,
-            host=host,
         )
 
 
@@ -187,8 +166,6 @@ def download(
     port=22,
     user=None,
     ssh_keyscan=False,
-    ssh_user=None,
-    state=None, host=None,
 ):
     '''
     Download files from other servers using ``scp``.
@@ -204,9 +181,6 @@ def download(
 
     local_filename = local_filename or filename
 
-    # TODO: remove this (ssh_user is a legacy arg)
-    user = _user_or_ssh_user(user, ssh_user)
-
     # Get local file info
     local_file_info = host.get_fact(File, path=local_filename)
 
@@ -225,13 +199,18 @@ def download(
 
     # Figure out where we're connecting (host or user@host)
     connection_target = hostname
-    if ssh_user:
-        connection_target = '@'.join((ssh_user, hostname))
+    if user:
+        connection_target = '@'.join((user, hostname))
 
     if ssh_keyscan:
-        yield keyscan(hostname, state=state, host=host)
+        yield from keyscan(hostname)
 
     # Download the file with scp
     yield 'scp -P {0} {1}:{2} {3}'.format(
         port, connection_target, filename, local_filename,
     )
+    host.create_fact(
+        File,
+        kwargs={'path': local_filename},
+        data={'mode': None, 'group': None, 'user': user, 'mtime': None},
+    )
