@@ -2,18 +2,15 @@
 Manage systemd services.
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.systemd import _make_systemctl_cmd, SystemdEnabled, SystemdStatus
 
 from .util.service import handle_service_control
 
 
-@operation
-def daemon_reload(
-    user_mode=False, machine=None, user_name=None, state=None, host=None,
-):
+@operation(is_idempotent=False)
+def daemon_reload(user_mode=False, machine=None, user_name=None):
     '''
     Reload the systemd daemon to read unit file changes.
 
@@ -40,7 +37,6 @@ def service(
     running=True, restarted=False, reloaded=False,
     command=None, enabled=None, daemon_reload=False,
     user_mode=False, machine=None, user_name=None,
-    state=None, host=None,
 ):
     '''
     Manage the state of systemd managed units.
@@ -56,21 +52,21 @@ def service(
     + machine: the machine name to connect to
     + user_name: connect to a specific user's systemd session
 
-    Example:
+    **Examples:**
 
     .. code:: python
 
         systemd.service(
-            name='Restart and enable the dnsmasq service',
-            service='dnsmasq.service',
+            name="Restart and enable the dnsmasq service",
+            service="dnsmasq.service",
             running=True,
             restarted=True,
             enabled=True,
         )
 
         systemd.service(
-            name='Enable logrotate timer',
-            service='logrotate.timer',
+            name="Enable logrotate timer",
+            service="logrotate.timer",
             running=True,
             enabled=True,
         )
@@ -87,15 +83,13 @@ def service(
         service = '{0}.service'.format(service)
 
     if daemon_reload:
-        yield _daemon_reload(
+        yield from _daemon_reload(
             user_mode=user_mode,
             machine=machine,
             user_name=user_name,
-            state=state,
-            host=host,
         )
 
-    yield handle_service_control(
+    yield from handle_service_control(
         host,
         service, host.get_fact(
             SystemdStatus,
