@@ -2,8 +2,7 @@
 Manage sysvinit services (``/etc/init.d``).
 '''
 
-from __future__ import unicode_literals
-
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.files import FindLinks
 from pyinfra.facts.server import LinuxDistribution
@@ -18,7 +17,6 @@ def service(
     service,
     running=True, restarted=False, reloaded=False,
     enabled=None, command=None,
-    state=None, host=None,
 ):
     '''
     Manage the state of SysV Init (/etc/init.d) services.
@@ -41,19 +39,19 @@ def service(
         For other distributions and more granular service control, see the
         ``sysvinit.enable`` operation.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         sysvinit.service(
-            name='Restart and enable rsyslog',
-            service='rsyslog',
+            name="Restart and enable rsyslog",
+            service="rsyslog",
             restarted=True,
             enabled=True,
         )
     '''
 
-    yield handle_service_control(
+    yield from handle_service_control(
         host,
         service, host.get_fact(InitdStatus),
         '/etc/init.d/{0} {1}',
@@ -102,7 +100,6 @@ def enable(
     service,
     start_priority=20, stop_priority=80,
     start_levels=(2, 3, 4, 5), stop_levels=(0, 1, 6),
-    state=None, host=None,
 ):
     '''
     Manually enable /etc/init.d scripts by creating /etc/rcX.d/Y links.
@@ -113,13 +110,13 @@ def enable(
     + start_levels: which runlevels should the service run when enabled
     + stop_levels: which runlevels should the service stop when enabled
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         init.d_enable(
-            name='Finer control on which runlevels rsyslog should run',
-            service='rsyslog',
+            name="Finer control on which runlevels rsyslog should run",
+            service="rsyslog",
             start_levels=(3, 4, 5),
             stop_levels=(0, 1, 2, 6),
         )
@@ -136,8 +133,7 @@ def enable(
 
     # Ensure all the new links exist
     for link in links:
-        yield files.link(
+        yield from files.link(
             path=link,
             target='/etc/init.d/{0}'.format(service),
-            state=state, host=host,
         )
