@@ -2,10 +2,9 @@
 Manage upstart services.
 '''
 
-from __future__ import unicode_literals
-
-import six
+from io import StringIO
 
+from pyinfra import host
 from pyinfra.api import operation
 from pyinfra.facts.upstart import UpstartStatus
 
@@ -18,7 +17,6 @@ def service(
     service,
     running=True, restarted=False, reloaded=False,
     command=None, enabled=None,
-    state=None, host=None,
 ):
     '''
     Manage the state of upstart managed services.
@@ -37,7 +35,7 @@ def service(
         "manual" to disable automatic start of services.
     '''
 
-    yield handle_service_control(
+    yield from handle_service_control(
         host,
         service, host.get_fact(UpstartStatus),
         'initctl {1} {0}',
@@ -47,18 +45,15 @@ def service(
     # Upstart jobs are setup w/runlevels etc in their config files, so here we just check
     # there's no override file.
     if enabled is True:
-        yield files.file(
+        yield from files.file(
             '/etc/init/{0}.override'.format(service),
             present=False,
-            state=state, host=host,
         )
 
     # Set the override file to "manual" to disable automatic start
     elif enabled is False:
-        file = six.StringIO('manual\n')
-        yield files.put(
+        file = StringIO('manual\n')
+        yield from files.put(
             src=file,
             dest='/etc/init/{0}.override'.format(service),
-            state=state,
-            host=host,
         )
