@@ -1,10 +1,6 @@
-from __future__ import unicode_literals
-
-import six
-
-from six import StringIO
-from six.moves import shlex_quote
-from six.moves.urllib.parse import urlparse
+import shlex
+from io import StringIO
+from urllib.parse import urlparse
 
 from pyinfra.facts.files import File
 from pyinfra.facts.rpm import RpmPackage
@@ -30,7 +26,7 @@ def ensure_packages(
     host, packages, current_packages, present,
     install_command, uninstall_command,
     latest=False, upgrade_command=None,
-    version_join=None, lower=True,
+    version_join=None,
     expand_package_fact=None,
 ):
     '''
@@ -52,23 +48,15 @@ def ensure_packages(
         upgrade_command (str): as above for upgrading
         version_join (str): the package manager specific "joiner", ie ``=`` for \
             ``<apt_pkg>=<version>``
-        lower (bool): whether to lowercase package names
     '''
 
     if packages is None:
         return
 
     # Accept a single package as string
-    if isinstance(packages, six.string_types):
+    if isinstance(packages, str):
         packages = [packages]
 
-    # Lowercase packaging?
-    if lower:
-        packages = [
-            package.lower()
-            for package in packages
-        ]
-
     # Version support?
     if version_join:
         # Split where versions present
@@ -99,7 +87,7 @@ def ensure_packages(
 
             else:
                 # Present packages w/o version specified - for upgrade if latest
-                if isinstance(package, six.string_types):
+                if isinstance(package, str):
                     upgrade_packages.append(package)
 
                 if not latest:
@@ -134,7 +122,7 @@ def ensure_packages(
 
         yield '{0} {1}'.format(
             command,
-            ' '.join([shlex_quote(pkg) for pkg in diff_packages]),
+            ' '.join([shlex.quote(pkg) for pkg in diff_packages]),
         )
 
         for package in diff_packages:  # add/remove from current packages
@@ -153,7 +141,7 @@ def ensure_packages(
     if latest and upgrade_command and upgrade_packages:
         yield '{0} {1}'.format(
             upgrade_command,
-            ' '.join([shlex_quote(pkg) for pkg in upgrade_packages]),
+            ' '.join([shlex.quote(pkg) for pkg in upgrade_packages]),
         )
 
 
@@ -166,7 +154,7 @@ def ensure_rpm(state, host, files, source, present, package_manager_command):
         temp_filename = '{0}.rpm'.format(state.get_temp_filename(source))
 
         # Ensure it's downloaded
-        yield files.download(source, temp_filename, state=state, host=host)
+        yield from files.download(source, temp_filename)
 
         # Override the source with the downloaded file
         source = temp_filename
@@ -224,13 +212,13 @@ def ensure_yum_repo(
 
     # If we don't want the repo, just remove any existing file
     if not present:
-        yield files.file(filename, present=False, state=state, host=host)
+        yield from files.file(filename, present=False)
         return
 
     # If we're a URL, download the repo if it doesn't exist
     if url:
         if not host.get_fact(File, path=filename):
-            yield files.download(url, filename, state=state, host=host)
+            yield from files.download(url, filename)
         return
 
     # Description defaults to name
@@ -256,4 +244,4 @@ def ensure_yum_repo(
     repo = StringIO(repo)
 
     # Ensure this is the file on the server
-    yield files.put(repo, filename, state=state, host=host)
+    yield from files.put(repo, filename)
