@@ -2,8 +2,6 @@
 The windows module handles misc windows operations.
 '''
 
-from __future__ import unicode_literals
-
 from pyinfra.api import operation
 
 # Tip: Use 'Get-Command -Noun Service' to search for what commands are available or
@@ -20,7 +18,7 @@ from pyinfra.api import operation
 
 
 @operation
-def service(service, running=True, restart=False, suspend=False, state=None, host=None):
+def service(service, running=True, restart=False, suspend=False):
     '''
     Stop/Start a Windows service.
 
@@ -29,13 +27,13 @@ def service(service, running=True, restart=False, suspend=False, state=None, hos
     + restart: whether the the service should be restarted
     + suspend: whether the the service should be suspended
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         windows.service(
-            {'Stop the spooler service'},
-            'service',
+            name="Stop the spooler service",
+            service="service",
             running=False,
         )
     '''
@@ -53,8 +51,8 @@ def service(service, running=True, restart=False, suspend=False, state=None, hos
                 yield 'Start-Service -Name {0}'.format(service)
 
 
-@operation
-def reboot(state=None, host=None):
+@operation(is_idempotent=False)
+def reboot():
     '''
     Restart the server.
     '''
