@@ -2,15 +2,11 @@
 The windows_files module handles windows filesystem state, file uploads and template generation.
 '''
 
-from __future__ import unicode_literals
-
 import ntpath
 import os
 from datetime import timedelta
 
-import six
-
-from pyinfra import logger
+from pyinfra import host, state
 from pyinfra.api import (
     FileUploadCommand,
     operation,
@@ -18,17 +14,16 @@ from pyinfra.api import (
     OperationTypeError,
 )
 from pyinfra.api.util import get_file_sha1
-from pyinfra.facts.windows import WindowsDate
+from pyinfra.facts.windows import Date
 from pyinfra.facts.windows_files import (
-    WindowsDirectory,
-    WindowsFile,
-    WindowsLink,
-    WindowsMd5File,
-    WindowsSha1File,
-    WindowsSha256File,
+    Directory,
+    File,
+    Link,
+    Md5File,
+    Sha1File,
+    Sha256File,
 )
 
-from .util.compat import fspath
 from .util.files import ensure_mode_int
 
 
@@ -39,7 +34,6 @@ def download(
     src, dest,
     user=None, group=None, mode=None, cache_time=None, force=False,
     sha256sum=None, sha1sum=None, md5sum=None,
-    state=None, host=None,
 ):
     '''
     Download files from remote locations using curl or wget.
@@ -55,18 +49,18 @@ def download(
     + sha1sum: sha1 hash to checksum the downloaded file against
     + md5sum: md5 hash to checksum the downloaded file against
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         winows_files.download(
-            name='Download the Docker repo file',
-            src='https://download.docker.com/linux/centos/docker-ce.repo',
-            dest='C:\\docker',
+            name="Download the Docker repo file",
+            src="https://download.docker.com/linux/centos/docker-ce.repo",
+            dest="C:\\docker",
         )
     '''
 
-    info = host.get_fact(WindowsFile, name=dest)
+    info = host.get_fact(File, path=dest)
     # Destination is a directory?
     if info is False:
         raise OperationError(
@@ -85,23 +79,23 @@ def download(
     else:
         if cache_time:
             # Time on files is not tz-aware, and will be the same tz as the server's time,
-            # so we can safely remove the tzinfo from WindowsDate before comparison.
+            # so we can safely remove the tzinfo from Date before comparison.
             cache_time = (
-                host.get_fact(WindowsDate).replace(tzinfo=None) - timedelta(seconds=cache_time)
+                host.get_fact(Date).replace(tzinfo=None) - timedelta(seconds=cache_time)
             )
             if info['mtime'] and info['mtime'] > cache_time:
                 download = True
 
         if sha1sum:
-            if sha1sum != host.get_fact(WindowsSha1File, name=dest):
+            if sha1sum != host.get_fact(Sha1File, path=dest):
                 download = True
 
         if sha256sum:
-            if sha256sum != host.get_fact(WindowsSha256File, name=dest):
+            if sha256sum != host.get_fact(Sha256File, path=dest):
                 download = True
 
         if md5sum:
-            if md5sum != host.get_fact(WindowsMd5File, name=dest):
+            if md5sum != host.get_fact(Md5File, path=dest):
                 download = True
 
     # If we download, always do user/group/mode as SSH user may be different
@@ -150,7 +144,6 @@ def put(
     src, dest,
     user=None, group=None, mode=None, add_deploy_dir=True,
     create_remote_dir=True, force=False, assume_exists=False,
-    state=None, host=None,
 ):
     '''
     Upload a local file to the remote system.
@@ -174,15 +167,15 @@ def put(
         This operation is not suitable for large files as it may involve copying
         the file before uploading it.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # Note: This requires a 'files/motd' file on the local filesystem
         files.put(
-            name='Update the message of the day file',
-            src='data/content.json',
-            dest='C:\\data\\content.json'
+            name="Update the message of the day file",
+            src="data/content.json",
+            dest="C:\\data\\content.json",
         )
     '''
 
@@ -193,8 +186,8 @@ def put(
     # Assume string filename
     else:
         # Add deploy directory?
-        if add_deploy_dir and state.deploy_dir:
-            src = os.path.join(state.deploy_dir, src)
+        if add_deploy_dir and state.cwd:
+            src = os.path.join(state.cwd, src)
 
         local_file = src
 
@@ -202,10 +195,10 @@ def put(
             raise IOError('No such file: {0}'.format(local_file))
 
     mode = ensure_mode_int(mode)
-    remote_file = host.get_fact(WindowsFile, name=dest)
+    remote_file = host.get_fact(File, path=dest)
 
     if create_remote_dir:
-        yield _create_remote_dir(state, host, dest, user, group)
+        yield from _create_remote_dir(state, host, dest, user, group)
 
     # No remote file, always upload and user/group/mode if supplied
     if not remote_file or force:
@@ -224,7 +217,7 @@ def put(
     # File exists, check sum and check user/group/mode if supplied
     else:
         local_sum = get_file_sha1(src)
-        remote_sum = host.get_fact(WindowsSha1File, name=dest)
+        remote_sum = host.get_fact(Sha1File, path=dest)
 
         # Check sha1sum, upload if needed
         if local_sum != remote_sum:
@@ -268,7 +261,6 @@ def file(
     present=True, assume_present=False,
     user=None, group=None, mode=None, touch=False,
     create_remote_dir=True,
-    state=None, host=None,
 ):
     '''
     Add/remove/update files.
@@ -287,22 +279,22 @@ def file(
         user & group as passed to ``files.put``. The mode will *not* be copied over,
         if this is required call ``files.directory`` separately.
 
-    Example:
+    **Example:**
 
     .. code:: python
 
         files.file(
-            name='Create c:\\temp\\hello.txt',
-            path='c:\\temp\\hello.txt',
+            name="Create c:\\temp\\hello.txt",
+            path="c:\\temp\\hello.txt",
             touch=True,
         )
     '''
 
-    if not isinstance(path, six.string_types):
+    if not isinstance(path, str):
         raise OperationTypeError('Name must be a string')
 
     # mode = ensure_mode_int(mode)
-    info = host.get_fact(WindowsFile, name=path)
+    info = host.get_fact(File, path=path)
 
     # Not a file?!
     if info is False:
@@ -311,7 +303,7 @@ def file(
     # Doesn't exist & we want it
     if not assume_present and info is None and present:
         if create_remote_dir:
-            yield _create_remote_dir(state, host, path, user, group)
+            yield from _create_remote_dir(state, host, path, user, group)
 
         yield 'New-Item -ItemType file {0}'.format(path)
 
@@ -342,21 +334,11 @@ def file(
 #            yield chown(path, user, group)
 
 
-def windows_file(*args, **kwargs):
-    # COMPAT
-    # TODO: remove this
-    logger.warning((
-        'Use of `windows_files.windows_file` is deprecated, '
-        'please use `windows_files.file` instead.'
-    ))
-    return file(*args, **kwargs)
-
-
 def _create_remote_dir(state, host, remote_filename, user, group):
     # Always use POSIX style path as local might be Windows, remote always *nix
     remote_dirname = ntpath.dirname(remote_filename)
     if remote_dirname:
-        yield directory(
+        yield from directory(
             remote_dirname,
             state=state,
             host=host,
@@ -372,7 +354,6 @@ def directory(
     path,
     present=True, assume_present=False,
     user=None, group=None, mode=None, recursive=False,
-    state=None, host=None,
 ):
     '''
     Add/remove/update directories.
@@ -385,36 +366,36 @@ def directory(
     + TODO: mode: permissions of the folder
     + TODO: recursive: recursively apply user/group/mode
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         files.directory(
-            name='Ensure the c:\\temp\\dir_that_we_want_removed is removed',
-            path='c:\\temp\\dir_that_we_want_removed',
+            name="Ensure the c:\\temp\\dir_that_we_want_removed is removed",
+            path="c:\\temp\\dir_that_we_want_removed",
             present=False,
         )
 
         files.directory(
-            name='Ensure c:\\temp\\foo\\foo_dir exists',
-            path='c:\\temp\\foo\\foo_dir',
+            name="Ensure c:\\temp\\foo\\foo_dir exists",
+            path="c:\\temp\\foo\\foo_dir",
             recursive=True,
         )
 
         # multiple directories
-        dirs = ['c:\\temp\\foo_dir1', 'c:\\temp\\foo_dir2']
+        dirs = ["c:\\temp\\foo_dir1", "c:\\temp\\foo_dir2"]
         for dir in dirs:
             files.directory(
-                name='Ensure the directory `{}` exists'.format(dir),
+                name="Ensure the directory `{}` exists".format(dir),
                 path=dir,
             )
 
     '''
 
-    if not isinstance(path, six.string_types):
+    if not isinstance(path, str):
         raise OperationTypeError('Name must be a string')
 
-    info = host.get_fact(WindowsDirectory, name=path)
+    info = host.get_fact(Directory, path=path)
 
     # Not a directory?!
     if info is False:
@@ -430,8 +411,8 @@ def directory(
 #
         # Somewhat bare fact, should flesh out more
         host.create_fact(
-            WindowsDate,
-            kwargs={'name': path},
+            Date,
+            kwargs={'path': path},
             data={'type': 'directory'},
         )
 
@@ -458,19 +439,9 @@ def directory(
 #            yield chown(path, user, group, recursive=recursive)
 
 
-def windows_directory(*args, **kwargs):
-    # COMPAT
-    # TODO: remove this
-    logger.warning((
-        'Use of `windows_files.windows_directory` is deprecated, '
-        'please use `windows_files.directory` instead.'
-    ))
-    return directory(*args, **kwargs)
-
-
 def _validate_path(path):
     try:
-        path = fspath(path)
+        path = os.fspath(path)
     except TypeError:
         raise OperationTypeError('`path` must be a string or `os.PathLike` object')
 
@@ -483,7 +454,6 @@ def link(
     target=None, present=True, assume_present=False,
     user=None, group=None, symbolic=True, force=True,
     create_remote_dir=True,
-    state=None, host=None,
 ):
     '''
     Add/remove/update links.
@@ -506,15 +476,15 @@ def link(
         If the link exists and points to a different target, pyinfra will remove it and
         recreate a new one pointing to then new target.
 
-    Examples:
+    **Examples:**
 
     .. code:: python
 
         # simple example showing how to link to a file
         files.link(
-            name=r'Create link C:\\issue2 that points to C:\\issue',
-            path=r'C:\\issue2',
-            target=r'C\\issue',
+            name=r"Create link C:\\issue2 that points to C:\\issue",
+            path=r"C:\\issue2",
+            target=r"C\\issue",
         )
     '''
 
@@ -523,7 +493,7 @@ def link(
     if present and not target:
         raise OperationError('If present is True target must be provided')
 
-    info = host.get_fact(WindowsLink, name=path)
+    info = host.get_fact(Link, path=path)
 
     # Not a link?
     if info is not None and not info:
@@ -543,7 +513,7 @@ def link(
     # or does not exist
     if (info is None or force) and present:
         if create_remote_dir:
-            yield _create_remote_dir(state, host, path, user, group)
+            yield from _create_remote_dir(state, host, path, user, group)
 
         yield add_cmd
 
