@@ -1,16 +1,14 @@
+from collections import defaultdict
 from os import listdir, path
 from types import GeneratorType
 
-import six
-
-from pyinfra import logger, pseudo_inventory
+from pyinfra import logger
 from pyinfra.api.inventory import Inventory
+from pyinfra.context import ctx_inventory
 from pyinfra_cli.util import exec_file
 
 # Hosts in an inventory can be just the hostname or a tuple (hostname, data)
-ALLOWED_HOST_TYPES = tuple(
-    six.string_types + (tuple,),
-)
+ALLOWED_HOST_TYPES = (str, tuple)
 
 
 def _is_inventory_group(key, value):
@@ -38,9 +36,11 @@ def _is_inventory_group(key, value):
     )
 
 
-def _get_group_data(deploy_dir):
+def _get_group_data(dirname):
     group_data = {}
-    group_data_directory = path.join(deploy_dir, 'group_data')
+    group_data_directory = path.join(dirname, 'group_data')
+
+    logger.debug(f'Checking possible group_data directory: {dirname}')
 
     if path.exists(group_data_directory):
         files = listdir(group_data_directory)
@@ -60,7 +60,7 @@ def _get_group_data(deploy_dir):
 
             group_data[group_name] = {
                 key: value
-                for key, value in six.iteritems(attrs)
+                for key, value in attrs.items()
                 if key in keys and not key.startswith('_')
             }
 
@@ -72,12 +72,12 @@ def _get_groups_from_filename(inventory_filename):
 
     return {
         key: value
-        for key, value in six.iteritems(attrs)
+        for key, value in attrs.items()
         if _is_inventory_group(key, value)
     }
 
 
-def make_inventory(inventory_filename, override_data=None, deploy_dir=None):
+def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_directories=None):
     '''
     Builds a ``pyinfra.api.Inventory`` from the filesystem. If the file does not exist
     and doesn't contain a / attempts to use that as the only hostname.
@@ -128,25 +128,33 @@ def make_inventory(inventory_filename, override_data=None, deploy_dir=None):
     # mode we want to be define this in separate files (inventory / group data). The
     # issue is we want inventory access within the group data files - but at this point
     # we're not ready to make an Inventory. So here we just create a fake one, and
-    # attach it to pseudo_inventory while we import the data files.
+    # attach it to the inventory context while we import the data files.
     logger.debug('Creating fake inventory...')
 
     fake_groups = {
         # In API mode groups *must* be tuples of (hostnames, data)
         name: group if isinstance(group, tuple) else (group, {})
-        for name, group in six.iteritems(groups)
+        for name, group in groups.items()
     }
     fake_inventory = Inventory((all_hosts, all_data), **fake_groups)
-    pseudo_inventory.set(fake_inventory)
 
-    # Get all group data (group_data/*.py)
-    group_data = _get_group_data(deploy_dir)
+    possible_group_data_folders = [cwd]
+    inventory_dirname = path.abspath(path.dirname(inventory_filename))
+    if inventory_dirname != cwd:
+        possible_group_data_folders.append(inventory_dirname)
 
-    # Reset the pseudo inventory
-    pseudo_inventory.reset()
+    if group_data_directories:
+        possible_group_data_folders.extend(group_data_directories)
+
+    group_data = defaultdict(dict)
+
+    with ctx_inventory.use(fake_inventory):
+        for folder in possible_group_data_folders:
+            for group_name, data in _get_group_data(folder).items():
+                group_data[group_name].update(data)
 
     # For each group load up any data
-    for name, hosts in six.iteritems(groups):
+    for name, hosts in groups.items():
         data = {}
 
         if isinstance(hosts, tuple):
@@ -161,7 +169,7 @@ def make_inventory(inventory_filename, override_data=None, deploy_dir=None):
     # Loop back through any leftover group data and create an empty (for now)
     # group - this is because inventory @connectors can attach arbitrary groups
     # to hosts, so we need to support that.
-    for name, data in six.iteritems(group_data):
+    for name, data in group_data.items():
         groups[name] = ([], data)
 
     return Inventory(
