@@ -1,36 +1,27 @@
-from __future__ import division, print_function
-
 import logging
 import sys
 import warnings
 
 from fnmatch import fnmatch
-from os import getcwd, path
+from os import chdir as os_chdir, getcwd, path
 
 import click
 
 from pyinfra import (
     __version__,
     logger,
-    pseudo_config,
-    pseudo_inventory,
-    pseudo_state,
+    state,
 )
 from pyinfra.api import Config, State
 from pyinfra.api.connect import connect_all, disconnect_all
 from pyinfra.api.exceptions import NoGroupError, PyinfraError
-from pyinfra.api.facts import (
-    get_fact_class,
-    get_fact_names,
-    get_facts,
-    ShortFactBase,
-)
+from pyinfra.api.facts import get_facts
 from pyinfra.api.operation import add_op
 from pyinfra.api.operations import run_ops
 from pyinfra.api.util import get_kwargs_str
+from pyinfra.context import ctx_config, ctx_inventory, ctx_state
 from pyinfra.operations import server
 
-from .config import extract_file_config
 from .exceptions import (
     CliError,
     UnexpectedExternalError,
@@ -40,10 +31,8 @@ from .inventory import make_inventory
 from .log import setup_logging
 from .prints import (
     print_facts,
-    print_facts_list,
     print_inventory,
     print_meta,
-    print_operations_list,
     print_results,
     print_state_facts,
     print_state_operations,
@@ -53,47 +42,18 @@ from .util import (
     exec_file,
     get_facts_and_args,
     get_operation_and_args,
-    list_dirs_above_file,
     load_deploy_file,
     parse_cli_arg,
 )
 from .virtualenv import init_virtualenv
 
 
-# Exit handler
 def _exit():
-    if pseudo_state.isset() and pseudo_state.failed_hosts:
+    if ctx_state.isset() and state.failed_hosts:
         sys.exit(1)
     sys.exit(0)
 
 
-def _print_facts(ctx, param, value):
-    if not value:
-        return
-
-    click.echo(click.style(
-        '--facts is deprecated and will be removed in the future.',
-        'yellow',
-    ), err=True)
-
-    click.echo('--> Available facts:', err=True)
-    print_facts_list()
-    ctx.exit()
-
-
-def _print_operations(ctx, param, value):
-    if not value:
-        return
-
-    click.echo(click.style(
-        '--operations is deprecated and will be removed in the future.',
-        'yellow',
-    ), err=True)
-    click.echo('--> Available operations:', err=True)
-    print_operations_list()
-    ctx.exit()
-
-
 def _print_support(ctx, param, value):
     if not value:
         return
@@ -104,8 +64,8 @@ def _print_support(ctx, param, value):
 
 
 @click.command()
-@click.argument('inventory', nargs=1)
-@click.argument('operations', nargs=-1, required=True)
+@click.argument('inventory', nargs=1, type=click.Path(exists=False))
+@click.argument('operations', nargs=-1, required=True, type=click.Path(exists=False))
 @click.option(
     'verbosity', '-v',
     count=True,
@@ -126,12 +86,21 @@ def _print_support(ctx, param, value):
     multiple=True,
     help='Override data values, format key=value.',
 )
+@click.option(
+    '--group-data',
+    multiple=True,
+    help='Paths to load additional group data from (overrides matching keys).',
+)
 @click.option(
     '--config',
     'config_filename',
     help='Specify config file to use (default: config.py).',
     default='config.py',
 )
+@click.option(
+    '--chdir',
+    help='Set the working directory before executing.',
+)
 # Auth args
 @click.option(
     '--sudo', is_flag=True, default=False,
@@ -256,44 +225,34 @@ def cli(*args, **kwargs):
         raise UnexpectedInternalError(e)
 
     finally:
-        if pseudo_state.isset() and pseudo_state.initialised:
+        if ctx_state.isset() and state.initialised:
             # Triggers any executor disconnect requirements
-            disconnect_all(pseudo_state)
-
-
-if '--help' not in sys.argv:
-    click.option(
-        '--facts', is_flag=True, is_eager=True, callback=_print_facts,
-        help='Print available facts list and exit.',
-    )(cli)
-    click.option(
-        'print_operations', '--operations',
-        is_flag=True,
-        is_eager=True,
-        callback=_print_operations,
-        help='Print available operations list and exit.',
-    )(cli)
-    click.option(
-        '--debug-data', is_flag=True, default=False,
-        help='Print host/group data before connecting and exit.',
-    )(cli)
+            disconnect_all(state)
 
 
 def _main(
-    inventory, operations, verbosity,
+    inventory, operations, verbosity, chdir,
     ssh_user, ssh_port, ssh_key, ssh_key_password, ssh_password,
     winrm_username, winrm_password, winrm_port,
     winrm_transport, shell_executable,
     sudo, sudo_user, use_sudo_password, su_user,
-    parallel, fail_percent, data, config_filename,
+    parallel, fail_percent, data, group_data, config_filename,
     dry, limit, no_wait, serial, quiet,
-    debug, debug_data, debug_facts, debug_operations,
-    facts=None, print_operations=None, support=None,
+    debug, debug_facts, debug_operations,
+    support=None,
 ):
+    # Setup working directory
+    #
+
+    if chdir:
+        os_chdir(chdir)
+
+    # Setup logging
+    #
+
     if not debug and not sys.warnoptions:
         warnings.simplefilter('ignore')
 
-    # Setup logging
     log_level = logging.INFO
     if debug:
         log_level = logging.DEBUG
@@ -305,76 +264,8 @@ def _main(
     # Bootstrap any virtualenv
     init_virtualenv()
 
-    cwd = getcwd()
-    # Unconditionally adding cwd into sys.path
-    sys.path.append(cwd)
-
-    deploy_dir = cwd
-    potential_deploy_dirs = []
-
-    # This is the most common case: we have a deploy file so use it's
-    # pathname - we only look at the first file as we can't have multiple
-    # deploy directories.
-    if operations[0].endswith('.py') and path.isfile(operations[0]):
-        potential_deploy_dirs.extend(list_dirs_above_file(operations[0], cwd))
-
-    # If we have a valid inventory, look in it's path and it's parent for
-    # group_data or config.py to indicate deploy_dir (--fact, --run).
-    if inventory.endswith('.py') and path.isfile(inventory):
-        potential_deploy_dirs.extend([
-            dirname for dirname in list_dirs_above_file(inventory, cwd)
-            if dirname not in potential_deploy_dirs
-        ])
-
-    for potential_deploy_dir in potential_deploy_dirs:
-        logger.debug('Checking potential directory: {0}'.format(
-            potential_deploy_dir,
-        ))
-
-        if any((
-            path.isdir(path.join(potential_deploy_dir, 'group_data')),
-            path.isfile(path.join(potential_deploy_dir, config_filename)),
-        )):
-            logger.debug('Setting directory to: {0}'.format(potential_deploy_dir))
-            deploy_dir = potential_deploy_dir
-            break
-    else:
-        logger.debug('Deploy directory remains as cwd')
-
-    # Make sure imported files (deploy.py/etc) behave as if imported from the cwd
-    if deploy_dir not in sys.path:
-        sys.path.append(deploy_dir)
-
-    # Create an empty/unitialised state object
-    state = State()
-    # Set the deploy directory
-    state.deploy_dir = deploy_dir
-
-    pseudo_state.set(state)
-
-    if verbosity > 0:
-        state.print_fact_info = True
-        state.print_noop_info = True
-
-    if verbosity > 1:
-        state.print_input = state.print_fact_input = True
-
-    if verbosity > 2:
-        state.print_output = state.print_fact_output = True
-
-    if not quiet:
-        click.echo('--> Loading config...', err=True)
-
-    config = Config()
-    pseudo_config.set(config)
-
-    # Load up any config.py from the filesystem
-    config_filename = path.join(deploy_dir, config_filename)
-    if path.exists(config_filename):
-        extract_file_config(config_filename, config)  # TODO: remove this
-        exec_file(config_filename)
-
-    # TODO: lock the config here, moving up from below when possible (v2)
+    #  Check operations are valid and setup command
+    #
 
     # Make a copy before we overwrite
     original_operations = operations
@@ -383,26 +274,6 @@ def _main(
     if operations[0] == 'debug-inventory':
         command = 'debug-inventory'
 
-    # Get all non-arg facts
-    elif operations[0] == 'all-facts':
-        click.echo(click.style(
-            'all-facts is deprecated and will be removed in the future.',
-            'yellow',
-        ), err=True)
-
-        command = 'fact'
-        fact_ops = []
-
-        for fact_name in get_fact_names():
-            fact_class = get_fact_class(fact_name)
-            if (
-                not issubclass(fact_class, ShortFactBase)
-                and not callable(fact_class.command)
-            ):
-                fact_ops.append((fact_class, None, None))
-
-        operations = fact_ops
-
     # Get one or more facts
     elif operations[0] == 'fact':
         command = 'fact'
@@ -416,11 +287,26 @@ def _main(
     # Execute one or more deploy files
     elif all(cmd.endswith('.py') for cmd in operations):
         command = 'deploy'
-        operations = operations[0:]
 
-        for file in operations:
-            if not path.exists(file):
-                raise CliError('No deploy file: {0}'.format(file))
+        filenames = []
+
+        for filename in operations[0:]:
+            if path.exists(filename):
+                filenames.append(filename)
+                continue
+            if chdir and filename.startswith(chdir):
+                correct_filename = path.relpath(filename, chdir)
+                logger.warning((
+                    'Fixing deploy filename under `--chdir` argument: '
+                    f'{filename} -> {correct_filename}'
+                ))
+                filenames.append(correct_filename)
+                continue
+            raise CliError('No deploy file: {0}'.format(
+                path.join(chdir, filename) if chdir else filename,
+            ))
+
+        operations = filenames
 
     # Operation w/optional args (<module>.<op> ARG1 ARG2 ...)
     elif len(operations[0].split('.')) == 2:
@@ -436,10 +322,37 @@ def _main(
     pyinfra INVENTORY exec -- echo "hello world"
     pyinfra INVENTORY fact os [users]...'''.format(operations))
 
-    # TODO: remove this - legacy load of any config variables from the top of
-    # the first deploy file.
-    if command == 'deploy':
-        extract_file_config(operations[0], config)
+    # Setup state, config & inventory
+    #
+
+    cwd = getcwd()
+    if cwd not in sys.path:  # ensure cwd is present in sys.path
+        sys.path.append(cwd)
+
+    state = State()
+    state.cwd = cwd
+    ctx_state.set(state)
+
+    if verbosity > 0:
+        state.print_fact_info = True
+        state.print_noop_info = True
+
+    if verbosity > 1:
+        state.print_input = state.print_fact_input = True
+
+    if verbosity > 2:
+        state.print_output = state.print_fact_output = True
+
+    if not quiet:
+        click.echo('--> Loading config...', err=True)
+
+    config = Config()
+    ctx_config.set(config)
+
+    # Load up any config.py from the filesystem
+    config_filename = path.join(state.cwd, config_filename)
+    if path.exists(config_filename):
+        exec_file(config_filename)
 
     # Lock the current config, this allows us to restore this version after
     # executing deploy files that may alter them.
@@ -497,12 +410,11 @@ def _main(
     # Load up the inventory from the filesystem
     inventory, inventory_group = make_inventory(
         inventory,
-        deploy_dir=deploy_dir,
+        cwd=state.cwd,
         override_data=override_data,
+        group_data_directories=group_data,
     )
-
-    # Attach to pseudo inventory
-    pseudo_inventory.set(inventory)
+    ctx_inventory.set(inventory)
 
     # Now that we have inventory, apply --limit config override
     initial_limit = None
@@ -513,17 +425,7 @@ def _main(
             try:
                 limit_hosts = inventory.get_group(limiter)
             except NoGroupError:
-                limits = limiter.split(',')
-                if len(limits) > 1:
-                    logger.warning((
-                        'Specifying comma separated --limit values is deprecated, '
-                        'please use multiple --limit options.'
-                    ))
-
-                limit_hosts = [
-                    host for host in inventory
-                    if any(fnmatch(host.name, match) for match in limits)
-                ]
+                limit_hosts = [host for host in inventory if fnmatch(host.name, limiter)]
 
             if not limit_hosts:
                 logger.warning('No host matches found for --limit pattern: {0}'.format(limiter))
@@ -534,24 +436,18 @@ def _main(
     # Initialise the state
     state.init(inventory, config, initial_limit=initial_limit)
 
-    # If --debug-data dump & exit
-    if command == 'debug-inventory' or debug_data:
-        if debug_data:
-            logger.warning((
-                '--debug-data is deprecated, '
-                'please use `pyinfra INVENTORY debug-inventory` instead.'
-            ))
+    if command == 'debug-inventory':
         print_inventory(state)
         _exit()
 
-    # Connect to all the servers
+    # Connect to the hosts & start handling the user commands
+    #
+
     if not quiet:
         click.echo(err=True)
         click.echo('--> Connecting to hosts...', err=True)
-    connect_all(state)
 
-    # Just getting a fact?
-    #
+    connect_all(state)
 
     if command == 'fact':
         if not quiet:
@@ -586,14 +482,8 @@ def _main(
         print_facts(fact_data)
         _exit()
 
-    # Prepare the deploy!
-    #
-
-    # Execute a raw command with server.shell
     if command == 'exec':
-        # Print the output of the command
         state.print_output = True
-
         add_op(
             state,
             server.shell,
@@ -601,7 +491,6 @@ def _main(
             _allow_cli_mode=True,
         )
 
-    # Deploy files(s)
     elif command == 'deploy':
         if not quiet:
             click.echo(err=True)
@@ -610,11 +499,13 @@ def _main(
         # Number of "steps" to make = number of files * number of hosts
         for i, filename in enumerate(operations):
             logger.info('Loading: {0}'.format(click.style(filename, bold=True)))
+
+            state.current_op_file_number = i
             load_deploy_file(state, filename)
+
             # Remove any config changes introduced by the deploy file & any includes
             config.reset_locked_state()
 
-    # Operation w/optional args
     elif command == 'op':
         if not quiet:
             click.echo(err=True)
@@ -635,7 +526,9 @@ def _main(
 
         add_op(state, op, *args, **kwargs)
 
-    # Always show meta output
+    # Print proposed changes, execute unless --dry, and exit
+    #
+
     if not quiet:
         click.echo(err=True)
         click.echo('--> Proposed changes:', err=True)
@@ -651,7 +544,6 @@ def _main(
 
         _exit()
 
-    # Run the operations we generated with the deploy file
     if dry:
         _exit()
 
