@@ -1,5 +1,3 @@
-from __future__ import division, print_function
-
 import json
 import os
 
@@ -9,7 +7,7 @@ from os import path
 from types import FunctionType, ModuleType
 
 # py2/3 switcheroo
-try:  # pragma: no cover
+try:
     from StringIO import StringIO
     from cStringIO import OutputType, InputType
     from types import FileType
@@ -20,13 +18,14 @@ except ImportError:  # pragma: no cover
     io_bases = IOBase
 
 import click
-import six
+import gevent
 
-from pyinfra import logger, pseudo_host, pseudo_state
+from pyinfra import logger, state
 from pyinfra.api.command import PyinfraCommand
 from pyinfra.api.exceptions import PyinfraError
-from pyinfra.api.facts import get_fact_class, is_fact
-from pyinfra.api.util import FallbackDict
+from pyinfra.api.host import HostData
+from pyinfra.context import ctx_config, ctx_host
+from pyinfra.progress import progress_spinner
 
 from .exceptions import CliError, UnexpectedExternalError
 
@@ -41,28 +40,13 @@ def is_subdir(child, parent):
     return not relative.startswith(os.pardir)
 
 
-def list_dirs_above_file(filename, parent):
-    dirs = []
-
-    current_dir = path.dirname(filename)
-    dirs.append(current_dir)
-
-    while True:
-        current_dir, _ = path.split(current_dir)
-        if not current_dir or not is_subdir(current_dir, parent):
-            break
-
-        dirs.append(current_dir)
-    return dirs
-
-
 def exec_file(filename, return_locals=False, is_deploy_code=False):
     '''
     Execute a Python file and optionally return it's attributes as a dict.
     '''
 
-    old_current_exec_filename = pseudo_state.current_exec_filename
-    pseudo_state.current_exec_filename = filename
+    old_current_exec_filename = state.current_exec_filename
+    state.current_exec_filename = filename
 
     if filename not in PYTHON_CODES:
         with open(filename, 'r') as f:
@@ -84,13 +68,13 @@ def exec_file(filename, return_locals=False, is_deploy_code=False):
             raise
         raise UnexpectedExternalError(e, filename)
 
-    pseudo_state.current_exec_filename = old_current_exec_filename
+    state.current_exec_filename = old_current_exec_filename
     return data
 
 
 def json_encode(obj):
     # pyinfra types
-    if isinstance(obj, FallbackDict):
+    if isinstance(obj, HostData):
         return obj.dict()
 
     elif isinstance(obj, PyinfraCommand):
@@ -119,7 +103,7 @@ def json_encode(obj):
     elif isinstance(obj, set):
         return sorted(list(obj))
 
-    elif isinstance(obj, six.binary_type):
+    elif isinstance(obj, bytes):
         return obj.decode()
 
     else:
@@ -205,7 +189,7 @@ def get_facts_and_args(commands):
     for command in commands:
         if '=' in command:
             if not current_fact:
-                raise CliError('Invalid fact commands: {0}'.format(commands))
+                raise CliError('Invalid fact commands: `{0}`'.format(commands))
 
             key, value = command.split('=', 1)
             current_fact[2][key] = value
@@ -216,35 +200,22 @@ def get_facts_and_args(commands):
             current_fact = None
 
         if '.' not in command:
-            args = None
-            if ':' in command:
-                command, args = command.split(':', 1)
-                args = args.split(',')
+            raise CliError(f'Invalid fact: `{command}`, should be in the format `module.cls`')
 
-            if not is_fact(command):
-                raise CliError('No fact: {0}'.format(command))
-
-            fact_cls = get_fact_class(command)
-            logger.warning((
-                'Named facts are deprecated, please use the explicit import: {0}.{1}'
-            ).format(fact_cls.__module__.replace('pyinfra.facts.', ''), fact_cls.__name__))
-            current_fact = (fact_cls, args, {})
-
-        else:
-            fact_module, fact_name = command.rsplit('.', 1)
+        fact_module, fact_name = command.rsplit('.', 1)
+        try:
+            fact_module = import_module('pyinfra.facts.{0}'.format(fact_module))
+        except ImportError:
             try:
-                fact_module = import_module('pyinfra.facts.{0}'.format(fact_module))
+                fact_module = import_module(str(fact_module))
             except ImportError:
-                try:
-                    fact_module = import_module(str(fact_module))
-                except ImportError:
-                    raise CliError('No such module: {0}'.format(fact_module))
+                raise CliError('No such module: `{0}`'.format(fact_module))
 
-            fact_cls = getattr(fact_module, fact_name, None)
-            if not fact_cls:
-                raise CliError('No such fact: {0}'.format(command))
+        fact_cls = getattr(fact_module, fact_name, None)
+        if not fact_cls:
+            raise CliError('No such fact: `{0}`'.format(command))
 
-            current_fact = (fact_cls, (), {})
+        current_fact = (fact_cls, (), {})
 
     if current_fact:
         facts.append(current_fact)
@@ -255,19 +226,23 @@ def get_facts_and_args(commands):
 def load_deploy_file(state, filename):
     state.current_deploy_filename = filename
 
-    # Copy the inventory hosts (some might be removed during deploy)
-    hosts = list(state.inventory.iter_active_hosts())
+    def load_file(local_host):
+        with ctx_config.use(state.config.copy()):
+            with ctx_host.use(local_host):
+                exec_file(filename)
+                logger.info('{0}{1} {2}'.format(
+                    local_host.print_prefix,
+                    click.style('Ready:', 'green'),
+                    click.style(filename, bold=True),
+                ))
 
-    for host in hosts:
-        pseudo_host.set(host)
+    greenlet_to_host = {
+        state.pool.spawn(load_file, host): host
+        for host in state.inventory.iter_active_hosts()
+    }
 
-        exec_file(filename)
-
-        logger.info('{0}{1} {2}'.format(
-            host.print_prefix,
-            click.style('Ready:', 'green'),
-            click.style(filename, bold=True),
-        ))
-
-    # Remove any pseudo host
-    pseudo_host.reset()
+    with progress_spinner(greenlet_to_host.values()) as progress:
+        for greenlet in gevent.iwait(greenlet_to_host.keys()):
+            host = greenlet_to_host[greenlet]
+            greenlet.get()
+            progress(host)
