@@ -4,7 +4,6 @@ from pyinfra.api.util import (
     format_exception,
     get_caller_frameinfo,
     try_int,
-    underscore,
 )
 
 
@@ -20,15 +19,7 @@ class TestApiUtil(TestCase):
             return get_caller_frameinfo()
 
         frameinfo = _get_caller_frameinfo()
-        assert frameinfo.lineno == 22  # called by the line above
-
-    def test_underscore(self):
-        for camel_string, snake_string in (
-            ('ThisIsHowYouDoIt', 'this_is_how_you_do_it'),
-            ('This What_nope', 'this _what_nope'),
-            ('myNameIs', 'my_name_is'),
-        ):
-            assert underscore(camel_string) == snake_string
+        assert frameinfo.lineno == 21  # called by the line above
 
     def test_format_exception(self):
         exception = Exception('I am a message', 1)
