@@ -1,15 +1,14 @@
 # encoding: utf-8
 
+import shlex
 from subprocess import PIPE
 from unittest import TestCase
-
-from mock import MagicMock, mock_open, patch
-from six.moves import shlex_quote
+from unittest.mock import MagicMock, mock_open, patch
 
 from pyinfra.api import Config, State
 from pyinfra.api.connect import connect_all
-from pyinfra.api.connectors.util import make_unix_command
 from pyinfra.api.exceptions import PyinfraError
+from pyinfra.connectors.util import make_unix_command
 
 from ..util import make_inventory
 
@@ -21,14 +20,14 @@ def fake_chroot_shell(command, splitlines=None):
     raise PyinfraError('Invalid command: {0}'.format(command))
 
 
-@patch('pyinfra.api.connectors.chroot.local.shell', fake_chroot_shell)
-@patch('pyinfra.api.connectors.chroot.mkstemp', lambda: (None, '__tempfile__'))
-@patch('pyinfra.api.connectors.chroot.os.remove', lambda f: None)
-@patch('pyinfra.api.connectors.chroot.open', mock_open(read_data='test!'), create=True)
+@patch('pyinfra.connectors.chroot.local.shell', fake_chroot_shell)
+@patch('pyinfra.connectors.chroot.mkstemp', lambda: (None, '__tempfile__'))
+@patch('pyinfra.connectors.chroot.os.remove', lambda f: None)
+@patch('pyinfra.connectors.chroot.open', mock_open(read_data='test!'), create=True)
 @patch('pyinfra.api.util.open', mock_open(read_data='test!'), create=True)
 class TestChrootConnector(TestCase):
     def setUp(self):
-        self.fake_popen_patch = patch('pyinfra.api.connectors.util.Popen')
+        self.fake_popen_patch = patch('pyinfra.connectors.util.Popen')
         self.fake_popen_mock = self.fake_popen_patch.start()
 
     def tearDown(self):
@@ -56,8 +55,7 @@ class TestChrootConnector(TestCase):
 
     def test_run_shell_command(self):
         inventory = make_inventory(hosts=('@chroot/not-a-chroot',))
-        state = State(inventory, Config())
-        connect_all(state)
+        State(inventory, Config())
         host = inventory.get_host('@chroot/not-a-chroot')
         host.connect()
 
@@ -73,7 +71,7 @@ class TestChrootConnector(TestCase):
         assert out[0] is True
 
         command = make_unix_command(command).get_raw_value()
-        command = shlex_quote(command)
+        command = shlex.quote(command)
         docker_command = 'chroot /not-a-chroot sh -c {0}'.format(command)
         shell_command = make_unix_command(docker_command).get_raw_value()
 
@@ -86,6 +84,7 @@ class TestChrootConnector(TestCase):
         inventory = make_inventory(hosts=('@chroot/not-a-chroot',))
         state = State(inventory, Config())
         connect_all(state)
+
         host = inventory.get_host('@chroot/not-a-chroot')
 
         command = 'echo hoi'
@@ -99,6 +98,7 @@ class TestChrootConnector(TestCase):
         inventory = make_inventory(hosts=('@chroot/not-a-chroot',))
         state = State(inventory, Config())
         connect_all(state)
+
         host = inventory.get_host('@chroot/not-a-chroot')
 
         command = 'echo hoi'
