@@ -1,14 +1,12 @@
 # encoding: utf-8
 
-from __future__ import unicode_literals
-
 from socket import (
     error as socket_error,
     gaierror,
 )
 from unittest import TestCase
+from unittest.mock import call, MagicMock, mock_open, patch
 
-from mock import call, MagicMock, mock_open, patch
 from paramiko import (
     AuthenticationException,
     PasswordRequiredException,
@@ -19,8 +17,8 @@ import pyinfra
 
 from pyinfra.api import Config, MaskString, State, StringCommand
 from pyinfra.api.connect import connect_all
-from pyinfra.api.connectors.ssh import _get_sftp_connection
 from pyinfra.api.exceptions import ConnectError, PyinfraError
+from pyinfra.connectors.ssh import _get_sftp_connection
 
 from ..util import make_inventory
 
@@ -31,11 +29,11 @@ def make_raise_exception_function(cls, *args, **kwargs):
     return handler
 
 
-@patch('pyinfra.api.connectors.ssh.SSHClient.get_transport', MagicMock())
-@patch('pyinfra.api.connectors.ssh.open', mock_open(read_data='test!'), create=True)
+@patch('pyinfra.connectors.ssh.SSHClient.get_transport', MagicMock())
+@patch('pyinfra.connectors.ssh.open', mock_open(read_data='test!'), create=True)
 class TestSSHConnector(TestCase):
     def setUp(self):
-        self.fake_connect_patch = patch('pyinfra.api.connectors.ssh.SSHClient.connect')
+        self.fake_connect_patch = patch('pyinfra.connectors.ssh.SSHClient.connect')
         self.fake_connect_mock = self.fake_connect_patch.start()
         _get_sftp_connection.cache = {}
 
@@ -56,7 +54,7 @@ class TestSSHConnector(TestCase):
         assert len(state.active_hosts) == 0
 
     def test_connect_all_password(self):
-        inventory = make_inventory(ssh_password='test')
+        inventory = make_inventory(override_data={'ssh_password': 'test'})
 
         # Get a host
         somehost = inventory.get_host('somehost')
@@ -67,8 +65,8 @@ class TestSSHConnector(TestCase):
 
         assert len(state.active_hosts) == 2
 
-    @patch('pyinfra.api.connectors.ssh.path.isfile', lambda *args, **kwargs: True)
-    @patch('pyinfra.api.connectors.ssh.RSAKey.from_private_key_file')
+    @patch('pyinfra.connectors.ssh.path.isfile', lambda *args, **kwargs: True)
+    @patch('pyinfra.connectors.ssh.RSAKey.from_private_key_file')
     def test_connect_exceptions(self, fake_key_open):
         for exception_class in (
             AuthenticationException,
@@ -96,14 +94,12 @@ class TestSSHConnector(TestCase):
             ('somehost', {'ssh_key': 'testkey'}),
         )), Config())
 
-        with patch('pyinfra.api.connectors.ssh.path.isfile', lambda *args, **kwargs: True), \
-                patch('pyinfra.api.connectors.ssh.RSAKey.from_private_key_file') as fake_key_open:
+        with patch('pyinfra.connectors.ssh.path.isfile', lambda *args, **kwargs: True), \
+                patch('pyinfra.connectors.ssh.RSAKey.from_private_key_file') as fake_key_open:
 
             fake_key = MagicMock()
             fake_key_open.return_value = fake_key
 
-            state.deploy_dir = '/'
-
             connect_all(state)
 
             # Check the key was created properly
@@ -119,8 +115,10 @@ class TestSSHConnector(TestCase):
                 pkey=fake_key,
                 timeout=10,
                 username='vagrant',
-                _pyinfra_force_forward_agent=None,
+                _pyinfra_ssh_forward_agent=None,
                 _pyinfra_ssh_config_file=None,
+                _pyinfra_ssh_known_hosts_file=None,
+                _pyinfra_ssh_strict_host_key_checking=None,
             )
 
         # Check that loading the same key again is cached in the state
@@ -137,10 +135,10 @@ class TestSSHConnector(TestCase):
         )), Config())
 
         with patch(
-            'pyinfra.api.connectors.ssh.path.isfile',
+            'pyinfra.connectors.ssh.path.isfile',
             lambda *args, **kwargs: True,
         ), patch(
-            'pyinfra.api.connectors.ssh.RSAKey.from_private_key_file',
+            'pyinfra.connectors.ssh.RSAKey.from_private_key_file',
         ) as fake_key_open:
             fake_key = MagicMock()
 
@@ -151,8 +149,6 @@ class TestSSHConnector(TestCase):
 
             fake_key_open.side_effect = fake_key_open_fail
 
-            state.deploy_dir = '/'
-
             connect_all(state)
 
             # Check the key was created properly
@@ -166,13 +162,13 @@ class TestSSHConnector(TestCase):
         )), Config())
 
         with patch(
-            'pyinfra.api.connectors.ssh.path.isfile',
+            'pyinfra.connectors.ssh.path.isfile',
             lambda *args, **kwargs: True,
         ), patch(
-            'pyinfra.api.connectors.ssh.getpass',
+            'pyinfra.connectors.ssh.getpass',
             lambda *args, **kwargs: 'testpass',
         ), patch(
-            'pyinfra.api.connectors.ssh.RSAKey.from_private_key_file',
+            'pyinfra.connectors.ssh.RSAKey.from_private_key_file',
         ) as fake_key_open:
             fake_key = MagicMock()
 
@@ -183,8 +179,6 @@ class TestSSHConnector(TestCase):
 
             fake_key_open.side_effect = fake_key_open_fail
 
-            state.deploy_dir = '/'
-
             pyinfra.is_cli = True
             connect_all(state)
             pyinfra.is_cli = False
@@ -200,10 +194,10 @@ class TestSSHConnector(TestCase):
         )), Config())
 
         with patch(
-            'pyinfra.api.connectors.ssh.path.isfile',
+            'pyinfra.connectors.ssh.path.isfile',
             lambda *args, **kwargs: True,
         ), patch(
-            'pyinfra.api.connectors.ssh.RSAKey.from_private_key_file',
+            'pyinfra.connectors.ssh.RSAKey.from_private_key_file',
         ) as fake_key_open:
 
             fake_key_open.side_effect = make_raise_exception_function(PasswordRequiredException)
@@ -211,8 +205,6 @@ class TestSSHConnector(TestCase):
             fake_key = MagicMock()
             fake_key_open.return_value = fake_key
 
-            state.deploy_dir = '/'
-
             with self.assertRaises(PyinfraError) as e:
                 connect_all(state)
 
@@ -230,19 +222,19 @@ class TestSSHConnector(TestCase):
         fake_fail_from_private_key_file.side_effect = make_raise_exception_function(SSHException)
 
         with patch(
-            'pyinfra.api.connectors.ssh.path.isfile',
+            'pyinfra.connectors.ssh.path.isfile',
             lambda *args, **kwargs: True,
         ), patch(
-            'pyinfra.api.connectors.ssh.DSSKey.from_private_key_file',
+            'pyinfra.connectors.ssh.DSSKey.from_private_key_file',
             fake_fail_from_private_key_file,
         ), patch(
-            'pyinfra.api.connectors.ssh.ECDSAKey.from_private_key_file',
+            'pyinfra.connectors.ssh.ECDSAKey.from_private_key_file',
             fake_fail_from_private_key_file,
         ), patch(
-            'pyinfra.api.connectors.ssh.Ed25519Key.from_private_key_file',
+            'pyinfra.connectors.ssh.Ed25519Key.from_private_key_file',
             fake_fail_from_private_key_file,
         ), patch(
-            'pyinfra.api.connectors.ssh.RSAKey.from_private_key_file',
+            'pyinfra.connectors.ssh.RSAKey.from_private_key_file',
         ) as fake_key_open:
 
             def fake_key_open_fail(*args, **kwargs):
@@ -267,17 +259,15 @@ class TestSSHConnector(TestCase):
             ('somehost', {'ssh_key': 'testkey'}),
         )), Config())
 
-        with patch('pyinfra.api.connectors.ssh.path.isfile', lambda *args, **kwargs: True), \
-                patch('pyinfra.api.connectors.ssh.RSAKey.from_private_key_file') as fake_rsa_key_open, \
-                patch('pyinfra.api.connectors.ssh.DSSKey.from_private_key_file') as fake_key_open:  # noqa
+        with patch('pyinfra.connectors.ssh.path.isfile', lambda *args, **kwargs: True), \
+                patch('pyinfra.connectors.ssh.RSAKey.from_private_key_file') as fake_rsa_key_open, \
+                patch('pyinfra.connectors.ssh.DSSKey.from_private_key_file') as fake_key_open:  # noqa
 
             fake_rsa_key_open.side_effect = make_raise_exception_function(SSHException)
 
             fake_key = MagicMock()
             fake_key_open.return_value = fake_key
 
-            state.deploy_dir = '/'
-
             connect_all(state)
 
             # Check the key was created properly
@@ -291,8 +281,10 @@ class TestSSHConnector(TestCase):
                 pkey=fake_key,
                 timeout=10,
                 username='vagrant',
-                _pyinfra_force_forward_agent=None,
+                _pyinfra_ssh_forward_agent=None,
                 _pyinfra_ssh_config_file=None,
+                _pyinfra_ssh_known_hosts_file=None,
+                _pyinfra_ssh_strict_host_key_checking=None,
             )
 
         # Check that loading the same key again is cached in the state
@@ -308,9 +300,9 @@ class TestSSHConnector(TestCase):
             ('somehost', {'ssh_key': 'testkey', 'ssh_key_password': 'testpass'}),
         )), Config())
 
-        with patch('pyinfra.api.connectors.ssh.path.isfile', lambda *args, **kwargs: True), \
-                patch('pyinfra.api.connectors.ssh.RSAKey.from_private_key_file') as fake_rsa_key_open, \
-                patch('pyinfra.api.connectors.ssh.DSSKey.from_private_key_file') as fake_dss_key_open:  # noqa
+        with patch('pyinfra.connectors.ssh.path.isfile', lambda *args, **kwargs: True), \
+                patch('pyinfra.connectors.ssh.RSAKey.from_private_key_file') as fake_rsa_key_open, \
+                patch('pyinfra.connectors.ssh.DSSKey.from_private_key_file') as fake_dss_key_open:  # noqa
 
             def fake_rsa_key_open_fail(*args, **kwargs):
                 if 'password' not in kwargs:
@@ -328,8 +320,6 @@ class TestSSHConnector(TestCase):
 
             fake_dss_key_open.side_effect = fake_dss_key_func
 
-            state.deploy_dir = '/'
-
             connect_all(state)
 
             # Check the key was created properly
@@ -343,8 +333,10 @@ class TestSSHConnector(TestCase):
                 pkey=fake_dss_key,
                 timeout=10,
                 username='vagrant',
-                _pyinfra_force_forward_agent=None,
+                _pyinfra_ssh_forward_agent=None,
                 _pyinfra_ssh_config_file=None,
+                _pyinfra_ssh_known_hosts_file=None,
+                _pyinfra_ssh_strict_host_key_checking=None,
             )
 
         # Check that loading the same key again is cached in the state
@@ -368,7 +360,7 @@ class TestSSHConnector(TestCase):
     # SSH command tests
     #
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
     def test_run_shell_command(self, fake_ssh_client):
         fake_ssh = MagicMock()
         fake_stdin = MagicMock()
@@ -400,8 +392,8 @@ class TestSSHConnector(TestCase):
 
         fake_ssh.exec_command.assert_called_with("sh -c 'echo Šablony'", get_pty=False)
 
-    @patch('pyinfra.api.connectors.ssh.click')
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.click')
+    @patch('pyinfra.connectors.ssh.SSHClient')
     def test_run_shell_command_masked(self, fake_ssh_client, fake_click):
         fake_ssh = MagicMock()
         fake_stdout = MagicMock()
@@ -433,7 +425,7 @@ class TestSSHConnector(TestCase):
             err=True,
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
     def test_run_shell_command_success_exit_code(self, fake_ssh_client):
         fake_ssh = MagicMock()
         fake_stdout = MagicMock()
@@ -453,7 +445,7 @@ class TestSSHConnector(TestCase):
         assert len(out) == 3
         assert out[0] is True
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
     def test_run_shell_command_error(self, fake_ssh_client):
         fake_ssh = MagicMock()
         fake_stdout = MagicMock()
@@ -473,10 +465,10 @@ class TestSSHConnector(TestCase):
         assert len(out) == 3
         assert out[0] is False
 
-    @patch('pyinfra.api.connectors.util.getpass')
-    @patch('pyinfra.api.connectors.util.get_sudo_askpass_exe')
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.util.getpass')
+    @patch('pyinfra.connectors.util.get_sudo_askpass_exe')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_run_shell_command_sudo_password_prompt(
         self,
         fake_sftp_client,
@@ -514,10 +506,10 @@ class TestSSHConnector(TestCase):
             "sudo -H -A -k sh -c 'echo Šablony'"
         ), get_pty=False)
 
-    @patch('pyinfra.api.connectors.util.getpass')
-    @patch('pyinfra.api.connectors.util.get_sudo_askpass_exe')
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.util.getpass')
+    @patch('pyinfra.connectors.util.get_sudo_askpass_exe')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_run_shell_command_sudo_password_automatic_prompt(
         self,
         fake_sftp_client,
@@ -568,8 +560,11 @@ class TestSSHConnector(TestCase):
             "sudo -H -A -k sh -c 'echo Šablony'"
         ), get_pty=False)
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.util._get_sudo_password')
+    # SSH file put/get tests
+    #
+
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.util._get_sudo_password')
     def test_run_shell_command_retry_for_sudo_password(
         self,
         fake_get_sudo_password,
@@ -606,8 +601,8 @@ class TestSSHConnector(TestCase):
     # SSH file put/get tests
     #
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_put_file(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('anotherhost',))
         State(inventory, Config())
@@ -626,8 +621,8 @@ class TestSSHConnector(TestCase):
             fake_open(), 'not-another-file',
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_put_file_sudo(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('anotherhost',))
         State(inventory, Config())
@@ -658,8 +653,8 @@ class TestSSHConnector(TestCase):
             fake_open(), '/tmp/pyinfra-de01e82cb691e8a31369da3c7c8f17341c44ac24',
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_put_file_su_user_fail_acl(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('anotherhost',))
         State(inventory, Config())
@@ -689,8 +684,8 @@ class TestSSHConnector(TestCase):
             fake_open(), '/tmp/pyinfra-43db9984686317089fefcf2e38de527e4cb44487',
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_put_file_su_user_fail_copy(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('anotherhost',))
         State(inventory, Config())
@@ -727,8 +722,8 @@ class TestSSHConnector(TestCase):
             fake_open(), '/tmp/pyinfra-43db9984686317089fefcf2e38de527e4cb44487',
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_put_file_sudo_custom_temp_file(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('anotherhost',))
         State(inventory, Config())
@@ -760,8 +755,8 @@ class TestSSHConnector(TestCase):
             fake_open(), '/a-different-tempfile',
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_get_file(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('somehost',))
         State(inventory, Config())
@@ -780,8 +775,8 @@ class TestSSHConnector(TestCase):
             'not-a-file', fake_open(),
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_get_file_sudo(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('somehost',))
         State(inventory, Config())
@@ -818,7 +813,7 @@ class TestSSHConnector(TestCase):
             '/tmp/pyinfra-e9c0d3c8ffca943daa0e75511b0a09c84b59c508', fake_open(),
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
     def test_get_file_sudo_copy_fail(self, fake_ssh_client):
         inventory = make_inventory(hosts=('somehost',))
         State(inventory, Config())
@@ -845,8 +840,8 @@ class TestSSHConnector(TestCase):
             ), get_pty=False),
         ])
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_get_file_sudo_remove_fail(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('somehost',))
         State(inventory, Config())
@@ -883,8 +878,8 @@ class TestSSHConnector(TestCase):
             '/tmp/pyinfra-e9c0d3c8ffca943daa0e75511b0a09c84b59c508', fake_open(),
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_get_file_su_user(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('somehost',))
         State(inventory, Config())
@@ -921,8 +916,8 @@ class TestSSHConnector(TestCase):
             '/tmp/pyinfra-e9c0d3c8ffca943daa0e75511b0a09c84b59c508', fake_open(),
         )
 
-    @patch('pyinfra.api.connectors.ssh.SSHClient')
-    @patch('pyinfra.api.connectors.ssh.SFTPClient')
+    @patch('pyinfra.connectors.ssh.SSHClient')
+    @patch('pyinfra.connectors.ssh.SFTPClient')
     def test_get_sftp_fail(self, fake_sftp_client, fake_ssh_client):
         inventory = make_inventory(hosts=('anotherhost',))
         State(inventory, Config())
