@@ -1,6 +1,5 @@
 from unittest import TestCase
-
-from mock import MagicMock, patch
+from unittest.mock import MagicMock, patch
 
 from pyinfra.api import Config, State
 from pyinfra.api.connect import connect_all
@@ -10,7 +9,7 @@ from ..util import make_inventory
 
 class TestWinrmConnector(TestCase):
     def setUp(self):
-        self.fake_connect_patch = patch('pyinfra.api.connectors.winrm.connect')
+        self.fake_connect_patch = patch('pyinfra.connectors.winrm.connect')
         self.fake_connect_mock = self.fake_connect_patch.start()
 
     def tearDown(self):
