@@ -4,9 +4,7 @@ import os
 from datetime import datetime
 from io import open
 from os import listdir, path
-
-import six
-from mock import patch
+from unittest.mock import patch
 
 from pyinfra.api import Config, Inventory
 from pyinfra.api.util import get_kwargs_str
@@ -24,24 +22,29 @@ def get_command_string(command):
 
 
 def make_inventory(hosts=('somehost', 'anotherhost'), **kwargs):
+    override_data = kwargs.pop('override_data', {})
+    if 'ssh_user' not in override_data:
+        override_data['ssh_user'] = 'vagrant'
+
     return Inventory(
         (hosts, {}),
+        override_data=override_data,
         test_group=([
             'somehost',
         ], {
             'group_data': 'hello world',
         }),
-        ssh_user='vagrant',
         **kwargs
     )
 
 
 class FakeState(object):
     active = True
-    deploy_dir = '/'
+    cwd = '/'
     in_op = True
     in_deploy = True
     pipelining = False
+    is_executing = False
     deploy_name = None
     deploy_kwargs = None
 
@@ -59,12 +62,12 @@ def parse_fact(fact):
     '''
 
     # Handle datetimes
-    if isinstance(fact, six.string_types) and fact.startswith('datetime:'):
+    if isinstance(fact, str) and fact.startswith('datetime:'):
         return datetime.strptime(fact[9:], '%Y-%m-%dT%H:%M:%S')
 
     elif isinstance(fact, list):
         # Handle sets
-        if len(fact) > 1 and fact[0] == '_set':
+        if fact and fact[0] == 'set:':
             return set(parse_fact(value) for value in fact[1:])
 
         return [parse_fact(value) for value in fact]
@@ -72,7 +75,7 @@ def parse_fact(fact):
     elif isinstance(fact, dict):
         return {
             key: parse_fact(value)
-            for key, value in six.iteritems(fact)
+            for key, value in fact.items()
         }
 
     return fact
@@ -159,6 +162,18 @@ def _sort_kwargs_str(string):
 class FakeHost(object):
     noop_description = None
 
+    # Current context inside an @operation function
+    in_op = True
+    current_op_hash = None
+    current_op_global_kwargs = None
+
+    # Current context inside a @deploy function
+    in_deploy = True
+    current_deploy_name = None
+    current_deploy_kwargs = None
+    current_deploy_data = None
+    current_deploy_op_order = None
+
     def __init__(self, name, facts, data):
         self.name = name
         self.fact = FakeFacts(facts)
@@ -192,15 +207,19 @@ class FakeHost(object):
             return fact_ordered_keys.get(kwargs_str)
         return fact
 
-    def create_fact(self, fact_cls, data, kwargs):
+    def create_fact(self, fact_cls, data=None, kwargs=None):
         try:
             fact = self.get_fact(fact_cls)
         except KeyError:
             fact_key = self._get_fact_key(fact_cls)
             fact = self.fact[fact_key] = {}
-        fact[_sort_kwargs_str(get_kwargs_str(kwargs))] = data
+        if kwargs:
+            fact[_sort_kwargs_str(get_kwargs_str(kwargs))] = data
+        else:
+            fact_key = self._get_fact_key(fact_cls)
+            self.fact[fact_key] = data
 
-    def delete_fact(self, fact_cls, kwargs):
+    def delete_fact(self, fact_cls, kwargs=None):
         try:
             fact = self.get_fact(fact_cls)
         except KeyError:
@@ -266,7 +285,7 @@ class patch_files(object):
         self._directories = directories
 
     @staticmethod
-    def _parse_local_files(local_files, prefix=FakeState.deploy_dir):
+    def _parse_local_files(local_files, prefix=FakeState.cwd):
         files = []
         files_data = {}
         directories = {}
@@ -371,7 +390,7 @@ def create_host(name=None, facts=None, data=None):
     facts = facts or {}
     data = data or {}
 
-    for name, fact_data in six.iteritems(facts):
+    for name, fact_data in facts.items():
         real_facts[name] = fact_data
 
     return FakeHost(name, facts=real_facts, data=data)
